#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define TINY 1.0e-10;

void nrerror(char error_text[])
/* Numerical Recipes standard error handler */
{
	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}


float **matrix(int nrl,int nrh,int ncl,int nch)
/* allocate a float matrix with subscript range m[nrl..nrh][ncl..nch] */
{
	int i;
	float **m;

	/* allocate pointers to rows */
	m=(float **) malloc((unsigned) (nrh-nrl+1)*sizeof(float*))-nrl;
	if (!m) nrerror("allocation failure 1 in matrix()");

	/* allocate rows and set pointers to them */
	for(i=nrl;i<=nrh;i++) {
		m[i]=(float *) malloc((unsigned) (nch-ncl+1)*sizeof(float))-ncl;
		if (!m[i]) nrerror("allocation failure 2 in matrix()");
	}
	/* return pointer to array of pointers to rows */
	return m;
}

void free_matrix(float **m,int nrl,int nrh,int ncl,int nch)
/* free a float matrix allocated by matrix() */
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
	nch=nch;
}

int **imatrix(int nrl,int nrh,int ncl,int nch)

/* allocate an int matrix with subscript range m[nrl..nrh][ncl..nch] */
{
	int **m;
	long i;
	/* allocate pointers to rows */
	m=(int **)malloc((unsigned) (nrh-nrl+1)*sizeof(int*))-nrl;
	if (!m) nrerror("allocation failure 1 in imatrix()");

	/* allocate rows and set pointers to them */
	for(i=nrl;i<=nrh;i++) {
		m[i]=(int *)malloc((unsigned) (nch-ncl+1)*sizeof(int))-ncl;
		if (!m[i]) nrerror("allocation failure 2 in imatrix()");
	}
	/* return pointer to array of pointers to rows */
	return m;
}

void free_imatrix(int **m,int nrl,int nrh,int ncl,int nch)

/* free an int matrix allocated by imatrix() */
{
	long i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
	nch=nch;
}

float *vector(int nl,int nh)
/* allocate a float vector with subscript range v[nl..nh] */
{
	float *v;

	v=(float *)malloc((unsigned) (nh-nl+1)*sizeof(float))-nl;
	if (!v) nrerror("allocation failure in vector()");
	return v;
}


int *ivector(int nl,int nh)
/* allocate an int vector with subscript range v[nl..nh] */
{
	int *v;

	v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int))-nl;
	if (!v) nrerror("allocation failure in ivector()");
	return v;
}


void free_vector(float *v,int nl,int nh)
/* free a float vector allocated with vector() */
{
	free((char*) (v+nl));
	nh=nh;
}


void free_ivector(int *v,int nl,int nh)
/* free an int vector allocated with ivector() */
{
	free((char*) (v+nl));
	nh=nh;
}


void matrix_transpose(a,b,m,n)
float **a,**b;
int m,n;
{
        int i,j;
        for(i=1; i<=m; i++){
                for(j=1; j<=n; j++) b[j][i] = a[i][j] ;
        }
}

void matrix_by_matrix(a,b,c,l,m,n)
float **a,**b,**c;
int l,m,n;
{
        int i,j,k;
        for(i=1; i<=l; i++){
                for(j=1; j<=n; j++){
                        c[i][j] = 0.;
                        for(k=1; k<=m; k++) c[i][j] += a[i][k] * b[k][j] ;
                }
        }
}
