//-----------------------------------------------
// FILE:		MLPAgent.java
// PACKAGE:		yaes.application.eel6938.classagents
// CLASSES:		
// AUTHOR:		T. Ryan Fitz-Gibbon
// DATE:		Apr 11, 2005
//-----------------------------------------------
package yaes.application.eel6938.classagents;

import java.awt.Point;

import yaes.application.eel6938.classagents.mlp.MultiLayerPerceptron;
import yaes.application.eel6938.game.Constants;
import yaes.application.eel6938.game.objects.Agent;

//-----------------------------------------------
// CLASS:	
// PUBLIC METHODS:
//-----------------------------------------------
public class MLPAgent extends Agent {

    private static boolean DEBUG = false;
    
    private static int MOVE = 0;
    private static int ATTACK = 1;
    private static int EAT = 2;
    private static int FLEE = 3;
    private static int MATE = 4;
    private static int NORTH = 5;
    private static int SOUTH = 6;
    private static int EAST = 7;
    private static int WEST = 8;
    private static int SPEED_0 = 9;
    private static int SPEED_1 = 10;
    private static int SPEED_2 = 11;
    private static int SPEED_3 = 12;
    private static int SPEED_4 = 13;
    
    private MultiLayerPerceptron mlp;
    
    /**
     * 
     */
    public MLPAgent() {
        super();
        
        // CREATE EMPTY MLP
        mlp = new MultiLayerPerceptron(11, new int [] {20}, 14, 0.8, 0.3);
        
        // RESTORE MLP FROM FILE
        mlp.restoreNN("saved_nn.obj");
        
        // TODO Auto-generated constructor stub
    }

    /* (non-Javadoc)
     * @see yaes.application.eel6938.game.objects.Agent#intelligence()
     */
    public void intelligence() {
        // TODO Auto-generated method stub
        
        // GET INPUT FOR MLP
        if(DEBUG)
            System.out.println("get input");
        
        double [] input = new double [11];
        if(this.canAttack())
        {
            input[0] = 1;
        }
        else
        {
            input[0] = 0;
        }
        
        if(this.canEat())
        {
            input[1] = 1;
        }
        else
        {
            input[1] = 0;
        }
        
        if(this.canFlee())
        {
            input[2] = 1;
        }
        else
        {
            input[2] = 0;
        }
        
        if(this.canMate())
        {
            input[3] = 1;
        }
        else
        {
            input[3] = 0;
        }
        
        input[4] = ((double)this.getEnergyLevel()) / 15000.0;
        if((this.getSensorAgentList().length - 1) > 0)	// REMEMBER THIS '- 1' WHEN IT COMES TIME TO SEND INPUT TO THE NN TRAINED WITH THIS DATA!!!)
            input[5] = 1;
        else
            input[5] = 0;
        
        if(this.getSensorFoodList().length > 0)
            input[6] = 1;
        else
            input[6] = 0;
        
        if(this.getSensorObstacleList().length > 0)
            input[7] = 1;
        else
            input[7] = 0;
         
        // SAVE PATH TO CLOSEST AGENT
		if(getSensorAgentList().length > 1)	//Agents in sight
		{
			//Get second agent element, which is also the closest agent element by default (the first element is this agent)
			Point focusPoint = getSensorAgentList()[1]; 
			Point attackingPosition = getClosestOpenNeighbouringPoint(focusPoint);
			if(attackingPosition != null)
			{
				//Get directions to this element (if possible)
				int[] path = getDirections(attackingPosition);
				if(path != null && path.length > 0)
				{
				    input[8] = ((double)(path[0] + 1)) / 5.0; 	// Add 1 so that 0 means there is no path
				}
				else
				{
				    input[8] = 0;
				}
			}
			else
			{
			    input[8] = 0;
			}
		}
		
		// SAVE PATH TO CLOSEST FOOD
		if(this.getSensorFoodList().length > 0)	//Food in sight
		{
			//Get first food element, which is also the closest food element by default
			Point focusPoint = getSensorFoodList()[0]; 
			Point eatingPosition = getClosestOpenNeighbouringPoint(focusPoint);
			if(eatingPosition != null)
			{
				//Get directions to this element (if possible)
				int[] path = getDirections(eatingPosition);
				if(path != null && path.length > 0)
				{	
				    input[9] = ((double)(path[0] + 1)) / 5.0;	// Add 1 so that 0 means there is no path
				}
				else
				{
				    input[9] = 0;
				}
			}
			else
			{
			    input[9] = 0;
			}
		}
		
		// SAVE PATH AWAY FROM CLOSEST OBSTACLE
		if(this.getSensorObstacleList().length > 0)	//Obstacle
		{
			//Get first food element, which is also the closest food element by default
			Point focusPoint = getSensorObstacleList()[0]; 
			Point bad_position = getClosestOpenNeighbouringPoint(focusPoint);
			if(bad_position != null)
			{
				//Get directions away from this element (if possible)
				int[] path = getDirections(bad_position);
				if(path != null && path.length > 0)
				{	
				    // REVERSE THE PATH
				    if(path[0] == Constants.DirectionNorth)
				    {
				        path[0] = Constants.DirectionSouth;
				    }
				    else if(path[0] == Constants.DirectionSouth)
				    {
				        path[0] = Constants.DirectionNorth;
				    }
				    else if(path[0] == Constants.DirectionEast)
				    {
				        path[0] = Constants.DirectionWest;
				    }
				    else if(path[0] == Constants.DirectionWest)
				    {
				        path[0] = Constants.DirectionEast;
				    }
				    input[10] = ((double)(path[0] + 1)) / 5.0;	// Add 1 so that 0 means there is no path
				}
				else
				{
				    input[10] = 0;
				}
			}
			else
			{
			    input[10] = 0;
			}
		}

		// GET OUTPUT FROM MLP
        if(DEBUG)
            System.out.println("get output");
        
		int [] output = mlp.classify(input);

		// PERFORM ACTION
        if(DEBUG)
            System.out.println("act");
		if(output[MOVE] == 1)	// ACTION == MOVE
		{
		    int direction;
		    int speed;
		    
		    // GET SPEED
		    if(output[SPEED_0] == 1)
		        speed = 0;
		    else if(output[SPEED_1] == 1)
		        speed = 1;
		    else if(output[SPEED_2] == 1)
		        speed = 2;
		    else if(output[SPEED_3] == 1)
		        speed = 3;
		    else if(output[SPEED_4] == 1)
		        speed = 4;
		    else
		        speed = 0;
		    
		    // GET DIRECTION
		    if(output[NORTH] == 1)
		        direction = Constants.DirectionNorth;
		    else if(output[SOUTH] == 1)
		        direction = Constants.DirectionSouth;
		    else if(output[EAST] == 1)
		        direction = Constants.DirectionEast;
		    else if(output[WEST] == 1)
		        direction = Constants.DirectionWest;
		    else
		    {
		        direction = 0;
		        speed = 0;
		    }
		    		    
		    move(direction, speed);
		    
		    if(DEBUG)
		        System.out.println("move(" + direction + ", " + speed + ")");
		}
		else if((output[ATTACK] == 1) && canAttack())
		    attack();
		else if((output[EAT] == 1) && canEat())
		    eat();
		else if((output[FLEE] == 1) && canFlee())
		{
		    int direction;
		    
		    // GET DIRECTION
		    if(output[NORTH] == 1)
		        direction = Constants.DirectionNorth;
		    else if(output[SOUTH] == 1)
		        direction = Constants.DirectionSouth;
		    else if(output[EAST] == 1)
		        direction = Constants.DirectionEast;
		    else
		        direction = Constants.DirectionWest;
		    
		    flee(direction);
		}
		else if((output[MATE] == 1) && canMate())
		    mate(); 
		
        if(DEBUG)
        {
            for(int i = 0; i < input.length; i++)
            {
                System.out.print(input[i] + " ");
            }
            System.out.println("");
            
            for(int i = 0; i < output.length; i++)
            {
                System.out.print(output[i] + " ");
            }
            System.out.println("");
            System.out.println("move done");
        }
    }

    /* (non-Javadoc)
     * @see yaes.application.eel6938.game.objects.Agent#getName()
     */
    public String getName() {
        // TODO Auto-generated method stub
        return "MLP Agent";
    }

    /* (non-Javadoc)
     * @see yaes.application.eel6938.game.objects.Agent#getColor()
     */
    public int[] getColor() {
        // TODO Auto-generated method stub
		int[] rgb = {0,255,255};
        return rgb;
    }

}
