/*
 * Created on Mar 25, 2005
 *
 */
package yaes.application.eel6938.classagents.cxbr;

import java.awt.Point;
import java.util.Stack;
import yaes.application.eel6938.game.Constants;
import yaes.application.eel6938.game.Configuration;

/**
 * This class represents the Internal Map used for tracking and guiding the Agent 
 * @author Viet Trinh
 *
 */
public class CxBR_Map 
{
	private byte _intenal_map[][];
	private Point _current_location = null;

	public final static byte EMPTY = 1;
	public final static byte UNKNOWN = 0;
	public final static byte OBS = 9;
	private final static byte DATAMAP_OBS = Constants.ContentObstacle;
	private final static byte DATAMAP_AGENT = Constants.ContentAgent;
	private final static byte DATAMAP_EMPTY = Constants.ContentOpen;
	
	private final static int X_MAX = Configuration.AppSceneWidth;
	private final static int Y_MAX = Configuration.AppSceneHeight;
	public final static int MAX_SPEED = Configuration.AgentMaxSpeed;
	public final static int REGULAR_SPEED = 2;
	public final static int FLEE_SPEED = Configuration.AgentFleeSpeed;
	public final static int MAX_JIGGLE_CYCLES = 6;
	private int SPEED = REGULAR_SPEED;
	private final static int MAX_STACK_SIZE = 50;
	private final static int MAX_SPEED_CYCLES = 20;
	
	private int _last_direction;
	private Stack<Move> _move_set;
	private boolean _explore_n = false;
	private boolean _explore_e = false;
	private boolean _explore_w = false;
	private boolean _explore_s = false;
	private int _max_speed_cycles = 0;
	
	/**
	 * Constructor
	 *
	 */
	public CxBR_Map() 
	{
		this._current_location = new Point(X_MAX/2, Y_MAX/2);
		this._intenal_map = new byte[X_MAX][Y_MAX];
		for (int i=0;i<X_MAX;i++)
			for (int j=0;j<Y_MAX;j++)
				this._set_imap_value(i, j, UNKNOWN);
		//The spot you are standing on cannot be an obstacle
		this._set_imap_value(X_MAX/2, Y_MAX/2, EMPTY);
		this._last_direction = Constants.DirectionNorth;
		this._move_set = new Stack<Move>();
	}
	
	/**
	 * Use this functon to set the Agent to go it Maximum speed for traveling. This is primarily use for fleeing, and staying
	 * away from an attacking enemy.
	 *
	 */
	public void useMaxSpeed()
	{
		this.SPEED = MAX_SPEED;
		this._max_speed_cycles = MAX_SPEED_CYCLES;
	}
	
	/**
	 * Use this funtion to determine if the agent is going at Max speed or not.
	 * @return true if the agent is moving at max speed.
	 */
	public boolean isMaxSpeed()
	{
		return (this.SPEED == MAX_SPEED);
	}
	
	/**
	 * Use this function to get the speed that the agent should travel with.
	 * @return The speed that the agent should be traveling
	 */
	public int getSpeed() { return this.SPEED; }
	
	/**
	 * Use this function to check the speed timer, if the max speed timer runs out, then the speed of the agent is
	 * returned to normal.
	 *
	 */
	private void _check_speed()
	{
		if (this._max_speed_cycles < 0)
		{
			this._max_speed_cycles = 0;
			this.SPEED = REGULAR_SPEED;
		}
		this._max_speed_cycles--;
	}
	
	/**
	 * X coordinate function to accomodate a circular map.
	 * @param x
	 * @return the modded x value
	 */
	private int xx(int x) { return ((X_MAX + x) % X_MAX); }
	
	/**
	 * Y coordinate function to accomodate a circular map.
	 * @param y
	 * @return the modded y value
	 */
	private int yy(int y) { return ((Y_MAX + y) % Y_MAX); }
	
	/**
	 * Use this function to set the internal map value
	 * @param x
	 * @param y
	 * @param value
	 */
	private void _set_imap_value(int x, int y, byte value) { this._intenal_map[xx(x)][yy(y)] = value; }
	
	/**
	 * Use this function to get the value of a point in the internal map
	 * @param x
	 * @param y
	 * @return
	 */
	private int _get_imap_value(int x, int y) { return this._intenal_map[xx(x)][yy(y)]; }
	
	/**
	 * Use this function initate the exploring function of the map. It will merge the inputted sensor map
	 * with the internal map, as well as determining which directions that are not explored
	 * @param sensorDataMap The sensormap gathered from the agent
	 */
	public void explore(int sensorDataMap[][])
	{
		int points[][] = sensorDataMap;
		int half_point = -1 * (points.length - 1)/2;
		
		this._check_speed();
		
		//Explore the Visible Area
		for(int y=0;y<points[0].length;y++)
		{
			for(int x=0;x<points.length;x++)
			{
				if (points[x][y] == DATAMAP_OBS || points[x][y] == DATAMAP_AGENT)
					this._set_imap_value(half_point+x+this._current_location.x, half_point+y+this._current_location.y, OBS);
				else
					this._set_imap_value(half_point+x+this._current_location.x, half_point+y+this._current_location.y, EMPTY);
			}
		}
		
		//Get Unexplored Area
		int min_side_y = half_point - 1 + this._current_location.y;
		int max_side_y = half_point + points.length + this._current_location.y;
		int min_side_x = half_point - 1 + this._current_location.x;
		int max_side_x = half_point + points.length + this._current_location.x;
		
		
		boolean ndir = false;
		boolean sdir = false;
		boolean nwall = false;
		boolean swall = false;
		/*
		System.out.println("Current Location: (" + this._current_location.x + ", " + this._current_location.y + ")");
		System.out.println("Half Point: " + half_point);
		System.out.println("MinX, MinY: (" + min_side_x + ", " + min_side_y + ")");
		System.out.println("MaxX, MaxY: (" + max_side_x + ", " + max_side_y + ")");

		for(int y=-1;y<=points[0].length;y++)
		{
			for(int x=-1;x<=points.length;x++)
			{
				System.out.print(this._get_imap_value(half_point+x+this._current_location.x,half_point+y+this._current_location.y));
			}
			System.out.println();
		}
		
		for (int s = min_side_x + 1 ; s < max_side_x - 1 ; s++)
		{
			//North Wall
			if (this.isObstacle(s, min_side_y + 1) == false)
				nwall = false;
			if (this._get_imap_value(s, min_side_y) == UNKNOWN)
				ndir++;
			
			//South Wall
			if (this.isObstacle(s, max_side_y - 1) == false)
				swall = false;
			if (this._get_imap_value(s, max_side_y) == UNKNOWN)
				sdir++;
		}
		*/
		//System.out.println("Check North: (" + this._current_location.x + ", " + (min_side_y+1) + ")");
		if (this.isObstacle(this._current_location.x, min_side_y+1))
			nwall = true;
		if (this._get_imap_value(this._current_location.x, min_side_y) == UNKNOWN)
			ndir = true;
		
		//System.out.println("Check South: (" + this._current_location.x + ", " + (max_side_y-1) + ")");
		if (this.isObstacle(this._current_location.x, max_side_y-1))
			swall = true;
		if (this._get_imap_value(this._current_location.x, max_side_y) == UNKNOWN)
			sdir = true;
			
		boolean edir = false;
		boolean wdir = false;
		boolean ewall = false;
		boolean wwall = false;
		
		/*
		for (int s = min_side_y + 1 ; s < max_side_y - 1 ; s++)
		{		
			//East Wall
			if (this.isObstacle(min_side_x + 1, s) == false)
				ewall = false;
			if (this._get_imap_value(min_side_x, s) == UNKNOWN)
				edir++;
			
			//West Wall
			if (this.isObstacle(max_side_x -1, s) == false)
				wwall = false;
			if (this._get_imap_value(max_side_x, s) == UNKNOWN)
				wdir++;
		}
		*/
		//System.out.println("Check East: (" + (min_side_x + 1) + ", " + this._current_location.y + ")");
		if (this.isObstacle(min_side_x + 1, this._current_location.y))
			wwall = true;
		if (this._get_imap_value(min_side_x, this._current_location.y) == UNKNOWN)
			wdir = true;
		
		//System.out.println("Check West: (" + (max_side_x - 1) + ", " + this._current_location.y + ")");
		if (this.isObstacle(max_side_x - 1, this._current_location.y))
			ewall = true;
		if (this._get_imap_value(max_side_x, this._current_location.y) == UNKNOWN)
			edir = true;		
		
		this._explore_n = ((nwall == false) && (ndir) && this._can_move(Constants.DirectionNorth, SPEED, false));
		this._explore_e = ((ewall == false) && (edir) && this._can_move(Constants.DirectionEast, SPEED, false));
		this._explore_w = ((wwall == false) && (wdir) && this._can_move(Constants.DirectionWest, SPEED, false));
		this._explore_s = ((swall == false) && (sdir) && this._can_move(Constants.DirectionSouth, SPEED, false));
	
		/*if (this._explore_n)
		{
			System.out.println("    ^");
			System.out.println("    |");
			System.out.println("    |");
			System.out.println("    |");
		}
		if (this._explore_w)
		{
			if (this._explore_e)
				System.out.println("<---+--->");
			else
				System.out.println("<---+");
		}
		else if (this._explore_w == false && this._explore_e)
		{
			System.out.println("    +--->");
		}
		else
			System.out.println("    +");
		if (this._explore_s)
		{
			System.out.println("    |");
			System.out.println("    |");
			System.out.println("    |");
			System.out.println("    v");
		}
		*/
	}
	
	/**
	 * See explore(). This is the same as explore, however, it will consider food as an obstacle so the agent can flee properly
	 * @param sensorDataMap The sensormap gathered from the agent
	 */
	public void exploreFlee(int sensorDataMap[][])
	{
		int points[][] = sensorDataMap;
		int half_point = -1 * (points.length - 1)/2;
		
		for(int y=0;y<points[0].length;y++)
		{
			for(int x=0;x<points.length;x++)
			{
				if (points[x][y] != DATAMAP_EMPTY)
					this._set_imap_value(half_point+x+this._current_location.x, half_point+y+this._current_location.y, OBS);
				else
					this._set_imap_value(half_point+x+this._current_location.x, half_point+y+this._current_location.y, EMPTY);
			}
		}
	}
	
	/**
	 * Use this function to determine the next exploration direction.
	 * @return the direction to explore, -1 if there is nothing to explore
	 */
	public int getExplorationDirection()
	{
		int direction = this._last_direction;
		for (int i=0;i<4;i++)
		{
			if (this.canExploreDirection(direction))
				return direction;
			direction = this.getPerpendicularDirection(direction);
		}
		return -1;
	}
	
	/**
	 * Use this function to get the last move made by the agent.
	 * @return The last move the agent has made
	 */
	public Move getLastMove()
	{
		if (!this._move_set.isEmpty())
			return (Move) this._move_set.pop();
		return null;
	}
	
	/**
	 * Use this function to determine if you can backtrack or not, you can't backtrack if the stack is empty
	 * @return true if the agent still has moves to backtrack with
	 */
	public boolean canBackTrack()
	{
		if (!this._move_set.isEmpty())
			return true;
		return false;
	}
	
	/**
	 * Use this function to determine the last direction the agent has moved
	 * @return The last direction moved
	 */
	public int getDirection() { return this._last_direction; }
	
	/**
	 * Use this function to get the relative coordinates from the agent's current location to the point of interest
	 * @param p
	 * @return the point with relative coordinates
	 */
	public Point getRelativePoint(Point p)
	{
		return new Point(xx(p.x - this._current_location.x), yy(p.y - this._current_location.y));
	}
	
	/**
	 * Use this function to determine if the direction needs to be explored 
	 * @param direction
	 * @return true is the direction has been unexplored, false if it has been explored
	 */
	public boolean canExploreDirection(int direction)
	{
		if ((direction == Constants.DirectionNorth) && this._explore_n)
			return true;
		else if ((direction == Constants.DirectionSouth) && this._explore_s)
			return true;
		else if ((direction == Constants.DirectionEast)  && this._explore_e)
			return true;
		else if ((direction == Constants.DirectionWest) && this._explore_w)
			return true;
		return false;
	}
	
	/**
	 * Use this functon to determine if the point is an obstacle or not
	 * @param p The point of interest
	 * @return true if there is an obstacle
	 */
	public boolean isObstacle(Point p)
	{
		if (this._get_imap_value(p.x, p.y) == OBS)
			return true;
		return false;
	}
	
	/**
	 * Use this functon to determine if the point is an obstacle or not
	 * @param x
	 * @param y
	 * @return true if there is an obstacle
	 */
	public boolean isObstacle(int x, int y)
	{
		return this.isObstacle(new Point(x,y));
	}
	
	/**
	 * Use this function to determine if you can move in the direction with the given speed. The function will
	 * determine if the path is free of obstacles or not. When you call canMove, if it returns true, you have to 
	 * move in that direction because it logs the position
	 * @param direction
	 * @param speed
	 * @return true if the agent can move, false if there are obstacles in the way
	 */
	public boolean canMove(int direction, int speed)
	{
		Point old_location = (Point) this._current_location.clone();
		speed = (speed > MAX_SPEED ? MAX_SPEED : speed);
		if (this._can_move(direction, speed, true) == false)
			return false;
		else
		{
			this._last_direction = direction;
			this._move_set.push(new Move(old_location, direction, speed));
			if (this._move_set.size() > MAX_STACK_SIZE)
			{
				this._move_set.remove(0);
			}
			return true;
		}
	}

	/**
	 * Use this function to determine if you can flee in the direction with the flee speed. The function will
	 * determine if the path is free of obstacles or not. When you call canFlee, if it returns true, you have to 
	 * move in that direction because it logs the position
	 * @param direction
	 * @param speed
	 * @return true if the agent can flee, false if there are obstacles in the way
	 */
	public boolean canFlee(int direction)
	{
		Point old_location = (Point) this._current_location.clone();
		int speed = FLEE_SPEED;
		if (this._can_move(direction, speed, true) == false)
			return false;
		else
		{
			this._last_direction = direction;
			return true;
		}
	}
	
	/**
	 * Use this function to determine if you can move in the given direction with the given speed
	 * @param direction
	 * @param speed
	 * @param set_position Set this to true if you want the move to be recorded in the internal map
	 * @return true if the agent can move in the particular direction
	 */
	private boolean _can_move(int direction, int speed, boolean set_position)
	{
		if (direction == Constants.DirectionNorth)
		{
			for(int i=1;i<=speed;i++)
				if (this._get_imap_value(this._current_location.x, this._current_location.y-i) == OBS)
					return false;
			if (set_position) 
				this._current_location.y  = yy(this._current_location.y - speed);
			return true;
		}
		else if (direction == Constants.DirectionSouth)
		{
			for(int i=1;i<=speed;i++)
				if (this._get_imap_value(this._current_location.x, this._current_location.y+i) == OBS)
					return false;
			if (set_position) 
				this._current_location.y  = yy(this._current_location.y + speed);
			return true;
		}
		else if (direction == Constants.DirectionWest)
		{
			for(int i=1;i<=speed;i++)
				if (this._get_imap_value(this._current_location.x-i, this._current_location.y) == OBS)
					return false;
			if (set_position) 
				this._current_location.x  = xx(this._current_location.x - speed);
			return true;
		}
		else if (direction == Constants.DirectionEast)
		{
			for(int i=1;i<=speed;i++)
				if (this._get_imap_value(this._current_location.x+i, this._current_location.y) == OBS)
					return false;
			if (set_position) 
				this._current_location.x  = xx(this._current_location.x + speed);
			return true;
		}
		else
			return false;
	}
	
	/**
	 * Use this function to print out the saved move traces of the agents, it prints it out in LIFO order.
	 *
	 */
	public void printMoveTrace()
	{
		Stack temp = (Stack) this._move_set.clone();
		Move temp_move = null;
		while (!temp.empty())
		{
			temp_move = (Move) temp.pop();
			temp_move.printMove();
		}
	}
	
	/**
	 * Use this function to check if the point is in the same direction as the given direction
	 * @param p The point of interest
	 * @param direction The direction to check
	 * @return true if the point is in the given direction
	 */
	public boolean isSameDirection(Point p, int direction)
	{
		if (p.y > 0 && direction == Constants.DirectionNorth)
			return true;
		else if (p.y < 0 && direction == Constants.DirectionSouth)
			return true;
		else if (p.x < 0 && direction == Constants.DirectionWest)
			return true;
		else if (p.y > 0 && direction == Constants.DirectionEast)
			return true;
		return false;
	}
	
	/**
	 * Use this function to determine which direction is clear of obstacles, it searches the direction 90 degrees
	 * from the current direction if the current direction fails, then it searches the direction negative 90 degrees,
	 * then it will search the opposite 180 degree direction last.
	 * @param direction
	 * @param speed
	 * @return The first direction that is clear from obstacles
	 */
	public int getClearDirection(int direction, int speed)
	{
		//speed = (speed > MAX_SPEED ? MAX_SPEED : speed);
		//Check direction
		if (this._can_move(direction, speed, false))
			return direction;
		direction = this.getPerpendicularDirection(direction);
		if (this._can_move(direction, speed, false))
			return direction;
		direction = this.getOppositeDirection(direction);
		if (this._can_move(direction, speed, false))
			return direction;
		direction = this.getPerpendicularDirection(direction);
		direction = this.getOppositeDirection(direction);
		if (this._can_move(direction, speed, false))
			return direction;
		return this.getPerpendicularDirection(direction);
	}
	
	/**
	 * Use this function to get the closest point to the agent given a set of points.
	 * @param p The array of points to search
	 * @return The closest point to the agent
	 */
	public Point getClosestPoint(Point p[]) 
	{
		Point currentPoint = p[0];
		for (int i = 0; i < p.length; i++) 
		{
			if (Math.abs(p[i].x) + Math.abs(p[i].y) < Math.abs(currentPoint.x) + Math.abs(currentPoint.y)) 
			{
				currentPoint = p[i];
			}
		}
		return currentPoint;
	}
	
	/**
	 * Use this function to get the opposite direction of the given direction
	 * @param direction
	 * @return The opposite direction to the given direction
	 */
	public int getOppositeDirection(int direction)
	{
		if (direction == Constants.DirectionNorth)
			return Constants.DirectionSouth;
		else if (direction == Constants.DirectionSouth)
			return Constants.DirectionNorth;
		else if (direction == Constants.DirectionEast)
			return Constants.DirectionWest;
		else if (direction == Constants.DirectionWest)
			return Constants.DirectionEast;
		return Constants.DirectionNorth;
	}

	/**
	 * Use this function to get the perpendicular 90 degree rotated direction of the given direction. North would return East
	 * @param direction
	 * @return The perpendicular direction to the given direction
	 */
	public int getPerpendicularDirection(int direction)
	{
		if (direction == Constants.DirectionNorth)
			return Constants.DirectionEast;
		else if (direction == Constants.DirectionEast)
			return Constants.DirectionSouth;
		else if (direction == Constants.DirectionSouth)
			return Constants.DirectionWest;
		else if (direction == Constants.DirectionWest)
			return Constants.DirectionNorth;
		return Constants.DirectionNorth;
	}	
}
