/*
 * Created on Mar 25, 2005
 *
 */
package yaes.application.eel6938.classagents.cxbr;

import yaes.application.eel6938.game.Constants;
import yaes.application.eel6938.game.objects.Agent;
import java.awt.Point;
import yaes.application.eel6938.classagents.CxBRAgent;

/**
 * This class represents the NearDeathContext
 * The agent will not flee in this context back it cost too much. Instead it will try to survive and eat food if its visible.
 * If there is no food, then it will cope with the fact that it will die soon, therefore will try to mate, if that fails,
 * it will try to attack whoever is in range.
 * @author Viet Trinh
 *
 */
public class NearDeathContext extends CxBR_Context 
{
	private static final int[] CBLACK = {50, 0, 50};
	private static final int[] CWHITE = {255, 0, 255};
	private static final int BLACK = 0;
	private static int WHITE = 10;
	private int _status;
	private int _children = 0;
	
	/**
	 * Constructor
	 * @param name Name of the Agent
	 * @param a Agent
	 * @param m Map
	 */
	public NearDeathContext(String name, Agent a, CxBR_Map m) 
	{
		super(name, a, m);
		this._status = WHITE;
	}

	@Override
	public void execute() 
	{
		//System.out.println("Near Death!!");
		this._status = (this._status == BLACK ? WHITE : BLACK);
		Point enemy = this.canAttackNoLimit();
		if (this._agent.canMate() && this._children < 1)
		{
			this._agent.mate();
			this._children++;
		}
		else if (enemy != null)
		{
			this._attack(enemy);
		}
		else if (this._agent.canMate() && this._children < CxBRAgent.MAX_NEAR_DEATH_CHILDREN)
		{
			this._agent.mate();
			this._children++;
		}
		else if (this.canEat())
		{
			Point foodlist[] = this._agent.getSensorFoodList();
			Point food = this._map.getClosestPoint(foodlist);
			this._eat(food);
		}
		else
		{
			int speed = this._map.getSpeed();
			int direction = this._map.getDirection();
			if (this.move(direction, speed) == false)
			{
				direction = this._map.getClearDirection(direction, speed);
				this.move(direction, speed);
			}
		}
	}

	/**
	 * Use this function to eat a piece of food
	 * @param food The relative point where the food if located
	 * @return true is it eats, false if it is moving towards food
	 */
	protected boolean _eat(Point food)
	{
		int x = food.x;
		int y = food.y;
		
		if      (x == 0  && y == 0)	 { this._agent.eat(); return true; }
		else if (x == 1  && y == 0)  { this._agent.eat(Constants.DirectionEast);  return true; }
		else if (x == -1 && y == 0)  { this._agent.eat(Constants.DirectionWest);  return true; }
		else if (x == 0  && y == 1)  { this._agent.eat(Constants.DirectionNorth); return true; }
		else if (x == 0  && y == -1) { this._agent.eat(Constants.DirectionSouth); return true; }
		else 						 { this._move_towards(food); return false; }
	}
	
	/**
	 * Use this function to attack and enemy
	 * @param enemy The relative point where the enemy if located
	 * @return true is it attacks, false if it is moving towards an enemy
	 */
	private boolean _attack(Point enemy)
	{
		int x = enemy.x;
		int y = enemy.y;
		
		if (x == 0 && y == 0) 		{ this._agent.attack(); return true; }
		else if (x == 1 && y == 0) 	{ this._agent.attack(Constants.DirectionEast);  return true; }
		else if (x == -1 && y == 0) { this._agent.attack(Constants.DirectionWest);  return true; }
		else if (x == 0 && y == 1)  { this._agent.attack(Constants.DirectionNorth); return true; }
		else if (x == 0 && y == -1) { this._agent.attack(Constants.DirectionSouth); return true; }
		else  						{ this._move_towards(enemy); return false; }
	}
	
	@Override
	public String executeSentinelRules() 
	{
		//The universal Sentinel Rules will Override this is the agent if still near death
		return "ExploreContext";
	}
	
	@Override
	public int[] getColor()
	{
		//Color Changer
		return (this._status == WHITE ? CWHITE : CBLACK);
	}
}
