package yaes.application.eel6938.classagents.expertagent;
/* Author: Majid Ali Khan
 * Dated: 04-01-2005
*/
import java.util.Hashtable;
import java.util.Vector;

import yaes.application.eel6938.game.Constants;
/*
 * This class maintains the history of agent steps.
 * The number of steps to be maintained is determined by 
 * 'NumberOfSteps' constant defined inside this class.
 */
public class History {
	private Vector<Step> m_steps;//snapshot of steps
	private int m_stepIndex;
	private final int NumberOfSteps = 40; // number of steps to be stored in history
	private static History m_Instance;
	private Hashtable<Integer,AgentHistory> agentList; // list of interacting agents and their history
	private int[] movementDirectionsHistory;
	private int movementDirectionIndex = 0;
	private final int NumberOfDirections = 500; // number of directions stored
	public static History getInstance() {
		if (m_Instance == null)
			m_Instance = new History();
		return m_Instance;
	}
	private History() {
		m_steps = new Vector<Step>(NumberOfSteps);
		agentList = new Hashtable<Integer,AgentHistory>();
		movementDirectionsHistory = new int[NumberOfDirections];
		for (int i=0; i<NumberOfDirections; i++)
			movementDirectionsHistory[i] = -1;
		m_stepIndex = 0;
	}
	public void addStep(Step step) {
		m_steps.insertElementAt(step,m_stepIndex);
		m_stepIndex++;
		
		if(m_stepIndex == NumberOfSteps+1) {
			m_steps.removeElementAt(0);
			m_stepIndex--;
		}
	}
	public AgentHistory getAgentHistory(int agentID) {
		AgentHistory hist;
		Integer id = new Integer(agentID);
		if (agentList.containsKey(id)) {
			hist = agentList.get(id);
		} else {
			hist = new AgentHistory();
			hist.setAgentID(agentID);
			agentList.put(id,hist);
		}
		return hist;
	}
	public Step getCurrentStep() {
		return m_steps.elementAt(m_stepIndex-1);
	}
	public Step[] getHistory() {
		return (Step[])m_steps.toArray();
	}
	public String toString() {
		String returnValue = "";
		for (int i=0; i<m_steps.size();i++) {
			returnValue += m_steps.elementAt(i).toString();
		}
		return returnValue;
	}
	public void setMovementDirection(int direction) {
		movementDirectionsHistory[movementDirectionIndex]=direction;
		movementDirectionIndex = (movementDirectionIndex + 1) % NumberOfDirections;
	}
	public int getSuggestedMovementDirection() {
		// Choose the direction which has been minimally taken yet.
		int min_id=0, min_number=0;
		int[] num_side = new int[4];
		for (int i=1; i<NumberOfDirections; i++) {
			if (movementDirectionsHistory[i] != -1)
				num_side[movementDirectionsHistory[i]]++;
		}
		min_number = num_side[0];
		//System.out.println("Can move in direction " + 0 + ":" +num_side[0] +":"+ History.getInstance().getCurrentStep().getLocalMap().canMove(0,1));
		for (int j=1; j<4; j++) {
			//System.out.println("Can move in direction " + j + ":" +num_side[j] +":"+ History.getInstance().getCurrentStep().getLocalMap().canMove(j,1));
			if (num_side[j]<min_number
					&& History.getInstance().getCurrentStep().getLocalMap().canMove(j,4)) {
				min_number = num_side[j];
				min_id = j;
			}
		}
		return min_id;
	}
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		History h = History.getInstance();
		h.addStep(new Step(1,null));
		h.addStep(new Step(2,null));
		h.addStep(new Step(3,null));
		h.addStep(new Step(4,null));
		//System.out.println(h);
		h.addStep(new Step(5,null));
		h.addStep(new Step(6,null));
		//System.out.println(h);
	}

}
