package yaes.application.eel6938.classagents.expertagent;

import yaes.application.eel6938.game.Constants;

public class UtilityEvaluator {

	private int[][] utilityTable;
	private static final int UtilityRows = 36;
	private static final int UtilityCols = 3;
	
	private static final int ExpertCommand_Column = 0;
	private static final int OpponentCommand_Column = 1;
	private static final int Utility_Column = 2;
	
	private static UtilityEvaluator instance;
	
	private UtilityEvaluator() {
		utilityTable = new int[UtilityRows][UtilityCols];
		setupTable();
	}
	public static UtilityEvaluator getInstance() {
		if (instance == null) instance =  new UtilityEvaluator();
		return instance;
	}
	private void setupTable() {
		int expertCommand = 0;
		int opponentCommand = 0;
		int strenghtLevel = 0;
		int foodInsight = 0;
		
		for (int i=0; i<UtilityRows; i++) {
			for (int j=0; j<UtilityCols-1; j++) {
				if (j==ExpertCommand_Column) utilityTable[i][j] = expertCommand; 
				if (j==OpponentCommand_Column) utilityTable[i][j] = opponentCommand;
			}
			expertCommand = (expertCommand + 1)%6;
			if (expertCommand == 0) opponentCommand = (opponentCommand + 1)%6;
		}
			
	}
	private void assignUtilities(int agentID, int deltaStrength) {
		ruleUtilityForAttack(agentID, deltaStrength);
		ruleUtilityForFlee(agentID, deltaStrength);
	}
	// Rule: What utility do I get by attacking
	private void ruleUtilityForAttack(int agentID, int deltaStrength) {
		for (int i=0; i<UtilityRows; i++) {
			if ((utilityTable[i][ExpertCommand_Column] == Constants.CommandAttack)) {
				if (utilityTable[i][OpponentCommand_Column] == Constants.CommandAttack)
					utilityTable[i][Utility_Column] = 
						(int)(100*History.getInstance().getAgentHistory(agentID).getLikelinessOf(Constants.CommandFlee)) + (int)(deltaStrength*0.1);
				if (utilityTable[i][OpponentCommand_Column] == Constants.CommandFlee)
					utilityTable[i][Utility_Column] = (int)(100*History.getInstance().getAgentHistory(agentID).getLikelinessOf(Constants.CommandAttack)) + (int)(deltaStrength*0.1);

			}
		}
	}
	// Rule: What utility do I get by fleeing
	private void ruleUtilityForFlee(int agentID, int deltaStrength) {
		for (int i=0; i<UtilityRows; i++) {
			if ((utilityTable[i][ExpertCommand_Column] == Constants.CommandFlee)) {
				if (utilityTable[i][OpponentCommand_Column] == Constants.CommandAttack)
					utilityTable[i][Utility_Column] = (int)(50*History.getInstance().getAgentHistory(agentID).getLikelinessOf(Constants.CommandAttack)) - (int)(deltaStrength*0.1);
				if (utilityTable[i][OpponentCommand_Column] == Constants.CommandFlee)
					utilityTable[i][Utility_Column] = (int)(50*History.getInstance().getAgentHistory(agentID).getLikelinessOf(Constants.CommandFlee)) - (int)(deltaStrength*0.1);

			}
		}
	}
	// Rule: What utility do I get by other actions
	private void ruleUtilityForOtherActions(int agentID) {
		for (int i=0; i<UtilityRows; i++) {
			if ((utilityTable[i][ExpertCommand_Column] != Constants.CommandFlee)
					&& (utilityTable[i][ExpertCommand_Column] != Constants.CommandAttack)) {

				utilityTable[i][Utility_Column] = 0;

			}
		}
	}

	
	public int getOptimalStrategy(int agentID, int deltaStrength) {
		assignUtilities(agentID, deltaStrength);
		//System.out.println(agentID + ":" + this);
		int[] sumStrategy = new int[6];
		int max_number =0, max_strategy = Constants.CommandAttack;
		for (int i=0; i<UtilityRows; i++) {
			sumStrategy[utilityTable[i][ExpertCommand_Column]]+=utilityTable[i][Utility_Column];
		}
		for (int i=0; i<6; i++) {
			//System.out.println(sumStrategy[i]);
			if (sumStrategy[i] > max_number) {
				max_number = sumStrategy[i];
				max_strategy = i;
			}
		}
		return max_strategy;
	}
	public String toString() {
		String returnValue = "";
		for (int i=0; i<UtilityRows; i++) {
			for (int j=0; j<UtilityCols; j++) {
				returnValue += "  " + utilityTable[i][j];
			}
			returnValue += " \r\n";
		}
		return returnValue;
	}
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		UtilityEvaluator eval = UtilityEvaluator.getInstance();
		History.getInstance().getAgentHistory(0).setAgentBehavior(Constants.CommandAttack);
		History.getInstance().getAgentHistory(0).setAgentBehavior(Constants.CommandAttack);
		History.getInstance().getAgentHistory(0).setAgentBehavior(Constants.CommandFlee);
		History.getInstance().getAgentHistory(0).setAgentBehavior(Constants.CommandAttack);
		int action = eval.getOptimalStrategy(0,10);
		//System.out.println(eval);
		//System.out.println(action);
	
	}

}
