//-----------------------------------------------
// FILE:		MultiLayerPerceptron.java
// PACKAGE:		ann
// CLASSES:		MultiLayerPerceptron
// AUTHOR:		T. Ryan Fitz-Gibbon
// DATE:		Mar 10, 2005
//-----------------------------------------------
package yaes.application.eel6938.classagents.mlp;

/*import org.joone.engine.FullSynapse;
import org.joone.engine.Layer;
import org.joone.engine.LinearLayer;
import org.joone.engine.Monitor;
import org.joone.engine.NeuralNetEvent;
import org.joone.engine.NeuralNetListener;
import org.joone.engine.SigmoidLayer;
import org.joone.engine.learning.TeachingSynapse;
import org.joone.io.FileInputSynapse;
import org.joone.io.MemoryInputSynapse;
import org.joone.io.MemoryOutputSynapse; 
import org.joone.net.NeuralNet;*/
import org.joone.engine.*;
import org.joone.engine.learning.*;
import org.joone.io.*;
import org.joone.net.*;
import org.joone.util.*;
import java.io.*;


//----------------------------------------------------------------------------
// CLASS:			MultiLayerPerceptron
// DESCRIPTION:		This class uses the neural network package Joone to
//					implement neural networks in a more usable manner.
// EXTENDS:			The NeuralNet class from Joone.
// PUBLIC METHODS:
//	CONSTRUCTORS:
//		MultiLayerPerceptron()
//			This is the default constructor.  It calls the main constructor
//			with some default values.  Don't use this constructor.
//
//		MultiLayerPerceptron(int numInNodes, int [] numHiddenNodes,	int
//		numOutNodes, double learningRate, double momentum) 
//			Main constructor.
//
//	MUTATORS:
//		trainEpochs(double [][] trainData, int totalEpochs)
//			The networked is trained using the training data until the number
//			of epochs equas totalEpochs.  Each row of trainData is an input
//			pattern followed by the desired output(s).
//
//	ACCESSORS:
//		double [] getOutput(double [] input)
//			Provides the output of the network to one input pattern.
//
//		double [][] getOutput(double [][] inputs)
//			Provides the output of the network to many input patterns.
//
//		double getAccuarcy(double [][] testData)
//			Get the accuarcy of the network on the testData.
//
//----------------------------------------------------------------------------
public class MultiLayerPerceptron extends NeuralNet implements NeuralNetListener, NeuralValidationListener
{	
// PUBLIC
	// METHODS
		// CONSTRUCTORS
	//------------------------------------------------------------------------
	// METHOD:			MultiLayerPerceptron()
	// DESCRIPTION:		This is the default constructor.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	A MultiLayerPerceptron object is created by calling
	//					the main constructor with some default values.
	//------------------------------------------------------------------------
	public MultiLayerPerceptron()
	{
		this(2, new int [] {4}, 1, 0.8, 0.3);
	}
	
	//------------------------------------------------------------------------
	// METHOD:			MultiLayerPerceptron(int numInNodes, int [] 
	//					numHiddenNodes, int numOutNodes, double learningRate,	
	//					double momentum)
	// DESCRIPTION:		This is the main constructor.
	// PRECONDITIONS:	numInNodes: int >= 1
	//					numHiddenNodes: int [], any length, each element >= 1
	//					numOutNodes: int >= 1
	//					learningRate: double > 0
	//					momentum: double > 0
	// POSTCONDITIONS:	If any of the preconditions are note met, the boundry
	//					value is used instead.  A NeuralNet object is created
	//					with numInNodes input nodes, numHiddenNodes.length()
	//					hidden layers, numHiddenNodes(n-1) hidden nodes in the
	//					nth hidden layer, a learning rate of learningRate, and
	//					a momentum of momentum.  Sigmoid activation functions
	//					are used with every applicable node.  The networks
	//					is set to function approximation mode.
	//------------------------------------------------------------------------	
	public MultiLayerPerceptron(int numInNodes, int [] numHiddenNodes, int numOutNodes, double learningRate, double momentum)
	{
		// CALL SUPER CLASS, NeuralNet, CONSTRUCTOR
		super();
		
		// TEST INPUT PARAMETERS, IF NOT VALID, SET TO DEFAULT VALUES
			// NUMBER OF INPUT NODES >= 1
		if(numInNodes < 1)
		{
			numInNodes = 1;
		}
		
			// NUMBER OF HIDDEN NODES IN LAYER 1 >= 1
		if((numHiddenNodes[0] < 1) || (numHiddenNodes.length < 1))
		{
			numHiddenNodes[0] = 1;
		}
		
			// NUMBER OF OUTPUT NODES >= 1
		if(numOutNodes < 1)
		{
			numOutNodes = 1;
		}
		
			// LEARNING RATE > 0
		if(learningRate <= 0)
		{
			learningRate = 0.8;
		}
		
			// MOMENTUM > 0
		if(momentum <= 0)
		{
			momentum = 0.3;
		}
		
		// CREATE MONITOR OBJECT FOR LEARNING
		Monitor monitor = new Monitor();
		monitor.setLearningRate(learningRate);		// Set learning rate.
		monitor.setMomentum(momentum);				// Set momentum.
		this.setMonitor(monitor);
		monitor.addNeuralNetListener(this);
		
		// CREATE INPUT LAYER
		LinearLayer input = new LinearLayer();				// Input layer.
		input.setRows(numInNodes);							// Set the number of input nodes.
		input.setMonitor(monitor);							// Set the monitor for this layer for learning.
		FullSynapse synapse_IH = new FullSynapse();			// Synapse to connect input layer to first hidden layer.
		input.addOutputSynapse(synapse_IH);					// Connect the input layer to the input-hidden synapse.
		FullSynapse synapse_I_dummy = new FullSynapse();	// Add a dummy input syanpse just so an error won't be generated when getOutput is called the first time and tries to remove this synapse.
		input.addInputSynapse(synapse_I_dummy);				// Connect dummy input.
		this.addLayer(input, NeuralNet.INPUT_LAYER);		// Add the input layer to the neural network.
		
		// CREATE OUTPUT LAYER
		SigmoidLayer output = new SigmoidLayer();			// Output layer.
		output.setRows(numOutNodes);						// Set the number of output nodes.
		output.setMonitor(monitor);							// Set the monitor for this layer for learning.
		FullSynapse synapse_HO = new FullSynapse();			// Synapse to connect last hidden layer to output layer.
		output.addInputSynapse(synapse_HO);					// Connect the output layer to the hidden-output synapse.
		FullSynapse synapse_O_dummy = new FullSynapse();	// Add a dummy output syanpse just so an error won't be generated when getOutput is called the first time and tries to remove this synapse.
		output.addOutputSynapse(synapse_O_dummy);			// Connect dummy output.
		this.addLayer(output, NeuralNet.OUTPUT_LAYER);		// Add the output layer to the neural network.

		// CREATE HIDDEN LAYERS
			// CREATE ONE LAYER AND CONNECT IT
		SigmoidLayer pointer = new SigmoidLayer();			// Create 1 hidden layer.
		pointer.setRows(numHiddenNodes[0]);					// Set the number of nodes is this layer.
		pointer.setMonitor(monitor);						// Set the monitor for this layer for learning.
		pointer.addInputSynapse(synapse_IH);				// Connect the hidden layer to input layer.
		this.addLayer(pointer, NeuralNet.HIDDEN_LAYER);		// Store this hidden layer in the neural network.
		hidden = pointer;	// this is a band-aid fix for the save/restore problem

			// CREATE MORE LAYERS
		int num_hidden_layers = numHiddenNodes.length;		// Get number of hidden layers.
		for(int i = 1; i < num_hidden_layers; i++)				// Run this loop only if there are more than one hidden layers to be created.
		{
			// CREATE A NEW HIDDEN LAYER AND SYNAPSE
			SigmoidLayer new_hidden_layer = new SigmoidLayer();		// New hidden layer.
			new_hidden_layer.setRows(numHiddenNodes[i]);			// Set the number of nodes in this layer.
			new_hidden_layer.setMonitor(monitor);					// Set the monitor for this layer for learning.
			FullSynapse new_hidden_synapse = new FullSynapse();		// New input synapse to this layer.
			new_hidden_layer.addInputSynapse(new_hidden_synapse);	// Connect the syanpse to the input of this hidden layer.
			this.addLayer(new_hidden_layer, NeuralNet.HIDDEN_LAYER);	// Store this hidden layer in the neural network.
			
			// CONNECT THIS LAYER TO PREVIOUS LAYER
			pointer.addOutputSynapse(new_hidden_synapse);
			
			// INCREMENT POINTER TO NEW HIDDEN LAYER
			pointer = new_hidden_layer;
		}
		
			// CONNECT THE LAST INPUT LAYER TO THE OUTPUT LAYER
		pointer.addOutputSynapse(synapse_HO);
		
		// ADD TEACHER
		TeachingSynapse teacher = new TeachingSynapse();
		teacher.setMonitor(monitor);
		this.setTeacher(teacher);
		
		// SET ATTRIBUTES OF THIS SUB CLASS
		set_verbose(false);				// Default the network to not print messages during training.
		set_mode_func();				// Default the network to function approximation mode.
		set_stop_epochs();				// Also, default the network to only stop training when the total number of epochs has been reached.
		set_train_error(9999.99);		// Set the training error very high.
		set_valid_error(9999.99);		// Set the validation error very high.
	}
	


// PUBLIC
	// METHODS
		// MUTATORS
			// TRAINING
				// To maximize code reuse, training is split up over many 
				// methods.  The training algorithm and the related methods
				// are:
					// Public Separate Methods: trainTYPE
						// set stopping criteria
						// call private utility method to create synapses
						// call private common training method

					// Private Utility Method: create_mem_in_syn
						// create an input synapse with the training data
							// MemoryInputSynapse
							// FileInputSynapse
	
						// or create an input synapse with the desired outputs
							// MemoryInputSynapse
							// FileInputSynapse

					// Private Common Training Method
						// disconnect input and output synapses
						// connect training data input synapse
						// connect desired output synapse to teacher
						// connect teacher to output 
						// add the neural net listener (do you have to do this each time, or can we do it at construction?)
						// start training
	
	//------------------------------------------------------------------------
	// METHOD:			trainEpochs(double [][] trainData, int totalEpochs).
	// DESCRIPTION:		Trains the neural network.  The only stopping
	//					criterion used is the number of epochs.
	// PRECONDITIONS:	trainData is a 2D array; each row is an input pattern
	//					followed by the desired output(s).  totalEpocs is an 
	//					int; training stops when the number of epochs equals
	//					totalEpochs.
	// POSTCONDITIONS:	The nerual network is trained using the input patterns
	//					and desired outputs in trainData.  Training stops when
	//					totalEpocs is reached.
	//------------------------------------------------------------------------
	public void trainEpochs(double [][] trainData, int totalEpochs)
	{
		// CREATE THE INPUT AND DESIRED OUTPUT SYNAPSES
			// FOR CLAIRTY OF CODE, CALCULATE THE FIRST AND LAST COLUMNS OF INPUT AND DESIRED OUTPUTS
		int num_input_nodes = this.getInputLayer().getDimension();		// Get the number of input nodes.
		int num_output_nodes = this.getOutputLayer().getDimension();	// Get the number of output nodes.
		int first_col_input = 1;									// First column of the array that is input.
		int last_col_input = num_input_nodes;						// Last column of the array that is input.
		int first_col_output = num_input_nodes + 1;					// First coumn of the array that is the desired output.
		int last_col_output = num_input_nodes + num_output_nodes;	// Last column of the array that is the desired output.
		
			// CALL THE PRIVATE UTILITY FUNCTION TO CREATE INPUT SYNAPSES
		MemoryInputSynapse input_syn = create_mem_in_syn(trainData, first_col_input, last_col_input);
		MemoryInputSynapse output_syn = create_mem_in_syn(trainData, first_col_output, last_col_output);

		// ABOUT TO CHANGE THE NETWORK, SO WAIT FOR OTHER THREADS TO COMPLETE
		this.join();
		
		// SET STOPPING CRITEIRA TO EPOCHS
		set_stop_epochs();
		
		// TRAIN
		train(input_syn, output_syn, totalEpochs);
		
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			trainEpochs(double [][] trainData, int totalEpochs).
	// DESCRIPTION:		Trains the neural network.  The only stopping
	//					criterion used is the number of epochs.
	// PRECONDITIONS:	trainData is a 2D array; each row is an input pattern
	//					followed by the desired output(s).  totalEpocs is an 
	//					int; training stops when the number of epochs equals
	//					totalEpochs.
	// POSTCONDITIONS:	The nerual network is trained using the input patterns
	//					and desired outputs in trainData.  Training stops when
	//					totalEpocs is reached.
	//------------------------------------------------------------------------
	public void trainEpochs(String trainData, int totalEpochs)
	{
		// CREATE THE INPUT AND DESIRED OUTPUT SYNAPSES
			// FOR CLAIRTY OF CODE, CALCULATE THE FIRST AND LAST COLUMNS OF INPUT AND DESIRED OUTPUTS
		int num_input_nodes = this.getInputLayer().getDimension();		// Get the number of input nodes.
		int num_output_nodes = this.getOutputLayer().getDimension();	// Get the number of output nodes.
		int first_col_input = 1;									// First column of the array that is input.
		int last_col_input = num_input_nodes;						// Last column of the array that is input.
		int first_col_output = num_input_nodes + 1;					// First coumn of the array that is the desired output.
		int last_col_output = num_input_nodes + num_output_nodes;	// Last column of the array that is the desired output.
		
			// CALL THE PRIVATE UTILITY FUNCTION TO CREATE INPUT SYNAPSES
		FileInputSynapse input_syn = create_file_in_syn(trainData, first_col_input, last_col_input);
		FileInputSynapse output_syn = create_file_in_syn(trainData, first_col_output, last_col_output);

		// ABOUT TO CHANGE THE NETWORK, SO WAIT FOR OTHER THREADS TO COMPLETE
		this.join();
		
		// SET STOPPING CRITEIRA TO EPOCHS
		set_stop_epochs();
		
		// TRAIN
		train(input_syn, output_syn, totalEpochs);
		
		return;
	}
	//------------------------------------------------------------------------
	// METHOD:			trainErrorTrain(double [][] trainData, int totalEpochs
	//					double desiredError)
	// DESCRIPTION:		Trains the neural network.  The stopping criteria
	//					used is the number of epochs and the error on the
	//					training set.
	// PRECONDITIONS:	trainData is a 2D array; each row is an input pattern
	//					followed by the desired output(s).  totalEpocs is an 
	//					int representing the total number of epochs.  
	//					desiredError is a double representing the desired
	//					error on the training set.
	// POSTCONDITIONS:	The nerual network is trained using the input patterns
	//					and desired outputs in trainData.  Training stops when
	//					the total number of epochs is reached, or when the 
	//					error on the training set is less than or equal to the
	//					desired error on the training set, or when the
	//					accuracy on the training set is greater than or equal
	//					to the desired accuracy.
	//------------------------------------------------------------------------
	public void trainErrorTrain(double [][] trainData, int totalEpochs, double desiredError)
	{
		// CREATE THE INPUT AND DESIRED OUTPUT SYNAPSES
			// FOR CLAIRTY OF CODE, CALCULATE THE FIRST AND LAST COLUMNS OF INPUT AND DESIRED OUTPUTS
		int num_input_nodes = this.getInputLayer().getDimension();		// Get the number of input nodes.
		int num_output_nodes = this.getOutputLayer().getDimension();	// Get the number of output nodes.
		int first_col_input = 1;									// First column of the array that is input.
		int last_col_input = num_input_nodes;						// Last column of the array that is input.
		int first_col_output = num_input_nodes + 1;					// First coumn of the array that is the desired output.
		int last_col_output = num_input_nodes + num_output_nodes;	// Last column of the array that is the desired output.
		
			// CALL THE PRIVATE UTILITY FUNCTION TO CREATE INPUT SYNAPSES
		MemoryInputSynapse input_syn = create_mem_in_syn(trainData, first_col_input, last_col_input);
		MemoryInputSynapse output_syn = create_mem_in_syn(trainData, first_col_output, last_col_output);

		// ABOUT TO CHANGE THE NETWORK, SO WAIT FOR OTHER THREADS TO COMPLETE
		this.join();
		
		// SET STOPPING CRITERIA TO GIVEN ERROR ON TRAINING SET
		set_mode_func();				// Set the mode to function approximation because error is desired.
		set_stop_train(desiredError);
		
		// TRAIN
		train(input_syn, output_syn, totalEpochs);
		
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			trainErrorValidation(double [][] trainData, double
	//					[][] validationData, int totalEpochs, double 
	//					desiredError)
	// DESCRIPTION:		Trains the neural network.  The stopping criteria
	//					used is the number of epochs and the error on the
	//					validation set.	
	// PRECONDITIONS:	trainData is a 2D array; each row is an input pattern
	//					followed by the desired output(s).  totalEpocs is an 
	//					int representing the total number of epochs.  
	//					desiredError is a double representing the desired
	//					error or accuracy on the training set.
	// POSTCONDITIONS:	The nerual network is trained using the input patterns
	//					and desired outputs in trainData.  Training stops when
	//					the total number of epochs is reached, or when the 
	//					error on the validation set is less than or equal to
	//					the desired error on the validation set.
	//------------------------------------------------------------------------
	public void trainErrorValidation(double [][] trainData, double [][] validData, int totalEpochs, double desiredError)
	{
		// SET STOPPING CRITERIA TO GIVEN ERROR ON VALIDATION SET
		set_mode_func();				// Set the mode of the network to function approximation because error is desired.
		set_stop_valid(desiredError, validData);
		
		// CREATE THE INPUT AND DESIRED OUTPUT SYNAPSES
			// FOR CLAIRTY OF CODE, CALCULATE THE FIRST AND LAST COLUMNS OF INPUT AND DESIRED OUTPUTS
		int num_input_nodes = this.getInputLayer().getDimension();		// Get the number of input nodes.
		int num_output_nodes = this.getOutputLayer().getDimension();	// Get the number of output nodes.
		int first_col_input = 1;									// First column of the array that is input.
		int last_col_input = num_input_nodes;						// Last column of the array that is input.
		int first_col_output = num_input_nodes + 1;					// First coumn of the array that is the desired output.
		int last_col_output = num_input_nodes + num_output_nodes;	// Last column of the array that is the desired output.
		
			// CALL THE PRIVATE UTILITY FUNCTION TO CREATE SYNAPSES FOR THE TRAINING DATA
		MemoryInputSynapse input_syn_train = create_mem_in_syn(trainData, first_col_input, last_col_input);
		MemoryInputSynapse output_syn_train = create_mem_in_syn(trainData, first_col_output, last_col_output);

			// CALL THE PRIVATE UTILITY FUNCTION TO CREATE SYNAPSES FOR THE VALIDATION DATA
		MemoryInputSynapse input_syn_valid = create_mem_in_syn(validData, first_col_input, last_col_input);
		MemoryInputSynapse output_syn_valid = create_mem_in_syn(validData, first_col_output, last_col_output);

		// CREATE THE INPUT AND OUTPUT LEARNING SWITCH
		LearningSwitch input_learning_switch = create_switch(input_syn_train, input_syn_valid);
		LearningSwitch output_learning_switch = create_switch(output_syn_train, output_syn_valid);

		// SET THE NUMBER OF VALIDATION PATTERNS
		this.getMonitor().setValidationPatterns(validData.length);
		
		// TRAIN
		train(input_learning_switch, output_learning_switch, totalEpochs);
		
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			trainErrorValidation(double [][] trainData, double
	//					[][] validationData, int totalEpochs, double 
	//					desiredError)
	// DESCRIPTION:		Trains the neural network.  The stopping criteria
	//					used is the number of epochs and the error on the
	//					validation set.  The data is contained in files.
	// PRECONDITIONS:	trainData is String; it represents the name of a file
	//					of inputs and desired output(s).  validData is a
	//					String; it represents the name of a file of inputs and
	//					outputs.  totalEpocs is an int representing the total 
	//					number of epochs.
	//					desiredError is a double representing the desired
	//					error or accuracy on the training set.
	// POSTCONDITIONS:	The nerual network is trained using the input patterns
	//					and desired outputs in trainData.  Training stops when
	//					the total number of epochs is reached, or when the 
	//					error on the validation set is less than or equal to
	//					the desired error on the validation set.
	//------------------------------------------------------------------------
	public void trainErrorValidation(String trainData, String validData, int totalEpochs, double desiredError)
	{
		// SET STOPPING CRITERIA TO GIVEN ERROR ON VALIDATION SET
		set_mode_func();				// Set the mode of the network to function approximation because error is desired.
		set_stop_valid(desiredError, validData);
		
		// CREATE THE INPUT AND DESIRED OUTPUT SYNAPSES
			// FOR CLAIRTY OF CODE, CALCULATE THE FIRST AND LAST COLUMNS OF INPUT AND DESIRED OUTPUTS
		int num_input_nodes = this.getInputLayer().getDimension();		// Get the number of input nodes.
		int num_output_nodes = this.getOutputLayer().getDimension();	// Get the number of output nodes.
		int first_col_input = 1;									// First column of the array that is input.
		int last_col_input = num_input_nodes;						// Last column of the array that is input.
		int first_col_output = num_input_nodes + 1;					// First coumn of the array that is the desired output.
		int last_col_output = num_input_nodes + num_output_nodes;	// Last column of the array that is the desired output.
		
			// CALL THE PRIVATE UTILITY FUNCTION TO CREATE SYNAPSES FOR THE TRAINING DATA
		FileInputSynapse input_syn_train = create_file_in_syn(trainData, first_col_input, last_col_input);
		FileInputSynapse output_syn_train = create_file_in_syn(trainData, first_col_output, last_col_output);

			// CALL THE PRIVATE UTILITY FUNCTION TO CREATE SYNAPSES FOR THE VALIDATION DATA
		FileInputSynapse input_syn_valid = create_file_in_syn(validData, first_col_input, last_col_input);
		FileInputSynapse output_syn_valid = create_file_in_syn(validData, first_col_output, last_col_output);

		// CREATE THE INPUT AND OUTPUT LEARNING SWITCH
		LearningSwitch input_learning_switch = create_switch(input_syn_train, input_syn_valid);
		LearningSwitch output_learning_switch = create_switch(output_syn_train, output_syn_valid);

		// SET THE NUMBER OF VALIDATION PATTERNS
		this.getMonitor().setValidationPatterns(input_syn_valid.getLastRow());
		
		// TRAIN
		train(input_learning_switch, output_learning_switch, totalEpochs);
		
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			trainAccuracyTrain(double [][] trainData, int
	//					totalEpochs, double desiredAccuracy)
	// DESCRIPTION:		Trains the neural network.  The stopping criteria
	//					used is the number of epochs and the accuracy on the
	//					training set.
	// PRECONDITIONS:	trainData is a 2D array; each row is an input pattern
	//					followed by the desired output(s).  totalEpocs is an 
	//					int representing the total number of epochs.  
	//					desiredError is a double representing the desired
	//					error on the training set.
	// POSTCONDITIONS:	The nerual network is trained using the input patterns
	//					and desired outputs in trainData.  Training stops when
	//					the total number of epochs is reached, or when the 
	//					error on the training set is less than or equal to the
	//					desired error on the training set, or when the
	//					accuracy on the training set is greater than or equal
	//					to the desired accuracy.
	//------------------------------------------------------------------------
	public void trainAccuracyTrain(double [][] trainData, int totalEpochs, double desiredError)
	{
		// CREATE THE INPUT AND DESIRED OUTPUT SYNAPSES
			// FOR CLAIRTY OF CODE, CALCULATE THE FIRST AND LAST COLUMNS OF INPUT AND DESIRED OUTPUTS
		int num_input_nodes = this.getInputLayer().getDimension();		// Get the number of input nodes.
		int num_output_nodes = this.getOutputLayer().getDimension();	// Get the number of output nodes.
		int first_col_input = 1;									// First column of the array that is input.
		int last_col_input = num_input_nodes;						// Last column of the array that is input.
		int first_col_output = num_input_nodes + 1;					// First coumn of the array that is the desired output.
		int last_col_output = num_input_nodes + num_output_nodes;	// Last column of the array that is the desired output.
		
			// CALL THE PRIVATE UTILITY FUNCTION TO CREATE INPUT SYNAPSES
		MemoryInputSynapse input_syn = create_mem_in_syn(trainData, first_col_input, last_col_input);
		MemoryInputSynapse output_syn = create_mem_in_syn(trainData, first_col_output, last_col_output);

		// ABOUT TO CHANGE THE NETWORK, SO WAIT FOR OTHER THREADS TO COMPLETE
		this.join();
		
		// SET STOPPING CRITERIA TO GIVEN ERROR ON TRAINING SET
		set_mode_func();				// Set the mode to function approximation because error is desired.
		set_stop_train(desiredError);
		
		// TRAIN
		train(input_syn, output_syn, totalEpochs);
		
		return;
	}
	
	public void trainAccuracyTrain(String trainData, int totalEpochs, double desiredError)
	{
		// CREATE THE INPUT AND DESIRED OUTPUT SYNAPSES
			// FOR CLAIRTY OF CODE, CALCULATE THE FIRST AND LAST COLUMNS OF INPUT AND DESIRED OUTPUTS
		int num_input_nodes = this.getInputLayer().getDimension();		// Get the number of input nodes.
		int num_output_nodes = this.getOutputLayer().getDimension();	// Get the number of output nodes.
		int first_col_input = 1;									// First column of the array that is input.
		int last_col_input = num_input_nodes;						// Last column of the array that is input.
		int first_col_output = num_input_nodes + 1;					// First coumn of the array that is the desired output.
		int last_col_output = num_input_nodes + num_output_nodes;	// Last column of the array that is the desired output.
		
			// CALL THE PRIVATE UTILITY FUNCTION TO CREATE INPUT SYNAPSES
		FileInputSynapse input_syn = create_file_in_syn(trainData, first_col_input, last_col_input);
		FileInputSynapse output_syn = create_file_in_syn(trainData, first_col_output, last_col_output);

		// ABOUT TO CHANGE THE NETWORK, SO WAIT FOR OTHER THREADS TO COMPLETE
		this.join();
		
		// SET STOPPING CRITERIA TO GIVEN ERROR ON TRAINING SET
		set_mode_func();				// Set the mode to function approximation because error is desired.
		set_stop_train(desiredError);
		
		// TRAIN
		train(input_syn, output_syn, totalEpochs);
		
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			setVerbose()
	// DESCRIPTION:		Print messages to the screen during training.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	Messages are printed to the screen during training.
	//------------------------------------------------------------------------
	public void setVerbose()
	{
		set_verbose(true);
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			setReticent()
	// DESCRIPTION:		Do not print messages to the screen during training.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	Messages aren't printed to the screen during training.
	//------------------------------------------------------------------------
	public void setReticent()
	{
		set_verbose(false);
		return;
	}
	
// PUBLIC
	// METHODS
		// ACCESSORS
	public int [] classify(double [] input)
	{
	    boolean was_func = is_mode_func();
	    set_mode_class();
	    
	    double [] output_temp = getOutput(input);
	    
	    int [] output = new int [output_temp.length];
	    
	    for(int i = 0; i < output_temp.length; i++)
	    {
	        output[i] = ((int)output_temp[i]);
	    }
	    
	    if(was_func)
	        set_mode_func();
	    
	    return output;
	}
	//------------------------------------------------------------------------
	// METHOD:			getOutput(double [] input)
	// DESCRIPTION:		Provides the output of the network to one input
	//					pattern.
	// PRECONDITIONS:	The passed parameter, input, is an array of doubles.
	//					It represents one input pattern.
	// POSTCONDITOINS:	The output of the network to the passed parameter,
	//					input, is returned as a double [].  This method calls
	//					getOutput(double [][] inputs).
	//------------------------------------------------------------------------
	public double [] getOutput(double [] input)
	{
		// CONVERT SINGLE INPUT TO ARRAY OF ARRAYS WITH ONE ELEMENT
		double [][] inputs = new double [][]
		{
				input
		};
		
		double [][] output = getOutput(inputs);
		return output[0];
	}
	
	//------------------------------------------------------------------------
	// METHOD:			getOutput(double [][] inputs)
	// DESCRIPTION:		Provides the output of the network to all of the input
	//					patterns.
	// PRECONDITIONS:	The passed parameter, inputs, is an array of double
	//					arrays.  It represents many input patterns.  The first
	//					n columns will be considered input, where n is the
	//					number of input nodes in this network, the rest of the
	//					columns will be ignored.  So, desired outputs can be
	//					be included in the data, it will be safely ignored.
	// POSTCONDITOINS:	The output of the network to the passed parameter,
	//					inputs, is returned as a double [][].
	//------------------------------------------------------------------------
	public double [][] getOutput(double [][] inputs)
	{
		// GET THE INPUT AND OUTPUT LAYERS
		Layer output_layer = this.getOutputLayer();			// Get the output layer.
		Layer input_layer = this.getInputLayer();			// Get the input layer.

		// SETUP INPUT - CREATE MEMORY SYNAPSE
			// FOR CLAIRTY OF CODE, CALCULATE THE FIRST AND LAST COLUMNS OF INPUT AND DESIRED OUTPUTS
		int num_input_nodes = this.getInputLayer().getDimension();		// Get the number of input nodes.
		int first_col_input = 1;									// First column of the array that is input.
		int last_col_input = num_input_nodes;						// Last column of the array that is input.

			// CALL THE PRIVATE UTILITY FUNCTION TO CREATE INPUT SYNAPSES
		MemoryInputSynapse input_syn = create_mem_in_syn(inputs, first_col_input, last_col_input);
//		MemoryInputSynapse output_syn = create_mem_in_syn(trainData, first_col_output, last_col_output);

		/*		MemoryInputSynapse memIn = new MemoryInputSynapse();	// Create a memory input synapse.
		memIn.setInputArray(inputs);							// Set the input data.	

			// BUILD COLUMN SELECTOR STRING
		String selector_string = "1";							// This string is used define which columns of the input matrix are inputs
		for(int i = 2; i <= num_input_nodes; i++)
		{
			selector_string = selector_string + ", " + i;
		}

		memIn.setAdvancedColumnSelector(selector_string);		// Define which inputs are inputs (all of them).
*/
		// THE NETWORK IS ABOUT TO BE CHANGED, SO WAIT FOR THREADS TO STOP
		//this.join();

		// REMOVE THE INPUT AND OUTPUT SYNAPSES
		input_layer.removeAllInputs();
		output_layer.removeAllOutputs();
		
		// ADD INPUT
		input_layer.addInputSynapse(input_syn);						// Connect it.
		
		// ADD OUTPUT
		MemoryOutputSynapse output_syn = new MemoryOutputSynapse();	// Create a memory output synapse.
		output_layer.addOutputSynapse(output_syn);				// Connect it.

		
		// SEND INPUT
		this.getMonitor().setTrainingPatterns(inputs.length);
		this.getMonitor().setTotCicles(1);
		this.getMonitor().setLearning(false);
		this.start();
		this.getMonitor().Go();

		// READ THE OUTPUT
		double[][] output = new double [inputs.length][output_layer.getDimension()];
		
		for(int i = 0; i < output.length; i++)
		{
			output[i] = output_syn.getNextPattern();

			// IF THE MODE IS CLASSIFICATION, ROUND OUTPUT TO 1 OR 0
			if(is_mode_class())
			{
				for(int j = 0; j < output[0].length; j++)
				{
					output[i][j] = (double)Math.round(output[i][j]);
				}
			}
		}

		
		return output;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			getAccuarcy(double [][] testData)
	// DESCRIPTION:		Get the accuarcy of the network at classifying the
	//					given test data.
	// PRECONDITONS:	The parameter testData is a 2D array of doubles.
	//					Each row is an input pattern followed by the desired
	//					output of the network.  The outputs must be 1's or 0's
	//					else an accuracy of -1.0 will be returned.
	// POSTCONDITIONS:	The percent of input patterns correctly classified
	//					by the network is returned as a double.
	//------------------------------------------------------------------------
	public double getAccuracy(double [][] testData)
	{
		// ACCURACY IMPLIES CLASSIFICATION, CHANGE MODE IF NEEDED
		boolean mode_was_func = is_mode_func();
		set_mode_class();
		
		// GET THE OUTPUT OF THE NETWORK FOR testData
		double [][] output = getOutput(testData);		// We can send testData because getOutput safely ignores desired outputs included with inputs.

		// FIND START OF DESIRED OUTPUTS
		int start_output = this.getInputLayer().getDimension();

		// COMPARE OUTPUT TO DESIRED OUTPUT, COUNT CORRECT CLASSIFICATIONS
		boolean correct;						// Used to check correctness of all output nodes.
		int number_correct = 0;						// Number of correctly classified inputs.
		for(int i = 0; i < output.length; i++)							// For each input pattern,
		{
			correct = true;												// start by assuming output is correct,
			for(int j = 0; (correct) && (j < output[0].length); j++) 	// then check correctness of each output node.
			{
				if(output[i][j] != testData[i][start_output + j])		// If output != desired output,
				{
					correct = false;									// then set correct to false which will stop the inner for loop.
				}
			}
			
			if(correct)													// If all the output nodes are correct,
			{
				number_correct++;										// add 1 to the count.
			}
		}
		
		// CHANGE MODE BACK TO FUNC IF IT WAS CHANGED AT THE TOP OF THIS METHOD
		if(mode_was_func)
		{
			set_mode_func();
		}
		
		// RETURN VALUE
		return (double)(number_correct / testData.length);				// Return the number of correctly classified inputs / total number inputs.
	}
	
	//------------------------------------------------------------------------
	// METHOD:			getTrainError()
	// DESCRIPTION:		Accessor for the training error of the network.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	The error on the training set of the network after the
	//					last training session is returned.  If the network
	//					has never been trained, a very high number, set
	//					during object construction, is returned.
	//------------------------------------------------------------------------
	public double getTrainError()
	{
		return get_train_error();
	}
	
	//------------------------------------------------------------------------
	// METHOD:			getValidError()
	// DESCRIPTION:		Accessor for the validation error of the network.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	The error on the validation set of the network after
	//					the last training session is returned.  If the network
	//					has never been trained, a very high number, set
	//					during object construction, is returned.
	//------------------------------------------------------------------------
	public double getValidError()
	{
		return get_valid_error();
	}
	
	public void saveMLP(String fileName)
	{
	    try
	    {
	        FileOutputStream stream = new FileOutputStream(fileName);
	        ObjectOutputStream out = new ObjectOutputStream(stream);
	        out.writeObject(this);
	        out.close();
	    }
	    catch (Exception excp)
	    {
	        excp.printStackTrace();
	    }
	}
	
	public void saveNN(String fileName)
	{
	    try
	    {
	        FileOutputStream stream = new FileOutputStream(fileName);
	        ObjectOutputStream out = new ObjectOutputStream(stream);
	        //out.writeObject(this.cloneNet());
	        out.writeObject(this.getInputLayer());
	        out.writeObject(this.hidden);
	        out.writeObject(this.getOutputLayer());
	        
	        out.close();
	    }
	    catch (Exception excp)
	    {
	        excp.printStackTrace();
	    }
	}
	
	public static void restoreMLP(String fileName, MultiLayerPerceptron mlp)
	{
	    try
	    {
	        FileInputStream stream = new FileInputStream(fileName);
	        ObjectInputStream in = new ObjectInputStream(stream);
	        //mlp = null;
	        mlp = (MultiLayerPerceptron)in.readObject();
	    }
	    catch (Exception excp)
	    {
	        System.out.println("error");
	        excp.printStackTrace();
	    }
	    
	    mlp.getMonitor().addNeuralNetListener(mlp);
	    mlp.start();
	    mlp.getMonitor().Go();
	    
	}
	
	public void restoreNN(String fileName)
	{
	    Layer inp = new LinearLayer();
	    Layer hid = new SigmoidLayer();
	    Layer out = new SigmoidLayer();
	    
	    try
	    {
	        FileInputStream stream = new FileInputStream(fileName);
	        ObjectInputStream in = new ObjectInputStream(stream);
	        //nn = null;
	        inp = (Layer)in.readObject();
	        hid = (Layer)in.readObject();
	        out = (Layer)in.readObject();
	    }
	    catch (Exception excp)
	    {
	        System.out.println("error");
	        excp.printStackTrace();
	    }
	    
	    // CREAT SYNAPSES
/*		FullSynapse synapse_IH = new FullSynapse();			// Synapse to connect input layer to first hidden layer.
		inp.addOutputSynapse(synapse_IH);					// Connect the input layer to the input-hidden synapse.
		FullSynapse synapse_I_dummy = new FullSynapse();	// Add a dummy input syanpse just so an error won't be generated when getOutput is called the first time and tries to remove this synapse.
		inp.addInputSynapse(synapse_I_dummy);	
		
		FullSynapse synapse_HO = new FullSynapse();			// Synapse to connect last hidden layer to output layer.
		out.addInputSynapse(synapse_HO);					// Connect the output layer to the hidden-output synapse.
		FullSynapse synapse_O_dummy = new FullSynapse();	// Add a dummy output syanpse just so an error won't be generated when getOutput is called the first time and tries to remove this synapse.
		out.addOutputSynapse(synapse_O_dummy);				// Connect dummy output.

		hid.addInputSynapse(synapse_IH);			// Connect the syanpse to the input of this hidden layer.
		hid.addOutputSynapse(synapse_HO);
*/
	    
	    this.addLayer(inp, NeuralNet.INPUT_LAYER);
	    this.addLayer(hid, NeuralNet.HIDDEN_LAYER);
	    this.addLayer(out, NeuralNet.OUTPUT_LAYER);
	    this.setMonitor(inp.getMonitor());
	    this.getMonitor().removeAllListeners();
	    this.getMonitor().addNeuralNetListener(this);
	    //this.start();
	    //this.getMonitor().Go();
	    
		// SET ATTRIBUTES OF THIS SUB CLASS
		set_verbose(false);				// Default the network to not print messages during training.
		set_mode_func();				// Default the network to function approximation mode.
		set_stop_epochs();				// Also, default the network to only stop training when the total number of epochs has been reached.
		set_train_error(9999.99);		// Set the training error very high.
		set_valid_error(9999.99);		// Set the validation error very high.
	}
	

	
// PUBLIC
	// METHODS
		// METHODS NEEDED TO IMPLEMENT NeuralNetListener
	public void netStarted(NeuralNetEvent arg0)
	{
	}

	public void cicleTerminated(NeuralNetEvent arg0)
	{
		if(is_verbose())
		{	
			// PRINT OUT EVERY 500 CYCLES
			Monitor monitor = (Monitor)arg0.getSource();
			long c = monitor.getCurrentCicle();
			if (c % 500 == 0)
			{
				System.out.println(c + " epochs remaining - training error = " + monitor.getGlobalError() + " - Learning = " + monitor.isLearning());
			}
		}
	}

	public void netStopped(NeuralNetEvent arg0)
	{
		if(is_verbose())
		{
			Monitor monitor = (Monitor)arg0.getSource();
			System.out.println("Net stopped with a training error of " + monitor.getGlobalError());

		}
	}

	// STOPPING CRITERIA ARE HANDLED IN THIS METHOD
	public void errorChanged(NeuralNetEvent arg0)
	{
		// GET THE MONITOR OF THIS OBJECT
        Monitor monitor = (Monitor)arg0.getSource();
        
        // CHECK FOR STOPPING CRITERIA
        	// IS THE ERROR ON THE TRAINING SET A STOPPING CRITERIA?
        if(is_stop_train() && is_mode_func())
        {
        	if(monitor.getGlobalError() <= get_stop_error())
        	{
        		if(is_verbose())
        		{
        			System.out.println("Net stopped on epoch " + monitor.getCurrentCicle());
        		}
        		
        		monitor.Stop();
        	}
        }
        
        	// IS THE ERROR ON THE VALIDATION SET A STOPPING CRITERIA?
        	// IF SO, VALIDATE ONLY 10% OF THE TIME
		int current_cicle = monitor.getCurrentCicle();
		int print_cicle = Math.round((float)(monitor.getTotCicles() * 0.10));

		if(is_stop_valid() && is_mode_func() && (current_cicle % print_cicle == 0))
		{
				// CREAT A COPY OF NEURAL NETWORK FOR VALIDATION
			monitor.setExporting(true);
			NeuralNet clonedNet = this.cloneNet();
			monitor.setExporting(false);
			
				// TO AVOID VALIDATION CALLING CICLE TERMINIATED METHOD REMOVE LISTENERS
			clonedNet.removeAllListeners();
			
				// VALIDATE
			Monitor clonedMonitor = clonedNet.getMonitor();
			clonedMonitor.setTotCicles(1);
			clonedMonitor.setValidation(true);
			clonedMonitor.setLearning(false);
			clonedNet.start();
			clonedMonitor.Go();
			
				// GET VALIDATION ERROR
			clonedNet.join();	// Wait for validation to stop.
			double valid_error = clonedMonitor.getGlobalError();

				// PRINT OUT VALIDATION ERROR IF VERBOSE

			if(is_verbose())
			{
				System.out.println(current_cicle + " epochs remaining - validation error = " + valid_error);
			}
				// TEST FOR STOPPING CONDITIONS
			if(valid_error <= get_stop_accuracy())
			{
        		if(is_verbose())
        		{
        			System.out.println("Net stopped on epoch " + monitor.getCurrentCicle());
        		}
        		
				monitor.Stop();
			}
		}
	}

	public void netStoppedError(NeuralNetEvent arg0, String arg1)
	{
	}
	
	public void netValidated(NeuralValidationEvent event)
	{
		// GET THE EVENT
		NeuralNet NN = (NeuralNet)event.getSource();
		
		// IF is_verbose() PRINT OUT VALIDATION ERROR EVERY 500 CYCLES
		if(is_verbose())
		{	
			System.out.println("Validation Error = " + NN.getMonitor().getGlobalError());

		}
	}

// PRIVATE
	// METHODS
		// UTILITIES
	//------------------------------------------------------------------------
	// METHOD:			create_mem_in_syn(double [][] data, int firstCol, int
	//					lastCol)
	// DESCRIPTION:		This method creates a memory input synapse that can
	//					be connected to any layer.
	// PRECONDITIONS:	inputData is a 2D array of double representing the
	//					data that will be used as input.  firstCol is an int
	//					representing the first coloumn of the data to be used
	//					as input.  lastCol is an int representing the last
	//					column of the data to be used as input
	// POSTCONDITIONS:	A MemoryInputSynase object is returned.
	//------------------------------------------------------------------------
	private MemoryInputSynapse create_mem_in_syn(double [][] data, int firstCol, int lastCol)
	{
		// CONSTRUCT ADVANCED COLUMN SELECTOR STRING
		String selector_string = String.valueOf(firstCol);				// This string is used define which columns of the data are inputs.
		for(int i = firstCol + 1; i <= lastCol; i++)
		{
			selector_string = selector_string + ", " + i;
		}
		
		// SETUP INPUT
		MemoryInputSynapse input_synapse = new MemoryInputSynapse();	// Create a memory input synapse.
		input_synapse.setInputArray(data);								// Set the input array.
		input_synapse.setAdvancedColumnSelector(selector_string);		// Define which columns of the array are input.
		input_synapse.setFirstRow(1);
		input_synapse.setLastRow(data.length);

		return input_synapse;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			create_mem_in_syn(double [][] data, int firstCol, int
	//					lastCol)
	// DESCRIPTION:		This method creates a memory input synapse that can
	//					be connected to any layer.
	// PRECONDITIONS:	inputData is a 2D array of double representing the
	//					data that will be used as input.  firstCol is an int
	//					representing the first coloumn of the data to be used
	//					as input.  lastCol is an int representing the last
	//					column of the data to be used as input
	// POSTCONDITIONS:	A MemoryInputSynase object is returned.
	//------------------------------------------------------------------------
	private FileInputSynapse create_file_in_syn(String data_file, int firstCol, int lastCol)
	{
		// CONSTRUCT ADVANCED COLUMN SELECTOR STRING
		String selector_string = String.valueOf(firstCol);				// This string is used define which columns of the data are inputs.
		for(int i = firstCol + 1; i <= lastCol; i++)
		{
			selector_string = selector_string + ", " + i;
		}
		
		// SETUP INPUT
		FileInputSynapse input_synapse = new FileInputSynapse();
		input_synapse.setFileName(data_file);
		input_synapse.setAdvancedColumnSelector(selector_string);

		return input_synapse;
	}
	
	//-----------------------------------------------------------------------
	// METHOD:			create_switch(StreamInputSynapse input_train_synapse,
	//					StreamInputSynapse input_valid_synapse)
	// DESCRIPTION:		This is a utility function that creates a learning
	//					switch to use while validating a network.
	// PRECONDITONS:	input_train_synapse is a StreamInputSynapse with the
	//					training data set.  input_valid_synapse is a 
	//					StreamInputSynapse with the validation data.
	// POSTCONDITIONS:	A LearningSwitch is retunred connected to both the
	//					training data and the validation data.  It can be
	//					connected to an input layer as an input syanpse.
	//-----------------------------------------------------------------------
	private LearningSwitch create_switch(StreamInputSynapse input_train_synapse, StreamInputSynapse input_valid_synapse)
	{
		LearningSwitch learning_switch = new LearningSwitch();
		learning_switch.addTrainingSet(input_train_synapse);
		learning_switch.addValidationSet(input_valid_synapse);
		return learning_switch;
	}
	
// PRIVATE
	// METHODS
		// MUTATORS
	//------------------------------------------------------------------------
	// METHOD:			train(StreamInputSynapse input_synapse, 
	//					StreamInputSynapse output_synapse, int total_epochs)
	// DESCRIPTION:		This is the general training algorithm that all
	//					public training methods call.
	// PRECONDITIONS:	input_synapse is a StreamInputSynapse object; it
	//					represents the training input data.  output_synpase is
	//					a StreamInputSynpase; it represents the desired
	//					outputs for the training data.  total_epochs is an
	//					int; it represents the total number of epochs to
	//					train.
	// POSTCONDITIONS:	The network is trained using the traing data.
	//					The stopping criteria for training is determined by
	//					the private attribute stopping_criteria and is 
	//					handled by the cicleTermindate method; the method
	//					that calls this method should use the set_stop_xxxx()
	//					methods to set the stopping criteria.
	//------------------------------------------------------------------------
	private void train(StreamInputSynapse input_synapse, StreamInputSynapse output_synapse, int total_epochs)
	{
		// GET THE INPUT AND OUTPUT LAYERS
		Layer input_layer = this.getInputLayer();
		Layer output_layer = this.getOutputLayer();
		
		// THE NETWORKED IS ABOUT TO BE CHANGED, SO WAIT
		this.join();	// Wait for the network to finish any other threads.

		// REMOVE INPUT AND OUTPUT SYNAPSES
		input_layer.removeAllInputs();
		output_layer.removeAllOutputs();
		
		// CONNECT INPUT TO NETWORK
		input_layer.addInputSynapse(input_synapse);	
		
		// GET TEACHER
		TeachingSynapse teacher = this.getTeacher();
	
		// CONNECT DESIRED OUPUT TO TEACHER
		teacher.setDesired(output_synapse);

		// SET TEARCHER AS OUTPUT
		output_layer.addOutputSynapse(teacher);

		// GET THE MONITOR
		Monitor monitor = this.getMonitor();
		
		// TRAIN
		monitor.setTrainingPatterns(144);	// Set the number of input patterns.
		System.out.println("  " + monitor.getValidationPatterns());
		monitor.setLearning(true);									// Yes, we want to train.
		monitor.setTotCicles(total_epochs);							// Set the number of epochs to train.
		this.start();												// Start each layer's thread.
		monitor.Go();												// Start training.
		
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			set_verbose(boolean new_verbose)
	// DESCRIPTION:		This is the mutator for the private attribute verbose.
	// PRECONDITIONS:	new_verbose is a boolean.
	// POSTCONDITIONS:	The private field verbose is set to the value of 
	//					new_verbose.  Setting to true causes messages to be
	//					printed during traing.
	//------------------------------------------------------------------------
	private void set_verbose(boolean new_verbose)
	{
		verbose = new_verbose;
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			set_mode_func()
	// DESCRIPTION:		This is one function that can set the private 
	//					attribute function_approximation.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	The private attribute function_approx is set to true.
	//					This puts the network in function approximation mode.
	//
	//					The mode affects what the network output and how it
	//					trains.  In function approximation mode, the network 
	//					outputs the exact value of its output nodes.  During
	//					training, the network will try to exactly match the
	//					desired output.  In classifcation mode, the network
	//					will round its output to the nearest 1 or 0.  This
	//					will reduce training as the output nodes will not have
	//					to match the desrired output exactly.  This mode may
	//					be useful for operations other than classification,
	//					that is just the name I gave it.
	//------------------------------------------------------------------------
	private void set_mode_func()
	{
		function_approximation = true;
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			set_mode_class()
	// DESCRIPTION:		This is one function that can set the private 
	//					attribute function_approximation.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	The private attribute function_approx is set to false.
	//					This puts the network in classification mode.
	//
	//					The mode affects what the network output and how it
	//					trains.  In function approximation mode, the network 
	//					outputs the exact value of its output nodes.  During
	//					training, the network will try to exactly match the
	//					desired output.  In classifcation mode, the network
	//					will round its output to the nearest 1 or 0.  This
	//					will reduce training as the output nodes will not have
	//					to match the desrired output exactly.  This mode may
	//					be useful for operations other than classification,
	//					that is just the name I gave it.
	//------------------------------------------------------------------------
	private void set_mode_class()
	{
		function_approximation = false;
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			set_stop_epochs()
	// DESCRIPTION:		Mutator for private attribute stopping_criteria.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	The value of the private attribute stopping_criteria
	//					is set to EPOCHS.  Training will stop when the total
	//					number of EPOCHS is reached.
	//------------------------------------------------------------------------
	private void set_stop_epochs()
	{
		stopping_criteria = EPOCHS;
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			set_stop_train()
	// DESCRIPTION:		Mutator for private attribute stopping_criteria.
	// PRECONDITIONS:	new_stopping_value is a double; it is either the 
	//					desired error or accuracy on the training set:
	//					if is_mode_func() then new_stopping_value = error,
	//					if is mode_class() then new_stopping_value = accuracy.
	// POSTCONDITIONS:	The value of the private attribute stopping_criteria
	//					is set to TRAIN.  The network's performance on the
	//					training set will be used to stop training, in
	//					addition to the total number of epochs.  The private
	//					attribute stopping_value is set to new_stopping_value.
	//------------------------------------------------------------------------
	private void set_stop_train(double new_stopping_value)
	{
		stopping_criteria = TRAIN;
		stopping_value = new_stopping_value;
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			set_stop_valid(double new_stopping_value)
	// DESCRIPTION:		Mutator for private attribute stopping_criteria.
	// PRECONDITIONS:	new_stopping_value is a double; it is either the
	//					desired error or accuracy on the validation set:
	//					if is_mode_func() then new_stopping_value = error,
	//					if is mode_class() then new_stopping_value = accuracy.
	//					new_validation_data is a 2D array of doubles; it is 
	//					the validation set.
	// POSTCONDITIONS:	The value of the private attribute stopping_criteria
	//					is set to VALID.  The network's performance on the 
	//					validation set will be used to stop training, in
	//					addition to the total number of epochs.  The private
	//					attribute stopping_value is set to new_stopping_value.
	//					the private attribute validation_data is set to
	//					new_validation_data
	//------------------------------------------------------------------------
	private void set_stop_valid(double new_stopping_value, double [][] new_validation_data)
	{
		stopping_criteria = VALID;
		stopping_value = new_stopping_value;
		validation_data = new_validation_data;
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			set_stop_valid(double new_stopping_value)
	// DESCRIPTION:		Mutator for private attribute stopping_criteria.
	// PRECONDITIONS:	new_stopping_value is a double; it is either the
	//					desired error or accuracy on the validation set:
	//					if is_mode_func() then new_stopping_value = error,
	//					if is mode_class() then new_stopping_value = accuracy.
	//					new_validation_data is a 2D array of doubles; it is 
	//					the validation set.
	// POSTCONDITIONS:	The value of the private attribute stopping_criteria
	//					is set to VALID.  The network's performance on the 
	//					validation set will be used to stop training, in
	//					addition to the total number of epochs.  The private
	//					attribute stopping_value is set to new_stopping_value.
	//					the private attribute validation_data is set to
	//					new_validation_data
	//------------------------------------------------------------------------
	private void set_stop_valid(double new_stopping_value, String new_validation_data_file)
	{
		stopping_criteria = VALID;
		stopping_value = new_stopping_value;
		validation_data_file = new_validation_data_file;
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			set_train_error(double new_train_error)
	// DESCRIPTION:		Mutator for private attribute train_error.
	// PRECONDITIONS:	new_train_data is a double; it represents the error
	//					on the training set after the last training session.
	// POSTCONDITIONS:	The private field train_error is set to the value in
	//					new_train_error
	//------------------------------------------------------------------------
	private void set_train_error(double new_train_error)
	{
		train_error = new_train_error;
		return;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			set_valid_error(double new_valid_error)
	// DESCRIPTION:		Mutator for private attribute valid_error.
	// PRECONDITIONS:	new_valid_data is a double; it represents the error
	//					on the validation set after the last training session.
	// POSTCONDITIONS:	The private field valid_error is set to the value in
	//					new_valid_error
	//------------------------------------------------------------------------
	private void set_valid_error(double new_valid_error)
	{
		valid_error = new_valid_error;
		return;
	}
		
// PRIVATE	
	//	METHODS				
		// ACCESSORS
	//------------------------------------------------------------------------
	// METHOD:			is_mode_func()
	// DESCRIPTION:		This is one function that can access the private 
	//					attribute function_approximation.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	The private attribute function_approx is returned.
	//					If it is true, the networks current mode of operation
	//					is function approximation.  If false, classification.
	//
	//					The mode affects what the network output and how it
	//					trains.  In function approximation mode, the network 
	//					outputs the exact value of its output nodes.  During
	//					training, the network will try to exactly match the
	//					desired output.  In classifcation mode, the network
	//					will round its output to the nearest 1 or 0.  This
	//					will reduce training as the output nodes will not have
	//					to match the desrired output exactly.  This mode may
	//					be useful for operations other than classification,
	//					that is just the name I gave it.
	//------------------------------------------------------------------------
	private boolean is_mode_func()
	{
		return function_approximation;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			is_mode_class()
	// DESCRIPTION:		This is one function that can access the private 
	//					attribute function_approximation.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	The opposite of the private attribute function_approx 
	//					is returned.  If it is true, the networks current mode
	//					of operation is classification.  If false, function 
	//					approximation.
	//
	//					The mode affects what the network output and how it
	//					trains.  In function approximation mode, the network 
	//					outputs the exact value of its output nodes.  During
	//					training, the network will try to exactly match the
	//					desired output.  In classifcation mode, the network
	//					will round its output to the nearest 1 or 0.  This
	//					will reduce training as the output nodes will not have
	//					to match the desrired output exactly.  This mode may
	//					be useful for operations other than classification,
	//					that is just the name I gave it.
	//------------------------------------------------------------------------
	private boolean is_mode_class()
	{
		return !function_approximation;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			is_stop_epochs()
	// DESCRIPTION:		Accessor for private attribute stopping_criteria.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	A boolean is returned.  When this function returns
	//					true, the network only stops training when the max
	//					number of epochs have been reached.
	//------------------------------------------------------------------------
	private boolean is_stop_epochs()
	{
		return stopping_criteria == EPOCHS;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			is_stop_train()
	// DESCRIPTION:		Accessor for private attribute stopping_criteria.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	A boolean is returned.  When this function returns
	//					true, the network's performance on the training set is
	//					used as a stopping criterion for training.
	//------------------------------------------------------------------------
	private boolean is_stop_train()
	{
		return stopping_criteria == TRAIN;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			is_stop_valid()
	// DESCRIPTION:		Accessor for private attribute stopping_criteria.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	A boolean is returned.  When this function returns
	//					true, the network's performance on a validation set is
	//					used as a stopping criterion for training.
	//------------------------------------------------------------------------
	private boolean is_stop_valid()
	{
		return stopping_criteria == VALID;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			is_verbose()
	// DESCRIPTION:		Accessor for private attbribute verbose;
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	A boolean is returned;  True is returned if comments
	//					during training should be printed to the screen, else
	//					false is returned.
	//------------------------------------------------------------------------
	private boolean is_verbose()
	{
		return verbose;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			get_stop_error()
	// DESCRIPTION:		Accessor for private attribute stopping_value.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	A double is returned; 
	//					if is_mode_class() and is_stop_train() then stop train
	//					when accuracy on train set >= get_stop_value().
	//					if is_mode_class() and is_stop_valid() then stop train
	//					when accuracy on validation set >= get_stop_value().
	//------------------------------------------------------------------------
	private double get_stop_error()
	{
		return stopping_value;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			get_stop_accuracy()
	// DESCRIPTION:		Accessor for private attribute stopping_value.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	A double is returned; 
	//					if is_mode_class() and is_stop_train() then stop train
	//					when accuracy on train set >= get_stop_value().
	//					if is_mode_class() and is_stop_valid() then stop train
	//					when accuracy on validation set >= get_stop_value().
	//------------------------------------------------------------------------
	private double get_stop_accuracy()
	{
		return stopping_value;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			get_train_error()
	// DESCRIPTION:		Accessor for private attribute train_error.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	A double is returned; 
	//					The training error at the end of the last training
	//					session is returned.
	//------------------------------------------------------------------------
	private double get_train_error()
	{
		return train_error;
	}
	
	//------------------------------------------------------------------------
	// METHOD:			get_valid_error()
	// DESCRIPTION:		Accessor for private attribute valid_error.
	// PRECONDITIONS:	None.
	// POSTCONDITIONS:	A double is returned; 
	//					The validation error at the end of the last training
	//					session is returned.
	//------------------------------------------------------------------------
	private double get_valid_error()
	{
		return valid_error;
	}
	
// PRIVATE
	// ATTRIBUTES
	private boolean verbose;				// When true, the network prints out message about traing.
	
	private boolean function_approximation;	// When true, the network is in the function approximation mode.
											// When false, the network is in the classification mode.
											// In func approx mode: actual outputs are returned, performance 
											// of the network is measured by error, and training is stopped
											// when the error on the training or validation set (determined by
											// stopping_criteria) is <= stopping_value.  In class mode:
											// outputs are rounded to 1 or 0, performance of the network is
											// measured by accuracy, and training is stopped when the accuracy
											// on the training or validation set (determined by stopping_criteria)
											// is >= stopping_value.

	private int stopping_criteria;			// Detrmines stopping criteria for training.
	private static int EPOCHS = 0;			// 0 = train until total epochs
	private static int TRAIN = 1;			// 1 = train until desired error or desired accuracy on training set or total epochs
	private static int VALID = 2;			// 2 = train until desired error or desired accuracy on validation set or total epochs

	private double stopping_value;			// This attribute is used to stop training.  It is the desired error or accuracy on the training or validation set.

	private double [][] validation_data;	// This is the validaion data set.
	private String validation_data_file;	// This is the name of the file that holds the validation set.
	
	private double train_error;				// Holds the latest training error of the network.
	private double valid_error;				// Holds the latest validation error of the network.
	private Layer hidden;
}
