/*
 * Created on Jan 25, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package yaes.application.eel6938.game;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Properties;

import yaes.application.eel6938.classagents.UserInputAgent;
import yaes.application.eel6938.game.cameras.Camera;
import yaes.application.eel6938.game.gui.VisualMap;
import yaes.application.eel6938.game.gui.painters.AgentPainter;
import yaes.application.eel6938.game.gui.painters.EnergyPainter;
import yaes.application.eel6938.game.gui.painters.MapPainter;
import yaes.application.eel6938.game.objects.Agent;
import yaes.application.eel6938.game.objects.Energy;
import yaes.application.eel6938.game.objects.ObjectController;
import yaes.application.eel6938.game.objects.Map;
import yaes.application.eel6938.game.statistics.StatisticsAnalyzer;
import yaes.application.eel6938.game.statistics.StatisticsWriter;
import yaes.application.eel6938.game.userinputs.UserInput;
import yaes.application.eel6938.game.userinputs.actions.AgentAttackAction;
import yaes.application.eel6938.game.userinputs.actions.AgentEastAction;
import yaes.application.eel6938.game.userinputs.actions.AgentEatAction;
import yaes.application.eel6938.game.userinputs.actions.AgentMateAction;
import yaes.application.eel6938.game.userinputs.actions.AgentNorthAction;
import yaes.application.eel6938.game.userinputs.actions.AgentSouthAction;
import yaes.application.eel6938.game.userinputs.actions.AgentWestAction;
import yaes.application.eel6938.game.userinputs.actions.CameraDownAction;
import yaes.application.eel6938.game.userinputs.actions.CameraLeftAction;
import yaes.application.eel6938.game.userinputs.actions.CameraRightAction;
import yaes.application.eel6938.game.userinputs.actions.CameraUpAction;
import yaes.application.eel6938.game.userinputs.actions.CameraZoomInAction;
import yaes.application.eel6938.game.userinputs.actions.CamreaZoomOutAction;
import yaes.application.eel6938.game.userinputs.actions.PrintToScreenAction;
import yaes.framework.world.World;

/**
 * @author Linus Luotsinen
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class Main{

	private static boolean m_ExecuteGameLoop = true;
	public static long GameStartTime = 0;	
	
	public static void main(String[] args) 
	{
		Agent.ResetAgentCounter();
		Properties properties = loadProperties(args);
		if(properties != null)
		{
			long startTime = Calendar.getInstance().getTimeInMillis();
			GameStartTime = startTime;
			if(Configuration.AppConsolePrint)
			{
				System.out.println("Game started. One dot represents " + Configuration.AppConsolePrintProgress + " simulation cycles." );	
			}
			Main m = new Main();
			
			//Keep track of agents and energyUnits
			ArrayList <Agent>agents = new ArrayList<Agent>();
			ArrayList <Energy>energyUnits = new ArrayList<Energy>();
			
			//Used for round robin processing of agents.
			int agentIndex = 0;
					
			//Create a world, just for fun!!
			World world = new World();
			
			//Setup visual map and data map from image 
			VisualMap visualMap = new VisualMap(Configuration.AppWindowWidth,
												Configuration.AppWindowHeight, 
												Configuration.AppResourcesPath + Configuration.AppIconPath,
												Configuration.AppDrawGUI);
			visualMap.getFrame().addWindowListener(new WindowAdapter() {
		        public void windowClosing(WindowEvent e) {
		        	m_ExecuteGameLoop = false;
		        }
		    });
			visualMap.getFrame().addKeyListener(new KeyAdapter() {
		        public void keyReleased(KeyEvent e)  {
		        	if(e.getKeyCode() == KeyEvent.VK_ESCAPE)
		        	{
		        		m_ExecuteGameLoop = false;
		        	}
		        }
		    });
				
			Map ground = new Map(Configuration.AppSceneWidth, Configuration.AppSceneHeight);
			
			//Statistics (writer and analyzer)
			StatisticsWriter agentCycleStats = new StatisticsWriter(world, startTime, properties);
            StatisticsAnalyzer agentStaisticAnalyzer = new StatisticsAnalyzer(startTime);
			
            //User input interface
			UserInput userInput = new UserInput(visualMap.getFrame());		
			
			//Setup camera
			Camera camera = new Camera();
			camera.centerAtFullscreen(Configuration.AppWindowWidth,Configuration.AppWindowHeight, (int)ground.getXHigh(), (int)ground.getYHigh());
			camera.zoom(0.9f);	//Zoom out a little bit
			visualMap.AttachCamera(camera); //Attach the camera to map
			
			//Painters
			AgentPainter agentPainter = new AgentPainter();
			EnergyPainter energyPainter = new EnergyPainter();
			MapPainter mapPainter =  new MapPainter();
			
			//Setup game objects
			visualMap.addObject(ground, mapPainter);	//Add ground map to scene
			ground.populateObstacleMap(Configuration.AppResourcesPath + Configuration.AppImagePath);
			List newEnergyUnits = ground.populateEnergyMap(energyUnits, Configuration.EnergyInitialPopulation);	//Creates and adds energy units to map
			if(newEnergyUnits != null)
			{
				visualMap.addObjectList(newEnergyUnits, energyPainter);
			}
			
			//Add agents
			ground.populateAgentList(agents);
			for(int i=0; i<Configuration.Agents.length;i++)
			{
				try {
					Agent newAgent = (Agent)Class.forName(Configuration.AgentClassPath + Configuration.Agents[i]).newInstance();
					agents.add(newAgent);
				} catch (InstantiationException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				} catch (ClassNotFoundException e) {
					e.printStackTrace();
				}
			}
			
			//Randomize round robin index
			agentIndex = (int)(Math.random()*agents.size());
			
			//Setup User Input Mappings
			userInput.MapKeyboardAction(KeyEvent.VK_HOME, new CameraZoomInAction(camera),false);
			userInput.MapKeyboardAction(KeyEvent.VK_END, new CamreaZoomOutAction(camera),false);
			userInput.MapKeyboardAction(KeyEvent.VK_DOWN, new CameraDownAction(camera),false);
			userInput.MapKeyboardAction(KeyEvent.VK_UP, new CameraUpAction(camera),false);
			userInput.MapKeyboardAction(KeyEvent.VK_LEFT, new CameraLeftAction(camera),false);
			userInput.MapKeyboardAction(KeyEvent.VK_RIGHT, new CameraRightAction(camera),false);
			//userInput.MapKeyboardAction(KeyEvent.VK_ESCAPE, new QuitAction(),false);
			
			//Game loop starts here
			for (double time = 1.0; time <= Configuration.AppMaxCycles; time++) {
	            world.setTime(time);
	            
	            //Print console progress
	            if(	Configuration.AppConsolePrint && Configuration.AppConsolePrintProgress > 0 && 
	            	time%Configuration.AppConsolePrintProgress==0)
	            {
	            	System.out.print(".");
	            }
	                   
	            //Add more energy elements?
	    		if(time % Configuration.EnergyPopulationIncreaseStep == 0 && 
	    				energyUnits.size() <= Configuration.EnergyMaxPopulation)
	    		{
	    			newEnergyUnits = ground.populateEnergyMap(energyUnits, Configuration.EnergyPopulationIncrease);
	    			if(newEnergyUnits != null)
	    			{
	    				visualMap.addObjectList(newEnergyUnits, energyPainter);
	    			}
	    		}
	    		
	    		//Delete if no energy in energy object?
	    		if(Configuration.EnergyRemoveWhenEmpty)
	    		{
	    			Energy[] energyArr = new Energy[energyUnits.size()];
	    			energyUnits.toArray(energyArr);
	    			for(int i=0; i< energyArr.length; i++)
	                {
	    				if(energyArr[i].getEnergyLevel() <= Configuration.EnergyMinLevel)
	    				{
	    					ground.removePropertyAt("Energy", ObjectController.getX(energyArr[i]), ObjectController.getY(energyArr[i]));
	    					visualMap.removeObject(energyArr[i]);
	    					energyUnits.remove(energyArr[i]);
	    				}
	                }
	    		}
	    		
	            //Poll for User Input
	            userInput.poll();
	            
	            //Record statistics
	            /*if(Configuration.AppLogAgentData)
	            {
	            	ground.refreshMaps();	//Update agent positions every turn!
	            	agentCycleStats.refresh();
	            }*/
	            
	            //Poll agent activity (in a round robin fashion)
	            Agent[] agentArr = new Agent[agents.size()];
	            agents.toArray(agentArr);
	            for(int i=0; i< agentArr.length; i++)
	            {
	            	if(!ObjectController.isEnabled(agentArr[i])) //If there are non-enabled agent. Then enable it them!
	            	{
	            		ObjectController.enable(agentArr[i], ground);
	            		visualMap.addObject(agentArr[i], agentPainter);
	            		if(agentArr[i] instanceof UserInputAgent)
	            		{
	            			userInput.MapKeyboardAction(KeyEvent.VK_A, new AgentWestAction(agentArr[i]),false);
	    					userInput.MapKeyboardAction(KeyEvent.VK_D, new AgentEastAction(agentArr[i]),false);
	    					userInput.MapKeyboardAction(KeyEvent.VK_S, new AgentSouthAction(agentArr[i]),false);
	    					userInput.MapKeyboardAction(KeyEvent.VK_W, new AgentNorthAction(agentArr[i]),false);
	    					userInput.MapKeyboardAction(KeyEvent.VK_P, new PrintToScreenAction(agentArr[i]),false);
	    					userInput.MapKeyboardAction(KeyEvent.VK_E, new AgentEatAction(agentArr[i]),false);
	    					userInput.MapKeyboardAction(KeyEvent.VK_E, new AgentAttackAction(agentArr[i]),false);
	    					userInput.MapKeyboardAction(KeyEvent.VK_M, new AgentMateAction(agentArr[i]),false);
	            		}
	            	}
	            	ground.refreshMaps();	//Update agent positions every turn!
	            	int selectedAgent = (i+Math.abs(agentIndex))%agentArr.length;
	            	//Record statistics
	                if(Configuration.AppLogAgentData)
	                {
	                	ground.refreshMaps();	//Update agent positions every turn!
	                	agentCycleStats.refresh(agentArr[selectedAgent]);
	                }
	            	ObjectController.process(agentArr[selectedAgent]);
	            }
	            agentIndex++;
	            
	            //Render objects
	            if(Configuration.AppDrawGUI && time%Configuration.AppGUIRepaintRate==0)
	            {
                    int progress = (int)(world.getTime()/((double)Configuration.AppMaxCycles)*100);
                    visualMap.getFrame().setTitle(Configuration.AppName + " " + progress + "%");
	            	visualMap.repaint();
	            }
	            
	            if(Configuration.AppFps > 0)
	            {
		            try {
						//Extremely weak real-time modeling of FPS
						Thread.sleep(Math.round(1000.0f/Configuration.AppFps));
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
	            }
				
				//Break loop if quit command have been invoked.
				if(Main.m_ExecuteGameLoop == false)
				{
					break;
				}
	        }
            
            if(Configuration.AppConsolePrint)
            {
                System.out.println("");
            }
            
	        if(Configuration.AppLogAgentData)
	        {
		    	Agent[] agentArr = new Agent[agents.size()];
		        agents.toArray(agentArr);
		    	for(int i=0; i< agentArr.length; i++)
		        {
		    		agentCycleStats.flushAgentData(agentArr[i]);
		        }
                
                if(Configuration.AppConsolePrint)
                {
                    System.out.print("Analyzing log files... ");
                }
                agentStaisticAnalyzer.analyze(agentArr);
                if(Configuration.AppConsolePrint)
                {
                    System.out.println("Done!");
                }
	        }
	        
	        visualMap.getFrame().dispose();
	        
	        long endTime = Calendar.getInstance().getTimeInMillis();
			if(Configuration.AppConsolePrint)
			{
				System.out.println("Game ended. Total execution time is " + (endTime - startTime)/1000.0f + " seconds.");
            }
		}
	}
	
	public static Properties loadProperties(String [] args)
	{
		Properties ret = null;
		//Initialize Configuration Constants
		/*if(Configuration.AppConsolePrint)
		{
			System.out.println("Loading default configuration. (" + Configuration.AppResourcesPath + Configuration.AppConfigurationPath + ")");
		}
		Properties defaultProperties = Configuration.LoadFromPropertiesFile(Configuration.AppResourcesPath + Configuration.AppConfigurationPath);
		ret = defaultProperties;*/
		//Find config parameter
		String path = null;
		for(int i=0; i<args.length; i++)
		{
			if( (args[i].compareToIgnoreCase("-config") == 0) &&  (i+1 < args.length) )
			{
				path = args[i+1];
				break;
			}
		}
		if(path != null)
		{
			File f = new File(path);
			if(f.exists())
			{
				Properties userProperties = Configuration.LoadFromPropertiesFile(path);
				//Configuration.UpdateConfiguation(userProperties);
				ret = userProperties;
				Configuration.UpdateConfiguation(ret);
				if(Configuration.AppConsolePrint)
				{
					System.out.println("Configuration file loaded. (" + path + ")");
				}
			}
			else
			{
				System.out.println("Error: Configuration file not found. (" + path + ")");
			}
		}
		else
		{
			System.out.println("Error: No configuration file argument found. Please use the '-config \"c:\\your\\config\\file.cfg\"' argument when starting the game.");
		}
		return ret;
	}
}
