/*
 * Created on Jan 27, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package yaes.application.eel6938.game.objects;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Stack;

import yaes.application.eel6938.game.Configuration;
import yaes.application.eel6938.game.Constants;
import yaes.application.eel6938.game.commands.AttackCommand;
import yaes.application.eel6938.game.commands.CommandReturn;
import yaes.application.eel6938.game.commands.EatCommand;
import yaes.application.eel6938.game.commands.FleeCommand;
import yaes.application.eel6938.game.commands.ICommandFunction;
import yaes.application.eel6938.game.commands.MateCommand;
import yaes.application.eel6938.game.commands.MoveCommand;
import yaes.application.eel6938.game.pathplanners.JAStar;
import yaes.application.eel6938.game.pathplanners.Node;
import yaes.framework.world.map.Location;


/**
 * @author Linus Luotsinen, lluotsin@mail.ucf.edu
 * Feb 9, 2005
 * This class provides the basic functionalities of an agent in the Flee-Feed-Mate game.
 * Simply inherit this class when a new agent is to be created for the game. 
 */
public abstract class Agent extends GameObject
{	
	private static int m_IdCounter = 0;
	
	private int m_EnergyLevel;
	private boolean m_Enabled;
	private Map m_Ground;
	private int m_Id;
	private Agent m_Parent = null;
	private boolean m_Attacked = false;
	private Agent m_AttackedBy = null;
	private int m_AttackedByCycles = 0;
	private boolean m_Alive = false;
	private boolean m_CommandSuccessful = false;
	private int m_CommandType;
	private int m_FleeCycles = 0;
	private int m_TotalEnergyIncrease;
	private int m_TotalEnergyDecrease;
	private Agent[][] m_LocalAgentMap;
	private Energy[][] m_LocalEnergyMap;
	private boolean[][] m_LocalObstacleMap;
	private ArrayList<Point> m_LocalAgentList;
	private ArrayList<Point> m_LocalFoodList;
	private ArrayList<Point> m_LocalObstacleList;
	private int[][] m_SensorData;
	private int m_AgentCount;
	private int m_FoodCount;
	private int m_ObstacleCount;
	private ICommandFunction m_Command = null;
	
	private JAStar m_PathPlanner = null;
	
	private static ManhattanDistanceComparator m_ManhattanDistanceComparator = null;
	
	/**
	 * Specifies the user defined agent intelligence, must be implemented by all agents.
	 * This function is called in a round-robin manner in the game. Available actions are shown in the 
	 * list below:
	 * <br><br>
	 * {@link  #eat() eat}<br>
	 * {@link  #eat(int) eat(int)}<br>
	 * {@link  #attack() attack()}<br>
	 * {@link  #attack(int) attack(int)}<br>
	 * {@link  #flee(int) flee(int)}<br>
	 * {@link  #mate() mate()}<br>
	 * {@link  #move(int, int) move(int, int)}<br>
	 * 
	 */
	public abstract void intelligence();
	
	/**
	 * Returns the name of the agent.
	 * Must be implemented by all agents in the game.
	 * @return A string describing the name of the agent.
	 */
	public abstract String getName();
	
	/**
	 * Returns the color of the agent. 
	 * Must be implemented by all agents in the game.
	 * @return An array of three integers descibing the color of the agent. For instance the color red 
	 * is specified by the array {255,0,0}. The default color is black.
	 */
	public abstract int[] getColor();
	
	
	/**
	 * Initializes agent sensors and path-planner.
	 */
	public Agent() {
		
        
		if(Agent.m_ManhattanDistanceComparator == null)
		{
			m_ManhattanDistanceComparator = new ManhattanDistanceComparator(new Point(0,0));
		}
		
		m_Enabled = false;
		m_Alive = false;
		m_CommandSuccessful = false;
		m_CommandType = Constants.CommandNone;
		m_AttackedBy = null;
		m_EnergyLevel = Configuration.AgentInitialEnergyLevel;	//Set energy level
		m_TotalEnergyIncrease = 0;
		m_TotalEnergyDecrease = 0;
		m_Id = m_IdCounter;
		m_IdCounter++;
		
		m_SensorData = new int[Configuration.AgentInitialSensorRange*2+1][Configuration.AgentInitialSensorRange*2+1];
		m_LocalAgentMap = new Agent[Configuration.AgentInitialSensorRange*2+1][Configuration.AgentInitialSensorRange*2+1];
		m_LocalEnergyMap = new Energy[Configuration.AgentInitialSensorRange*2+1][Configuration.AgentInitialSensorRange*2+1];
		m_LocalObstacleMap = new boolean[Configuration.AgentInitialSensorRange*2+1][Configuration.AgentInitialSensorRange*2+1];
		m_LocalAgentList = new ArrayList<Point>();
		m_LocalFoodList = new ArrayList<Point>();
		m_LocalObstacleList = new ArrayList<Point>();
		
		m_PathPlanner = new JAStar();
	}
	
	public static void ResetAgentCounter()
	{
		m_IdCounter = 0;
	}
	private void refreshLocalMaps()
	{
		m_AgentCount = 0;
		m_FoodCount = 0;
		m_ObstacleCount = 0;
		
		m_LocalAgentList.clear();
		m_LocalFoodList.clear();
		m_LocalObstacleList.clear();
		
		int sensorRange = Configuration.AgentInitialSensorRange;
		int widthSensorRange = Configuration.AgentInitialSensorRange*2+1;

		int fromX = (m_X - (sensorRange));
		int fromY = (m_Y - (sensorRange));
		int toX = (m_X + (sensorRange));
		int toY = (m_Y + (sensorRange));
		
		m_SensorData = new int[Configuration.AgentInitialSensorRange*2+1][Configuration.AgentInitialSensorRange*2+1];
		
		for(int x=0;x<widthSensorRange;x++)
		{
			for(int y=0;y<widthSensorRange;y++)
			{
				if(		(x+fromX)>=0 && 
						(x+fromX)<(m_Ground.getXHigh()) && 
						(y+fromY)>=0 && 
						(y+fromY)<(m_Ground.getYHigh()) )	//If inside map
				{
					m_LocalAgentMap[x][y] = (Agent)m_Ground.getPropertyAt("Agent", x+fromX, y+fromY);
					m_LocalEnergyMap[x][y] = (Energy)m_Ground.getPropertyAt("Energy",x+fromX, y+fromY);
					m_LocalObstacleMap[x][y] = m_Ground.getPropertyAtAsBoolean("Obstacle", new Location(x+fromX, y+fromY));
				
					if(m_LocalAgentMap[x][y] != null)
					{
						m_SensorData[x][y] = Constants.ContentAgent;
						int tmpX = x-Configuration.AgentInitialSensorRange;
						int tmpY = -(y-Configuration.AgentInitialSensorRange);
						m_LocalAgentList.add(new Point(tmpX, tmpY));
						m_AgentCount++;
					}			
					else if(m_LocalEnergyMap[x][y] != null)
					{
						m_SensorData[x][y] = Constants.ContentFood;
						m_LocalFoodList.add(new Point(x-Configuration.AgentInitialSensorRange, -(y-Configuration.AgentInitialSensorRange)));
						m_FoodCount++;
					}
					else if(m_LocalObstacleMap[x][y] == true)
					{
						m_SensorData[x][y] = Constants.ContentObstacle;
						m_LocalObstacleList.add(new Point(x-Configuration.AgentInitialSensorRange, -(y-Configuration.AgentInitialSensorRange)));
						m_ObstacleCount++;
					}
				}
				else //Otherwise outside map
				{
					m_SensorData[x][y] = Constants.ContentObstacle;
					m_LocalObstacleList.add(new Point(x-Configuration.AgentInitialSensorRange, -(y-Configuration.AgentInitialSensorRange)));
					m_ObstacleCount++;
					m_LocalObstacleMap[x][y] = true;	//If outside set to obstacle!!
				}
			}
		}
	}
	
	private String getMapInformation()
	{
		String str = "Obstacles: " + m_ObstacleCount + " Foods: " + m_FoodCount + " Agents: " + m_AgentCount;
		return str;
	}
			
	private final void preProcess()
	{
		if(isAlive()&& isEnabled())
		{
			//Check energy level
			if(m_EnergyLevel <= Configuration.AgentMinEnergyLevel)
			{
				if(m_AttackedBy != null)	//If death was a result of an attack then give creds to attacker :)
				{
					int killingGain = (int)(m_TotalEnergyIncrease*Configuration.AgentKilledReward);
					m_AttackedBy.increaseEnergyLevel(killingGain);
					m_AttackedBy = null;
				}
				
				m_Alive = false;
				m_Parent = null;
				m_Ground.removePropertyAt("Agent", m_X, m_Y);
			}
			
			//Removes parent if parent is not alive
			if(m_Parent != null)
			{
				if(!m_Parent.isAlive())
					m_Parent = null;
			}
			
			//For every cycle increase the flee counter if under attack
			if(m_Attacked == true)
			{
				m_FleeCycles++;	
				if(m_FleeCycles > Configuration.AgentFleeCycles)
				{
					m_FleeCycles = Configuration.AgentFleeCycles;
					m_Attacked = false;
				}	
			}
			
			if(m_AttackedBy != null)
			{
				m_AttackedByCycles++;
				//System.out.println(m_Id + " Attack cycle: " + m_AttackedByCycles + " Attacker: " + m_AttackedBy.m_Id);
				if(m_AttackedByCycles > Configuration.AgentAttackedKillCycles)
				{
					m_AttackedByCycles = Configuration.AgentAttackedKillCycles;
					m_AttackedBy = null;
				}				
			}
		}
	}
	
	private final void postProcess()
	{	
		m_Command = null;	//Reset the command value
	}
	 
	final void process()
	{
		preProcess();	//Preprocess agent data
		if(isAlive() && isEnabled())
		{
			decreaseEnergyLevel(Configuration.AgentProcessCost);	
			
			//Refresh user maps
			refreshLocalMaps();
			
			//Run user code (the user set which command to use)
			intelligence();
			
			//Set command successful flag to false
			m_CommandSuccessful = false;
			m_CommandType = Constants.CommandNone;
			
			//Process user selected command
			if(m_Command != null)
			{
				int initialEnergyLevel = m_EnergyLevel;
				CommandReturn ret = m_Command.commandFunction();
				m_CommandType = m_Command.getCommandType();
				
				if(ret != null)
				{			
					if(m_X != ret.getX() || m_Y != ret.getY())	//Move and Flee commands are successful
					{
						m_CommandSuccessful = true;
					}
					m_X = ret.getX();
					m_Y = ret.getY();
					
					if(ret.getEnergyDelta()< 0)
					{
						decreaseEnergyLevel(ret.getEnergyDelta());
					}
					else
					{
						increaseEnergyLevel(ret.getEnergyDelta());
					}
					
					
					if(m_Command.getCommandType() == Constants.CommandEat)
					{
						if(initialEnergyLevel < m_EnergyLevel)
						{
							m_CommandSuccessful = true;	
						}
					}
					
					//Process Attack command
					if(m_Command.getCommandType() == Constants.CommandAttack)
					{
						//Interactive agent data
						Agent otherAgent = (Agent)m_Ground.getPropertyAt("Agent", ret.getX2(), ret.getY2());
						if(otherAgent != null && otherAgent.m_Alive)
						{
							if(ret.getEnergyDelta2()< 0)	//This means we attacked the unit
							{
								m_CommandSuccessful = true;
								otherAgent.decreaseEnergyLevel(ret.getEnergyDelta2());
								otherAgent.m_Attacked = true;	//Notify other agent about the decrease of energy
								otherAgent.m_FleeCycles = 0;	//Reset other agents flee cycles
								otherAgent.m_AttackedBy = this;
								otherAgent.m_AttackedByCycles = 0;
							}
						}
					}
					
					if(m_Command.getCommandType() == Constants.CommandMate)
					{
						if(ret.getObject() != null)
						{
							m_CommandSuccessful = true;
							Agent child = (Agent) ret.getObject();
							child.m_Parent = this;
                            child.m_EnergyLevel = Configuration.AgentMateCost;
							//System.out.println(child.m_Id + " child of " + m_Id);
						}
					}	
				}//End Command return
				
				//If a parent is present we must give a share of the gains 
				//to this parent (the the parent of that parent etc.)
				if(m_Parent != null)
				{
					Agent child = this;
					Agent parent = child.getParent();
					
					int gain = m_EnergyLevel - initialEnergyLevel;
					int parentalGain = (int)(Configuration.AgentOffspringReward * gain);
					if(gain > 0 && parentalGain > 0)
					{
						child.decreaseEnergyLevel(parentalGain);//Decrease child
					}
                    
					while(parent != null && parentalGain > 0)
					{
						parentalGain = (int)(Configuration.AgentOffspringReward * parentalGain);
						parent.increaseEnergyLevel(parentalGain);	//Increase parent
					 	//System.out.println("Giving energy to parent! Child: " + child.m_Id + " Parent: " + parent.m_Id + " Gain: " + parentalGain);
					 	child = child.getParent();
					 	parent = child.getParent();
					}
				}
			}//End Command
		}
		
		postProcess();
	}
	
	private void increaseEnergyLevel(int delta)
	{
		m_TotalEnergyIncrease += Math.abs(delta);
		m_EnergyLevel += Math.abs(delta);
	}
	
	private void decreaseEnergyLevel(int delta)
	{
		m_EnergyLevel -= Math.abs(delta);
		m_TotalEnergyDecrease += Math.abs(delta);
		if(m_EnergyLevel<0)
		{
			m_TotalEnergyDecrease -= Math.abs(m_EnergyLevel);
			m_EnergyLevel = 0;
		}
	}
	
	private Agent getLocalAgent(Point p)
	{
		Agent ret = null;
		if(		p != null &&
				p.x >= -Configuration.AgentInitialSensorRange &&
				p.x <= Configuration.AgentInitialSensorRange &&
				p.y >= -Configuration.AgentInitialSensorRange &&
				p.y <= Configuration.AgentInitialSensorRange )
		{
			ret = (Agent)m_Ground.getPropertyAt("Agent", m_X+p.x, m_Y+(-p.y));
		}
		return ret;
	}
	
	private Energy getLocalFood(Point p)
	{
		Energy ret = null;
		if(		p.x >= -Configuration.AgentInitialSensorRange &&
				p.x <= Configuration.AgentInitialSensorRange &&
				p.y >= -Configuration.AgentInitialSensorRange &&
				p.y <= Configuration.AgentInitialSensorRange )
		{
			ret = (Energy)m_Ground.getPropertyAt("Energy", m_X+p.x, m_Y+(-p.y));
		}
		return ret;
	}
	
	final boolean isEnabled() {
		return m_Enabled;
	}
	
	final boolean isAlive()
	{
		return m_Alive;
	}
	
	final void enable(Map ground) {
		if(!isEnabled())
		{
			m_Ground = ground;
			
            if(m_Parent == null) //If there is no parent then place randomly
            {
                int x = (int)(Configuration.AppSceneWidth * Math.random());
                int y = (int)(Configuration.AppSceneHeight * Math.random());
                while(  !m_Ground.isOpen(x, y) )
                {
                    x = (int)(Configuration.AppSceneWidth * Math.random());
                    y = (int)(Configuration.AppSceneHeight * Math.random());
                }
                m_X = x;
                m_Y = y;
            }
            else    //If parent is existent then place agent somewhere in parents sensor-range
            {
                int x = (int)( ((Configuration.AgentInitialSensorRange*2) * Math.random()) - Configuration.AgentInitialSensorRange );
                int y = (int)( ((Configuration.AgentInitialSensorRange*2) * Math.random()) - Configuration.AgentInitialSensorRange );
                //System.out.println("x: " + x + " y: " + y);
                x+=m_Parent.m_X;
                y+=m_Parent.m_Y;
                while(  !m_Ground.isOpen(x, y) )
                {
                    x = (int)( ((Configuration.AgentInitialSensorRange*2) * Math.random()) - Configuration.AgentInitialSensorRange );
                    y = (int)( ((Configuration.AgentInitialSensorRange*2) * Math.random()) - Configuration.AgentInitialSensorRange );
                    x+=m_Parent.m_X;
                    y+=m_Parent.m_Y;
                }
                m_X = x;
                m_Y = y;
            }
            
			
			m_Alive = true;
			m_Enabled = true;
		}
	}
	
	final int getId()
	{
		return m_Id;
	}
	
	final Agent getParent()
	{
		return m_Parent;
	}
	
	/**
	 * This function is used to determine if the agent can mate or not. An agent can mate if it has enough
	 * food/energy.
	 * @return True if the agent can invoke the mate command, false otherwise.
	 */
	public final boolean canMate()
	{
		if(m_EnergyLevel >= Configuration.AgentMateCost)
		{
			return true;
		}
		return false;
	}
	
	/**
	 * This function is used to determine if the agent can feast from food/energy resources or not. An agent can eat
	 * food/energy resources if they are located in neighbouring cells.
	 * @return True if the agent can invoke the eat command, false otherwise.
	 */
	public final boolean canEat()
	{
		if( 	m_Ground.getPropertyAt("Energy", m_X+1,m_Y) != null ||
				m_Ground.getPropertyAt("Energy", m_X-1,m_Y) != null ||
				m_Ground.getPropertyAt("Energy", m_X,m_Y+1) != null ||
				m_Ground.getPropertyAt("Energy", m_X,m_Y-1) != null)
		{
			return true;
		}
		return false;
	}
	
	/**
	 * This function is used to determine if the agent can attack any other agent or not. An agent can attack
	 * other agents if they are located in neighbouring cells.
	 * @return True if the agent can invoke the attack command, false otherwise.
	 */
	public final boolean canAttack()
	{
		if( 	(m_Ground.getPropertyAt("Agent", m_X+1,m_Y) != null ||
				m_Ground.getPropertyAt("Agent", m_X-1,m_Y) != null ||
				m_Ground.getPropertyAt("Agent", m_X,m_Y+1) != null ||
				m_Ground.getPropertyAt("Agent", m_X,m_Y-1) != null ) &&
				m_EnergyLevel >= Configuration.AgentAttackCost)
		{
			return true;
		}
		return false;
	}
	
	/**
	 * This function is used to determine if the agent can flee or not. An agent can flee if it has been
	 * attacked.
	 * @return True if the agent can invoke flee command otherwise false.
	 */
	public final boolean canFlee()
	{
		return m_Attacked;
	}
	
	/**
	 * This function is used to give feedback weather or not an action was successful or not.
	 * @return True if last command (any command) was successful or not.
	 */
	public final boolean wasLastCommandSuccessful()
	{
		return m_CommandSuccessful;
	}
	
	/**
	 * This function is used to give feedback about the last command executed.
	 * @return The constant value corresponding to the last command executed.
	 */
	public final int getLastCommandType()
	{
		return m_CommandType;
	}
	
	/**
	 * Use this function to get directions to a point in the sensor range. This function will return
	 * a list/array of directions to this point if possible. The first entry in the array is the first move
	 * to make in order to get to the target. The function acts as a path-planner and will return the optimum
	 * path. 
	 * @param p The point to get directions to.
	 * @return An array of direction constants if path was found. Otherwise null is returned.
	 */
	public int[] getDirections(Point p)
	{
		int[] directions = null;
		if(	p.x >= -Configuration.AgentInitialSensorRange &&
			p.x <= Configuration.AgentInitialSensorRange &&
			p.y >= -Configuration.AgentInitialSensorRange &&
			p.y <= Configuration.AgentInitialSensorRange )
		{
			int goalX = m_X + p.x;
			int goalY = m_Y + (-p.y);	//Inervting y value from local user space to internal map space
			
			if(m_Ground.isOpen(goalX, goalY))
			{
				m_PathPlanner.findPath(m_Ground, m_X, m_Y, goalX, goalY);
				
				Stack stackPath = m_PathPlanner.getPath();
				stackPath.remove(stackPath.lastElement());
				Node[] nodePath = new Node[m_PathPlanner.getPath().size()];
				m_PathPlanner.getPath().toArray(nodePath);
				directions = new int[m_PathPlanner.getPath().size()];
				
				int cX = m_X; 
				int cY = m_Y;
				
				for(int i=(nodePath.length-1); i>=0; i--)	
				{
					int dirX = nodePath[i].m_X - cX;
					int dirY = nodePath[i].m_Y - cY;
					
					if(dirX>0) //EAST
					{
						directions[nodePath.length-1-i] = Constants.DirectionEast;
					}
					else if(dirX<0) //WEST
					{
						directions[nodePath.length-1-i] = Constants.DirectionWest;
					}
					else if(dirY<0) //NORTH
					{
						directions[nodePath.length-1-i] = Constants.DirectionNorth;
					}
					else if(dirY>0) //SOUTH
					{
						directions[nodePath.length-1-i] = Constants.DirectionSouth;
					}	
					cX = nodePath[i].m_X;
					cY = nodePath[i].m_Y;
				}
			}
		}
		
		return directions;
	}
	
	/**
	 * This function returns the closest neighbouring open spot/point to a point in the agents sensor range.
	 * Use this function to get an appropriate point from which your agent can feed or attack from.
	 * @param p The point used to find neighbouring points from. Usually a food resource location or an agent location.
	 * @return The closest open point to the given reference point provided as a input parameter.
	 */
	public final Point getClosestOpenNeighbouringPoint(Point p)
	{
		Point ret = null;
		if(	p.x >= -Configuration.AgentInitialSensorRange &&
				p.x <= Configuration.AgentInitialSensorRange &&
				p.y >= -Configuration.AgentInitialSensorRange &&
				p.y <= Configuration.AgentInitialSensorRange )
		{
			ArrayList <Point>points = new ArrayList<Point>();
			
			int tmpX = m_X + p.x + 1;
			int tmpY = m_Y + (-p.y);
			if(m_Ground.isOpen(tmpX, tmpY))
				points.add(new Point(p.x+1,p.y));
			
			tmpX = m_X + p.x - 1;
			tmpY = m_Y + (-p.y);
			if(m_Ground.isOpen(tmpX, tmpY))
				points.add(new Point(p.x-1,p.y));
			
			tmpX = m_X + p.x;
			tmpY = m_Y + (-p.y) + 1;
			if(m_Ground.isOpen(tmpX, tmpY))
				points.add(new Point(p.x,p.y+1));
			
			tmpX = m_X + p.x;
			tmpY = m_Y + (-p.y) - 1;
			if(m_Ground.isOpen(tmpX, tmpY))
				points.add(new Point(p.x,p.y-1));
			
			Collections.sort(points, m_ManhattanDistanceComparator);
			if(points.size() > 0)
			{
				ret = (Point)points.get(0);
			}
		}
		return ret;
	}
	
	/**
	 * This function returns a unique identifier for an agent
	 * that resides within the requesting agent's sensor range.
	 * @param p Sensor coordinate to the agent.
	 * @return Unique identifier for an agent. Returns -1 if no agent was found.
	 */
	public final int getId(Point p)
	{
		int ret = -1;
		Agent agent = getLocalAgent(p);
		if(agent != null)
		{
			ret = agent.getId();
		}
		return ret;
	}
	
	/**
	 * This function returns a name/type for an agent
	 * that resides within the requesting agent's sensor range.
	 * @param p Sensor coordinate to the agent.
	 * @return Name or type of an agent. Returns null if no agent was found.
	 */
	public final String getName(Point p)
	{
		String ret = null;
		Agent agent = getLocalAgent(p);
		if(agent != null)
		{
			ret = agent.getName();
		}
		return ret;
	}
	
	/**
	 * This function returns the energy level for an agent
	 * that resides within the requesting agent's sensor range.
	 * @param p Sensor coordinate to the agent.
	 * @return Energy level of an agent. Returns 0 if no agent was found.
	 */
	public final int getEnergyLevel(Point p)
	{
		int ret = 0;
		Agent agent = getLocalAgent(p);
		if(agent != null)
		{
			ret = agent.getEnergyLevel();
		}
		return ret;
	}
	
	
	/*public final int getFoodEnergyLevel(Point p)
	{
		int ret = 0;
		Energy agent = getLocalFood(p);
		if(agent != null)
		{
			ret = agent.getEnergyLevel();
		}
		return ret;
	}*/
	
	/**
	 * Feeds from a food/energy resource in the direction given by the parameter.
	 * @param direction
	 */
	public final void eat(int direction)
	{
		m_Command = new EatCommand(m_Ground, m_X, m_Y, direction);
	}
	
	/**
	 * Feeds from a food/energy resource if food/energy resource is available to the agent.
	 */
	public final void eat()
	{
		if(canEat())
		{
			if(getSensorFoodList()[0].getX() == 1)
			{
				eat(Constants.DirectionEast);
			}
			else if(getSensorFoodList()[0].getX() == -1)
			{
				eat(Constants.DirectionWest);
			}
			else if(getSensorFoodList()[0].getY() == 1)
			{
				eat(Constants.DirectionNorth);
			}
			else if(getSensorFoodList()[0].getY() == -1)
			{
				eat(Constants.DirectionSouth);
			}
		}
	}
	
	/**
	 * Moves the agent around in the world as specified by the parameters provided.
	 * @param direction The direction to move.
	 * @param speed Speed to move at.
	 */
	public final void move(int direction, int speed)
	{
		m_Command = new MoveCommand(m_Ground, m_X, m_Y, direction, speed);
	}
	
	/**
	 * Attacks an agent in a direction given by the parameter.
	 * @param direction The direction to attack.
	 */
	public final void attack(int direction)
	{
		m_Command = new AttackCommand(m_Ground, m_X, m_Y, direction);
	}
	
	/**
	 * Attacks an agent if an agent is available to attack.
	 */
	public final void attack()
	{
		if(canAttack())
		{
			if(getSensorAgentList()[1].getX() == -1)
			{
				attack(Constants.DirectionWest);
			}
			if(getSensorAgentList()[1].getX() == 1)
			{
				attack(Constants.DirectionEast);
			}
			if(getSensorAgentList()[1].getY() == -1)
			{
				attack(Constants.DirectionSouth);
			}
			if(getSensorAgentList()[1].getY() == 1)
			{
				attack(Constants.DirectionNorth);
			}
		}
	}
	
	/**
	 * Flees in a direction given by the parameter.
	 * @param direction The direction to flee.
	 */
	public final void flee(int direction)
	{
		if(m_Attacked == true)	//Can only flee if is attacked first
		{
			m_Command = new FleeCommand(m_Ground, m_X, m_Y, direction);	
		}
	}
	
	/**
	 * Spawns a child agent.
	 */
	public final void mate()
	{
		m_Command = new MateCommand(m_Ground, m_X, m_Y);
	}
	
	/**
	 * This function returns current energy level of the agent.
	 * @return Current energy level.
	 */
	public final int getEnergyLevel()
	{
		return m_EnergyLevel;
	}
	
	/**
	 * The points in the array returned by this function describes obstacles present 
	 * in an agent's sensor range. All obstacles are represented, from the agent
	 * reference point of (x,y) = (0,0), using the standard coordinate system as descibed below.
	 * <br>
	 * <br>
	 *(y)^<br> 
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|<br>
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|<br>
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;-----><br>
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;(x)<br>
	 * <br>
	 * For example:
	 * An obstacle north-east of the agent can be represented by (10,10). 
	 * An obstacle north of the agent can be represented by (0,10). 
	 * An obstacle south-west of the agent can be represented by (-10,-10). 
	 * An obstacle south of the agent can be represented by (0,-10). 
	 * 
	 * @return An array of points of (x,y) coordinates descibing obstacles present in the calling agent's sensor range. 
	 */
	public final Point[] getSensorObstacleList()
	{
		Point[] p = new Point[m_LocalObstacleList.size()];
		m_LocalObstacleList.toArray(p);
		return p;
	}
	
	/**
	 * The points in the array returned by this function describes other game agents present 
	 * in an agent's sensor range. The array is ordered so that closest agents are located in the beginning
	 * of the list. All agents are represented, from the agent
	 * reference point of (x,y) = (0,0), using the standard coordinate system as descibed below.
	 * <br>
	 * <br>
	 *(y)^<br> 
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|<br>
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|<br>
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;-----><br>
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;(x)<br>
	 * <br>
	 * For example:
	 * An agent north-east of the agent can be represented by (10,10). 
	 * An agent north of the agent can be represented by (0,10). 
	 * An agent south-west of the agent can be represented by (-10,-10). 
	 * An agent south of the agent can be represented by (0,-10). 
	 * 
	 * @return An array of points of (x,y) coordinates descibing agents present in the calling agent's sensor range. 
	 */
	public final Point[] getSensorAgentList()
	{
		Point[] p = new Point[m_LocalAgentList.size()];
		Collections.sort(m_LocalAgentList, m_ManhattanDistanceComparator);
		m_LocalAgentList.toArray(p);
		return p;
	}
	
	/**
	 * The points in the array returned by this function describes food/energy resources present 
	 * in an agent's sensor range. The array is ordered so that closest food/energy resources are 
	 * located in the beginning of the list. All food/energy resources are represented, from the agent
	 * reference point of (x,y) = (0,0), using the standard coordinate system as descibed below.
	 * <br>
	 * <br>
	 *(y)^<br> 
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|<br>
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|<br>
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;-----><br>
	 *&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;(x)<br>
	 * <br>
	 * For example:
	 * An food/energy resource north-east of the agent can be represented by (10,10). 
	 * An food/energy resource north of the agent can be represented by (0,10). 
	 * An food/energy resource south-west of the agent can be represented by (-10,-10). 
	 * An food/energy resource south of the agent can be represented by (0,-10). 
	 * 
	 * @return An array of points of (x,y) coordinates descibing food/energy resource present in the calling agent's sensor range. 
	 */
	public final Point[] getSensorFoodList()
	{
		Point[] p = new Point[m_LocalFoodList.size()];
		Collections.sort(m_LocalFoodList, m_ManhattanDistanceComparator);
		m_LocalFoodList.toArray(p);
		return p;
	}
	
	/**
	 * This function is used to retreive all agent sensors in a two-dimensional array (map). It can be used
	 * instead of or as a complement to {@link #getSensorObstacleList() getSensorObstacleList()}, {@link #getSensorFoodList() getSensorFoodList()} and {@link #getSensorAgentList() getSensorAgentList()}. 
	 * @return A two-dimensional array (map) descibing all objects (other agents, food/energy resources and obstacles) available in an agent's sensor range.
	 */
	public final int[][] getSensorDataMap()
	{
		return m_SensorData;
	}
}
