/*
 * Created on Mar 6, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package yaes.application.eel6938.game.pathplanners;

import java.util.Iterator;
import java.util.Stack;
import java.util.Vector;
import java.util.Calendar;

import yaes.application.eel6938.game.objects.Map;

/**
 * @author Linus Luotsinen
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class JAStar
{
	private Map m_Map;
	private Stack<Node> m_Open;
	private Stack<Node> m_Closed;
	private Stack<Node> m_Path;
	private Node m_Start, m_Goal;
	private int m_StepCnt;
	
	private int m_Timeout = 1000;	//Give one sec for path-finding
	
	private Vector m_Listeners = new Vector();
	private Vector m_Points = new Vector();
	private boolean m_Go = false;
	private long m_ElapsedTime = 0;
	private long m_StartTime = 0;
	
	public JAStar()
	{
		m_Open = new Stack<Node>();
		m_Closed = new Stack<Node>();
		m_Path = new Stack<Node>();
	}
	
	private void clear()
	{
		m_Open = new Stack<Node>();
		m_Closed = new Stack<Node>();
		m_Path = new Stack<Node>();
		m_ElapsedTime = 0;
		m_StartTime = 0;
		m_Go = false;
		m_Map = null;
		m_Start = null;
		m_Goal = null;
		m_StepCnt = 0;
	}
	
	
	/**
	 * @param ogm This is the map which A Star is working on
	 * @param startX 
	 * @param startY
	 * @param goalX
	 * @param goalY
	 * @param timeout This is the time A-Star has to find a path
	 * @param convolve Set this to a positive integer in order to convolve the map
	 * @param convolveFactor This is the number which must be exceeded in order to determine non passable areas
	 * @return
	 */
	public boolean findPath(final Map map, final int startX, final int startY, final int goalX, final int goalY)
	{	
		clear();
		stepInitialize(map,startX,startY,goalX,goalY);
		
		m_Go = true;
		boolean ok = false;
		if(m_Go == true)
		{
			m_StartTime = Calendar.getInstance().getTimeInMillis();
			ok = generatePath();
			if(ok)
			{
				updatePath();
			}
			m_ElapsedTime = Calendar.getInstance().getTimeInMillis() - m_StartTime;
			m_Go = false;
		}
		return ok;
	}
		
	
	
	private void stepInitialize(Map map, int startX, int startY, int goalX, int goalY)
	{
		this.m_Map = map;
		m_StepCnt = 0;
		
		m_Goal = new Node(goalX, goalY);	
		m_Goal.m_Number = ((int)map.getXHigh() * m_Goal.m_Y) + m_Goal.m_X;
		
		m_Start = new Node(startX, startY);
		m_Start.m_G = 0;
		m_Start.m_H =  manhattanDistance(m_Start, m_Goal); //Manhattan distance
		m_Start.m_F = m_Start.m_G + m_Start.m_H;
		m_Start.m_Number = ((int)map.getXHigh() * m_Start.m_Y) + m_Start.m_X;
		
		m_Open.push(m_Start);

		//System.out.println("Step initialize...");
		//System.out.print("Start: (" + start.x + "," + start.y + ")");
		//System.out.println(" Goal: (" + goal.x + "," + goal.y + ")");
	}
	
	
	private int manhattanDistance(Node n1, Node n2)
	{
		return (Math.abs(n1.m_X - n2.m_X) + Math.abs(n1.m_Y - n2.m_Y));
	}
	
	private boolean generatePath()
	{
		int ret = 0;
		while(ret == 0)
		{
			ret = step();
			if(m_StepCnt % 10 == 0)
			{
				m_ElapsedTime = Calendar.getInstance().getTimeInMillis() - m_StartTime;
				if(m_ElapsedTime > m_Timeout)
				{
					ret = -1;
				}
			}
		}
		m_ElapsedTime = Calendar.getInstance().getTimeInMillis() - m_StartTime;
		if(ret == -1)
		{
			return false;
		}
		
		return true;
	}
	
	private int step()
	{
		
		//System.out.println("Step " + stepCnt);
		//System.out.println("--------------------------------");
		m_StepCnt++;
		Node best = getBest();
		
		if(best == null)
		{
			//System.out.println("step(): Best node not available!" + stepCnt);
			return -1;
		}
		
		
		if(best.m_Number == m_Goal.m_Number)
		{
			//System.out.println("step(): Goal reached at iteration: " + stepCnt);
			return 1;
		}
		//System.out.println("step(): Best node is " + best.number + " (" + best.x + "," + best.y + ") f=" + best.f + " g=" + best.g + " h=" + best.h);
		createChildren(best);
		//path.push(best);
		
		//System.out.println(" children=" + best.numChildren);
		//System.out.println("step(): Open list: " + open.size());
		//System.out.println("step(): Closed list: " + closed.size());
		//System.out.println("step(): Path list: " + path.size());
		return 0;
	}
	
	
	
	
	private Node getBest()
	{
		Node best = null;
		int cnt = 0;
		
		Iterator iter = m_Open.iterator();
		while(iter.hasNext())
		{
			Node tmpNode = (Node) iter.next();
			if(cnt == 0)
			{
				best  = tmpNode;
			}
			
			if( tmpNode.m_F < best.m_F)
			{
				best = tmpNode;
			}
			cnt++;
		}
		
		return best;
	}
	
	private void createChildren(Node node)
	{
		int x = node.m_X;
		int y = node.m_Y;
		Node temp = new Node();
		
		if(m_Map.isOpen(x+1,y))
		{
			linkChild(node, new Node(x+1, y));
		}
		if(m_Map.isOpen(x-1,y))
		{
			linkChild(node, new Node(x-1, y));
		}
		if(m_Map.isOpen(x,y+1))
		{
			linkChild(node, new Node(x, y+1));
		}
		if(m_Map.isOpen(x,y-1))
		{
			linkChild(node, new Node(x, y-1));
		}
		
		m_Closed.push(node);
		m_Open.remove(node);
	}
	
	private void linkChild(Node node, Node temp)
	{
		int x = temp.m_X;
		int y = temp.m_Y;
		int num = (int)m_Map.getXHigh()*y + x;
		
		double g = node.m_G + 1;	//Cost is applied by simply adding 1 to the parents cost
		
		Node checkOpen = checkList(m_Open,num);
		Node checkClosed = checkList(m_Closed,num);
		
		if( checkOpen != null )
		{
			node.m_Children[node.m_NumChildren++] = checkOpen;
			if(g < checkOpen.m_G)
			{
				//System.out.println("linkChild(): CheckOpen " + "Parent: (x,y)=(" + checkOpen.x + "," + checkOpen.y + ") " + "Number=" + checkOpen.number);
				checkOpen.m_Parent = node;
				checkOpen.m_G = g;
				checkOpen.m_F = g + checkOpen.m_H;
				//System.out.println("OPEN RE-REOUTING!!");
				//updateParents(checkOpen);
				//System.out.println("linkChild(): CheckOpen " + "Parent: (x,y)=(" + checkOpen.x + "," + checkOpen.y + ") " + "Number=" + checkOpen.number);
			}
		}
		else if( checkClosed != null)
		{
			node.m_Children[node.m_NumChildren++] = checkClosed;
			if(g < checkClosed.m_G)
			{
				checkClosed.m_Parent = node;
				checkClosed.m_G = g;
				checkClosed.m_F = g + checkClosed.m_H;
				//System.out.println("CLOSED RE-REOUTING!!");
				updateParents(checkClosed);
			}
		}
		else
		{
			Node newNode = new Node(x,y);
			newNode.m_Parent = node;
			newNode.m_G = g;
			newNode.m_H = manhattanDistance(newNode, m_Goal);//(Math.abs(x - goal.x) + Math.abs(y - goal.y));
			newNode.m_F = newNode.m_G + newNode.m_H;
			newNode.m_Number = ((int)m_Map.getXHigh())*y + x;
			m_Open.push(newNode);
			node.m_Children[node.m_NumChildren++] = newNode;
			
			//System.out.println("linkChild(): New node added to open: " + "Parent: (x,y)=(" + node.x + "," + node.y + ") " + "Number=" + node.number + " New node:" + newNode.number + " (" + newNode.x + "," + newNode.y + ") f=" + newNode.f + " g=" + newNode.g + " h=" + newNode.h);
		}
	}
	
	private void updatePath()
	{
		Node best = getBest();
		m_Path.push(best);
		
		while(best.m_Number != m_Start.m_Number)
		{
			Node parent = best.m_Parent;
			m_Path.push(parent);
			best = parent;
		}
	}
	
	private void updateParents(Node node)
	{
		double g = node.m_G;
		int c = node.m_NumChildren;
		Node kid = null;
		//System.out.println("Update parents: " + node.number + " " + node.g + " Children=" + node.numChildren);
		for(int i = 0; i<c; i++)
		{
			kid = node.m_Children[i];
			//System.out.println("Update parents: " + kid.number + " " + kid.g);
			if(g+1 < kid.m_G)
			{
				System.out.println("updateParents: push");
				kid.m_G = g+1;
				kid.m_F = kid.m_G + kid.m_H;
				kid.m_Parent = node;
				m_Path.push(kid);
			}
		}
		
		Node parent;
		while(m_Path.size()> 0)
		{
			System.out.println("updateParents: pop");
			parent = (Node) m_Path.pop();
			c = parent.m_NumChildren;
			for(int i=0; i<c; i++)
			{
				kid = parent.m_Children[i];
				if(parent.m_G+1 < kid.m_G)
				{
					System.out.println("updateParents: update!!");
					kid.m_G = parent.m_G + 1;
					kid.m_F = kid.m_G + kid.m_H;
					kid.m_Parent = parent;
					m_Path.push(kid);
				}
			}
		}
	
	}
	
	private Node checkList(Stack list, int num)
	{
		Node ret = null;
		Iterator iter = list.iterator();
		while(iter.hasNext())
		{
			Node tmpNode = (Node) iter.next();
			
			if(tmpNode.m_Number == num )
			{
				//System.out.println("Item detected in list: " + num + "=" + tmpNode.number);
				ret = tmpNode;
				break;
			}
		}
		return ret;
	}
	/**
	 * @return
	 */
	public Stack getClosed() {
		return m_Closed;
	}

	/**
	 * @return
	 */
	public Node getGoal() {
		return m_Goal;
	}

	/**
	 * @return
	 */
	public Stack getOpen() {
		return m_Open;
	}

	/**
	 * @return
	 */
	public Stack<Node> getPath() {
		return m_Path;
	}

	/**
	 * @return
	 */
	public Node getStart() {
		return m_Start;
	}
	
	
	public double getG()
	{
		if(m_Path != null)
		{
			if(!m_Path.isEmpty())
			{
				Node node = (Node) m_Path.firstElement();
				return node.m_G;
			}
		}
		return -1;
	}
	
	public double getF()
	{
		if(m_Path != null)
		{
			if(!m_Path.isEmpty())
			{
				Node node = (Node) m_Path.firstElement();
				return node.m_F;
			}
		}
		return -1;
	}
	
	public double getH()
	{
		if(m_Path != null)
		{
			if(!m_Path.isEmpty())
			{
				Node node = (Node) m_Path.firstElement();
				return node.m_H;
			}
		}
		return -1;
	}
}
