/* 
 * Created on  Apr 7, 2005
 *
 */
package yaes.application.eel6938.game.statistics;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import yaes.application.eel6938.game.Configuration;
import yaes.application.eel6938.game.objects.Agent;
import yaes.application.eel6938.game.objects.ObjectController;

/**
 * @author Linus Luotsinen
 *
 */
public class StatisticsAnalyzer {
    private long m_TimeStamp;
    private ArrayList<IStatistic> m_Statistics;
    private ArrayList<IGroupStatistic> m_GroupStatistics;
    
    public StatisticsAnalyzer(long startTime)
    {
        m_TimeStamp = startTime;
        m_Statistics = new ArrayList<IStatistic>();
        m_GroupStatistics = new ArrayList<IGroupStatistic>();
        
        for(int i=0; i<Configuration.AppLogStatistics.length;i++)
        {
            try {
                IStatistic newStatistic = (IStatistic)Class.forName(Configuration.AppLogStatisticsClassPath + Configuration.AppLogStatistics[i]).newInstance();
                m_Statistics.add(newStatistic);
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }
        }
        
        for(int i=0; i<Configuration.AppLogGroupStatistics.length; i++)
        {
            try {
                IGroupStatistic newStatistic = (IGroupStatistic)Class.forName(Configuration.AppLogStatisticsClassPath + Configuration.AppLogGroupStatistics[i]).newInstance();
                m_GroupStatistics.add(newStatistic);
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }
        }
    }
    
    /*public void addStatistic(IStatistic statistic)
    {
        m_Statistics.add(statistic);
    }
    
    public void removeStatistic(IStatistic statistic)
    {
        m_Statistics.remove(statistic);
    }*/
    
    public void analyze(Agent[] agents)
    {
        if(Configuration.AppLogAgentData)
        {
            try 
            {
                String outFilePath =   Configuration.AppLogPath + m_TimeStamp + "\\" + Configuration.AppLogStatisticsPath;
                File file = new File(Configuration.AppLogPath + m_TimeStamp + "\\");
                file.mkdirs();
                BufferedWriter writer = new BufferedWriter(new FileWriter(outFilePath));
                
                //Add Groups
                String[] groups = getGroupList(agents);
                for(int i=0; i<groups.length; i++)
                {
                    writer.write("#Group\t" + getGroupMemberCount(groups[i], agents) + "\t" + groups[i]);
                    writer.newLine();
                    Iterator<IGroupStatistic> iter = m_GroupStatistics.iterator();
                    while(iter.hasNext())
                    {
                        IGroupStatistic stat = iter.next();
                        ArrayList<StatisticsNode[]> sNodes = getNodesFromFiles(groups[i], agents);
                        if(sNodes != null)
                        {
                            double[] value = stat.getStatisticValue(sNodes);
                            String name = stat.getStatisticName();
                            if(name != null)
                            {
                                writer.write(stat.getStatisticName().trim());
                            }
                            
                            if(value != null)
                            {
                                for(int j=0;j<value.length;j++)
                                {
                                    writer.write("\t" + value[j]);
                                }
                                writer.newLine();
                            }
                        }
                    }
                    writer.newLine();
                }
                
                //Add Agents
                for(int i=0; i<agents.length; i++)
                {
                    writer.write("#Agent\t" + ObjectController.getAgentId(agents[i]) + "\t" + agents[i].getClass().getSimpleName());
                    writer.newLine();
                    
                    String filePath =   Configuration.AppLogPath + m_TimeStamp + "\\" + 
                                        ObjectController.getAgentId(agents[i]) + "_" + 
                                        agents[i].getClass().getSimpleName() + ".log";
                    File f = new File(filePath);
                    if(f.exists())
                    {
                        Iterator<IStatistic> iter = m_Statistics.iterator();
                        while(iter.hasNext())
                        {
                            IStatistic stat = iter.next();
                            StatisticsNode[] sNodes = getNodesFromFile(filePath);
                            if(sNodes != null)
                            {
                                double[] value = stat.getStatisticValue(sNodes);
                                String name = stat.getStatisticName();
                                if(name != null)
                                {
                                    writer.write(stat.getStatisticName().trim());
                                }
                                
                                if(value != null)
                                {
                                    for(int j=0;j<value.length;j++)
                                    {
                                        writer.write("\t" + value[j]);
                                    }
                                    writer.newLine();
                                }
                            }
                        }
                        writer.newLine();
                    }
                }
                
                //Finally close the writer
                writer.close();
            } 
            catch (FileNotFoundException e) 
            {
                e.printStackTrace();
            }
            catch (IOException e1) 
            {
                e1.printStackTrace();
            }
        }
    }
    
    private int getGroupMemberCount(String group, Agent[] agents)
    {
        int ret = 0;
        for(int i=0;i<agents.length;i++)
        {
            if(agents[i].getClass().getSimpleName().compareTo(group) == 0)
            {
                ret++;   
            }
        }
        return ret;
    }
    
    private String[] getGroupList(Agent[] agents)
    {
        ArrayList<String> uniqueAgents = new ArrayList<String>();
        for(int i=0;i<agents.length;i++)
        {
            boolean contains = false;
            String[] tmpArr = new String[uniqueAgents.size()];
            uniqueAgents.toArray(tmpArr);
            for(int j=0;j<tmpArr.length;j++)
            {
                if(tmpArr[j].compareToIgnoreCase(agents[i].getClass().getSimpleName()) == 0)
                {
                    contains = true;
                    break;
                }
            }
            
            if(!contains)
            {
                uniqueAgents.add(agents[i].getClass().getSimpleName());
            }
        }
        String[] tmpArr = new String[uniqueAgents.size()];
        uniqueAgents.toArray(tmpArr);
        return tmpArr;
    }
    
    private ArrayList<StatisticsNode[]> getNodesFromFiles(String group, Agent[] agents)
    {
        ArrayList<StatisticsNode[]> agentNodes = new ArrayList<StatisticsNode[]>();   
        for(int i=0;i<agents.length;i++)
        {
            if(agents[i].getClass().getSimpleName().compareTo(group) == 0)
            {
                String filePath = Configuration.AppLogPath + m_TimeStamp + "\\" + 
                        ObjectController.getAgentId(agents[i]) + "_" + 
                        agents[i].getClass().getSimpleName() + ".log";
                File f = new File(filePath);
                if(f.exists())
                {
                    agentNodes.add(getNodesFromFile(filePath));
                }
            }
        }
        return agentNodes;
    }
    private StatisticsNode[] getNodesFromFile(String filePath)
    {
        StatisticsNode [] ret = null;
        try
        {    
            ArrayList<StatisticsNode> nodes = new ArrayList<StatisticsNode>();
            BufferedReader in = new BufferedReader(new FileReader(filePath));
            if (!in.ready())
                throw new IOException();
            String line;
            while ((line = in.readLine()) != null)
            {
                StatisticsNode n = new StatisticsNode(line);
                nodes.add(n);
            }
            in.close();
            
            ret = new StatisticsNode[nodes.size()];
            nodes.toArray(ret);
        }
        catch (IOException e)
        {
            System.out.println(e);
        }
        return ret;
    }
}
