package yaes.application.eel6938.statistics;

import yaes.application.eel6938.game.Configuration;
import yaes.application.eel6938.game.statistics.IStatistic;
import yaes.application.eel6938.game.statistics.StatisticsNode;

public class MapCoverageStatistic implements IStatistic {
	
    private double m_MaxCoverageArea;
    private int m_Width;
    private int m_Height;
    private int m_VisitedCounter = 0;
    private boolean[][] m_Map;
	
    public MapCoverageStatistic()
	{
        m_Width = Configuration.AppSceneWidth;
        m_Height = Configuration.AppSceneHeight;
        m_Map = new boolean[m_Width][m_Height];
	}
	
    private void markAsVisited(int xIn, int yIn)
    {
        int topX = xIn - Configuration.AgentInitialSensorRange;
        int topY = yIn - Configuration.AgentInitialSensorRange;
        int bottomX = xIn + Configuration.AgentInitialSensorRange;
        int bottomY = yIn + Configuration.AgentInitialSensorRange;
        
        if(topX<0)
            topX=0;
        if(topY<0)
            topY = 0;
        
        if(bottomX>=m_Width)
            bottomX = m_Width-1;
        if(bottomY>=m_Height)
            bottomY = m_Height-1;
        
        //System.out.println("org: " + xIn + "," + yIn + " top: " + topX + "," + topY + " bottom: " + bottomX + "," + bottomY);
        for(int x=topX; x<=bottomX;x++)
        {
            for(int y=topY; y<=bottomY;y++)
            {
                if(x>=0 && y>=0 && x<m_Width && y<m_Height)
                {
                    if(m_Map[x][y] == false)
                    {
                        //System.out.println("Updating: " + x + "," + y);
                        m_Map[x][y] = true;
                        m_VisitedCounter++;
                    }
                }
            }
        }
        
    }
	public double[] getStatisticValue(StatisticsNode[] nodes) {
		double[] ret = new double[1];
        m_Map = new boolean[m_Width][m_Height];
        m_VisitedCounter = 0;
        for(int i=0;i<nodes.length;i++)
        {
            markAsVisited(nodes[i].getX(), nodes[i].getY());
        }
        double ratio = (m_VisitedCounter/(double)(m_Width*m_Height));
        //System.out.println("Visited: " + m_VisitedCounter + " Maximum: " + (m_Width*m_Height) + " Ratio: " + ratio);
        ret[0] = ratio;
        return ret;
	}
    

    public String getStatisticName() {
        return "MapCoverage";
    }
	
	
}
