/*
 * Created on Oct 13, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package yaes.application.empiricalpathplanner.ga;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import yaes.application.empiricalpathplanner.Path;
import yaes.application.empiricalpathplanner.World;
import yaes.application.empiricalpathplanner.ga.selection.RandomSelection;
import yaes.application.empiricalpathplanner.ga.selection.RouletteWheelSelection;
import yaes.application.empiricalpathplanner.spline.BSpline;
import yaes.application.empiricalpathplanner.spline.XYZ;
import yaes.framework.algorithm.geneticalgorithm.GeneticPool;
import yaes.framework.algorithm.geneticalgorithm.ICrossover;
import yaes.framework.algorithm.geneticalgorithm.IMutation;


/**
 * @author Linus Luotsinen
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class PathPlanner 
	implements Runnable
{
	private GeneticPool gp;
	
	public static double SplineCurveLengthWeight = 1; //0 = Don't use 
	public static double SplineCurveTerrainWeight = 1; //0 = Don't use 
	public static double ControlPointTooCloseWeight = 00000; //0 = Don't use
	public static int ControlPointTooCloseMax = 50;
	public static int ControlPointTooCloseMin = 0;
	
	
	/**
	 * @return Returns the fittnessFunction.
	 */
	public FitnessFunction getFittnessFunction() {
		return fittnessFunction;
	}
	//The fitness function
	private FitnessFunction fittnessFunction;	
	
	//Selection method
	private RouletteWheelSelection rouletteSelectionMethod;
	private RandomSelection randomSelectionMethod;
	
	//Crossover operators
	private ICrossover [] crossoverOperators; 
	
	//Mutation operators
	private IMutation [] mutationOperators; 
	
	
	private int epochs;
	//Converge when no change has been made in a number of epochs
	int convergenceCriteria = 0;
	
	private boolean running = false;
	private Vector listeners;
	
	private World world;
	
	private Thread thread;
	
	
	
	/**
	 * @return Returns the convergenceCriteria.
	 */
	public int getConvergenceCriteria() {
		return convergenceCriteria;
	}
	/**
	 * @param convergenceCriteria The convergenceCriteria to set.
	 */
	public void setConvergenceCriteria(int convergenceCriteria) {
		this.convergenceCriteria = convergenceCriteria;
	}
	/**
	 * @return Returns the running.
	 */
	public boolean isRunning() {
		return running;
	}
	/**
	 * @param running The running to set.
	 */
	public void setNotRunning() {
		this.running = false;
	}
	
	public List getChromosomes()
	{
		return gp.getChromosomes();
	}
	
	public void setOperators(ICrossover [] crossoverOperators, IMutation [] mutationOperators)
	{
		
		if(crossoverOperators == null )
		{
			//Create crossover operators
			crossoverOperators = new ICrossover[3];
			crossoverOperators[0] = OperatorFactory.CreateCrossoverOperator(OperatorFactory.UNIFORM_CROSSOVER, this);
			crossoverOperators[1] = OperatorFactory.CreateCrossoverOperator(OperatorFactory.ONEPOINT_CROSSOVER, this);
			crossoverOperators[2] = OperatorFactory.CreateCrossoverOperator(OperatorFactory.HEURISTIC_CROSSOVER, this);
		}
		
		if(mutationOperators == null )
		{
			//Create mutation operators
			mutationOperators = new IMutation[2];
			String [] args = new String[1];
			args[0] = new String("400");
			mutationOperators[0] = OperatorFactory.CreateMutationOperator(OperatorFactory.ALLPOINT_MUTATION, args);
			args = new String[1];
			args[0] = new String("800");
			mutationOperators[1] = OperatorFactory.CreateMutationOperator(OperatorFactory.ONEPOINT_MUTATION, args);
		}
		
		
		//Add operators
		this.crossoverOperators = crossoverOperators;
		this.mutationOperators = mutationOperators;
		
		//Create genetic pool with first mutation and crossover operator
		if(crossoverOperators != null && mutationOperators != null)
		{
			gp = new GeneticPool(fittnessFunction, rouletteSelectionMethod, crossoverOperators[0], mutationOperators[0]);
			//Enable multi crossover
			gp.setUseMultiCrossover(true); //Generate two children from two parents.
		}
		else
		{
			System.out.println("Error no operators selected!");
			System.exit(0);	
		}
	}
	
	
	public PathPlanner(World world)
	{
		this.world = world;
		listeners = new Vector();
	
		//Create the operators!
		fittnessFunction = new FitnessFunction(world);
		
		//Use Roulette wheel selection
		rouletteSelectionMethod = new RouletteWheelSelection(fittnessFunction);		
		
		//Set the operators
		crossoverOperators = null;
		mutationOperators = null;
		setOperators(crossoverOperators, mutationOperators);
		
		/*//Create genetic pool with first mutation and crossover operator
		gp = new GeneticPool(fittnessFunction, rouletteSelectionMethod, crossoverOperators[0], mutationOperators[0]);
		
		//Enable multi crossover
		gp.setUseMultiCrossover(true); //Generate two children from two parents.*/
	}
	
	public void addPathPlannerListener(IPathPlannerListener obj)
	{
		listeners.add(obj);
	}
	
	public void removePathPlannerListener(IPathPlannerListener obj)
	{
		listeners.remove(obj);
	}
	
	public void SetMutationRate(float mr)
	{
		gp.setMutationRate(mr);
	}
	
	public void SetCrossoverRate(float cr)
	{
		gp.setCrossoverRate(cr);
	}
	
		
	public void evolve(int epochs)
	{
		this.epochs = epochs;
		thread = new Thread(this);
		thread.start();
	}
	
	public void evolveBatch(int epochs)
	{
		this.epochs = epochs;
		run();
	}
	
	
		
	/**
	 * @return Returns the world.
	 */
	public World getWorld() {
		return world;
	}
	
	public GeneticPool getGeneticPool()
	{
		return gp;
	}

	private void epochCompleteEvent(int epoch, int mutations, int crossovers)
	{
		Iterator iter = listeners.iterator();
		while(iter.hasNext())
		{
			IPathPlannerListener p = (IPathPlannerListener)iter.next();
			p.OnEpochComplete(epoch, mutations, crossovers);
		}
	}
	
	private void evolveCompleteEvent()
	{
		Iterator iter = listeners.iterator();
		while(iter.hasNext())
		{
			IPathPlannerListener p = (IPathPlannerListener)iter.next();
			p.OnEvolveComplete();
		}
	}
	
	public Vector getKnownChromosomes()
	{
		Vector knownChromosomes = new Vector();
		Iterator iter = world.getKnownPaths().iterator();
		while(iter.hasNext())
		{
			Path knownPath = (Path)iter.next();
			knownChromosomes.add(new Chromosome(knownPath));
		}
		
		return knownChromosomes;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		//System.out.println("CHROMOSOMES: " + gp.getChromosomes().size());
		running = true;
		//Vector newKnownPaths = getKnownChromosomes();
		double previousBest = 0; int cntBest=0;
		for(int i=1; i<=epochs; i++)
		{
			if(running == false)
			{
				break;
			}
			
			//Add known paths
			/*gp.getChromosomes().addAll(newKnownPaths); */
			
			//Force the selection method to recalculate the population fitness
			rouletteSelectionMethod.ResetFitnessValues();
			
			//Compete
			gp.compete();
			
			//Keep constant number of chromosomes
			/*for(int k=0;k<newKnownPaths.size();k++)
			{
				gp.getChromosomes().remove(gp.getChromosomes().size()-1);
			}*/
			
			//Update operators for next epoch
			int crossoverIndex = i % this.crossoverOperators.length;
			int mutationIndex = i % this.mutationOperators.length;
			gp.setCrossover(crossoverOperators[crossoverIndex]);
			gp.setMutation(mutationOperators[mutationIndex]);
			
			//System.out.println("------------------------");
			//System.out.println("Crossover: " + gp.getCrossover().toString() + " Mutation: " + gp.getMutation());
			epochCompleteEvent(i, 0, 0);
			
			//Check for convergence
			if(convergenceCriteria > 0)
			{
				if(convergenceCriteria == cntBest)
				{
					running = false;
				}
				else
				{
					double best = gp.getFittnessFunction().fittness((Chromosome)gp.getChromosomes().get(0));
					//System.out.println("1. " + cntBest + " " + best + " " + previousBest);
					if(previousBest == best)
					{
						cntBest++;
					}
					else
					{
						previousBest = best;
						cntBest = 0;
					}
					//System.out.println("2. " + cntBest + " " + best + " " + previousBest);
				}
			}
			//End check convergence
		}
		running = false;
		evolveCompleteEvent();
	}
	public double pathLengthCost(Path path)
	{
		return path.getPathLenght() * PathPlanner.SplineCurveLengthWeight;
	}
	
	//Expensive ??
	public double terrainCost(XYZ[] curve)
	{
		double cost = 0.0;
		
		if(curve != null)
		{
			//System.out.println(curve.length);
			//Need to find all pixels between current and next (add contents to total cost)
			for(int i=0; i<curve.length-1;i++)	 
			{
				//Get the first and second point
				XYZ[] inpArr = new XYZ[2];
				inpArr[0] = curve[i];
				inpArr[1] = curve[i+1];
				int tmpDist = (int)Math.round(XYZ.distance(curve[i],curve[i+1]));
				
				if(tmpDist > 1)
				{
					XYZ[] tmpCurve = BSpline.createSpline(2,tmpDist,inpArr);
					
					for(int j=0;j<tmpCurve.length;j++)
					{
						cost += (world.classifyTerrain((int)Math.round(tmpCurve[j].x),(int)Math.round(tmpCurve[j].y)) *
									PathPlanner.SplineCurveTerrainWeight);
					}
				}
			}
		}
		return cost;
	}
	
	//FIXED! FIX: Make this function look over all control points! not just two concecutive. 
	public double controlPointDistanceCost(XYZ[] controlPoints)
	{
		double cost = 0.0;
		if(controlPoints!= null)
		{
			for(int j=0; j<controlPoints.length; j++ )
			{
				XYZ compareWith = controlPoints[j];
				for(int i=0; i<controlPoints.length;i++)	 
				{
					XYZ compareTo = controlPoints[i];
					if(!compareWith.equals(compareTo))
					{
						int tmpDist = (int)Math.round(XYZ.distance(compareWith,compareTo));
						if(tmpDist >= PathPlanner.ControlPointTooCloseMin && tmpDist < PathPlanner.ControlPointTooCloseMax)
						{
							//Penalize control point to close to each other!
							if(tmpDist == 0)
							{
								cost += PathPlanner.ControlPointTooCloseWeight;
							}
							else
							{
								cost += PathPlanner.ControlPointTooCloseWeight/tmpDist;
							}							
						}
					}
				}
			}
			
			/*Old function
			//Need to find all pixels between current and next (add contents to total cost)
			for(int i=0; i<controlPoints.length-1;i++)	 
			{
				//Get the first and second point
				XYZ[] inpArr = new XYZ[2];
				inpArr[0] = controlPoints[i];
				inpArr[1] = controlPoints[i+1];
				int tmpDist = (int)Math.round(XYZ.distance(controlPoints[i],controlPoints[i+1]));
				if(tmpDist > 0 && tmpDist < 30)
				{
					//Penalize control point to close to each other!
					cost = 1000/tmpDist;
				}
			}*/
			
		}
		return cost;
	}
	
}
