/*
 * Created on Oct 21, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package yaes.application.empiricalpathplanner.ga.crossover;


import java.util.HashSet;
import java.util.Set;

import yaes.application.empiricalpathplanner.Path;
import yaes.application.empiricalpathplanner.ga.Chromosome;
import yaes.framework.algorithm.geneticalgorithm.IChromosome;
import yaes.framework.algorithm.geneticalgorithm.ICrossover;
import yaes.framework.algorithm.geneticalgorithm.IFittnessFunction;

/**
 * @author Linus Luotsinen
 *
 * This crossover operator uses the best fit chromosome of the parents.
 * The best fit parent is added with the difference between both parents multiplied by a random variable.
 * child = ((bestParent - otherParent) * constant) +  bestParent
 * Hence, this crossover acts somewhat like a heuristic.
 */
public class HeuristicCrossover implements ICrossover {

	IFittnessFunction fitness;
	private int crossOverCount;
	public HeuristicCrossover(IFittnessFunction fitness)
	{
		this.fitness = fitness;
		crossOverCount = 0;
	}
	/* (non-Javadoc)
	 * @see yaes.framework.algorithm.geneticalgorithm.ICrossover#canMate(yaes.framework.algorithm.geneticalgorithm.IChromosome, yaes.framework.algorithm.geneticalgorithm.IChromosome)
	 */
	public boolean canMate(IChromosome chromosome1, IChromosome chromosome2) {
		if(chromosome1.equals(chromosome2))
		{
			//System.out.println("Heuristic canMate failed! " );
			return false;
		}
		
		// TODO Auto-generated method stub
		return true;
	}
	
	public int getCrossOverCount() {
		return crossOverCount;
	}

	/* (non-Javadoc)
	 * @see yaes.framework.algorithm.geneticalgorithm.ICrossover#singleCrossover(yaes.framework.algorithm.geneticalgorithm.IChromosome, yaes.framework.algorithm.geneticalgorithm.IChromosome)
	 */
	public IChromosome singleCrossover(IChromosome chromosome1, IChromosome chromosome2) {
		Chromosome x1 = (Chromosome)chromosome1;
		Chromosome x2 = (Chromosome)chromosome2;
		double r = Math.random();
		Chromosome ret = null;
		Path path = null;
		double fitX2 = fitness.fittness(x2);
		double fitX1 = fitness.fittness(x1);
		
		
		if(fitX2  < fitX1)
		{
			path = mix(x2, x1, r); 
			/*System.out.println("x2 = " + fitX2 + " x1 = " + fitX1 + " r=" + r);
			System.out.println("x2=" + x2.getChromosomePath().toString());
			System.out.println("x1=" + x1.getChromosomePath().toString());
			System.out.println("=>" + path.toString());*/
		}
		else
		{
			path = mix(x1, x2, r); 
			/*System.out.println("x1 = " + fitX1 + " x2 = " + fitX2 + " r=" + r);
			System.out.println("x1=" + x1.getChromosomePath().toString());
			System.out.println("x2=" + x2.getChromosomePath().toString());
			System.out.println("=>" + path.toString());*/
		}
		crossOverCount++;
		ret = new Chromosome(path);
		//System.out.println("-->New path fitness" + fitness.fittness(ret));
		
		return ret;
	}
	
	private Path mix(Chromosome best, Chromosome worst, double r)
	{
		return Path.Add(Path.Multiply(Path.Subtract(best.getChromosomePath(), worst.getChromosomePath()), r), best.getChromosomePath());
	}
	/* (non-Javadoc)
	 * @see yaes.framework.algorithm.geneticalgorithm.ICrossover#multipleCrossover(yaes.framework.algorithm.geneticalgorithm.IChromosome, yaes.framework.algorithm.geneticalgorithm.IChromosome)
	 */
	public Set multipleCrossover(IChromosome chromosome1, IChromosome chromosome2) {
		Set set = new HashSet();
		set.add(singleCrossover(chromosome1, chromosome2));
		return set;
	}
	
	public String toString()
	{
		return "Heuristic Crossover Operator (mixed)";
	}

}
