/*
 * Created on Oct 13, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package yaes.application.empiricalpathplanner.gui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JPanel;

import yaes.application.empiricalpathplanner.IDrawable;
import yaes.application.empiricalpathplanner.Path;
import yaes.application.empiricalpathplanner.World;
import yaes.application.empiricalpathplanner.ga.Chromosome;
import yaes.application.empiricalpathplanner.spline.XYZ;



/**
 * @author Linus Luotsinen
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class Map
	extends JPanel
	implements MouseListener, MouseMotionListener, IDrawable
{
	/**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3762534529134835511L;
    //Data for splines	
	private Vector 	savedPaths;		//Each element is an array of control points (XYZ [] controlPoints)
	private Path 	startGoalPath; 	//Start goal path
	private Path 	currentPath;	//Currently selected path
	private XYZ 	selectedControlPoint;
	
	double curveLen = 0.0;
	double curveCtrlPtsCost = 0.0;
	double curveLenCost = 0.0;
	double curveTerrainCost = 0.0;
	double curveCost = 0.0;
	
	double prevCurveCtrlPtsCost = 0.0;
	double prevCurveLenCost = 0.0;
	double prevCurveTerrainCost = 0.0;
	double prevCurveCost = 0.0;
	
	private Vector mapListeners;
	
	private static int SPLINE_T = 3;
	private static int SPLINE_RESOLUTION = 50;
	
	//The world (a numeric representation)
	private World world;
	private MainFrame mainFrame;
	
	private boolean drawChromosomes = true;
	
	
	
	/**
	 * @return Returns the drawChromosomes.
	 */
	public boolean isDrawChromosomes() {
		return drawChromosomes;
	}
	/**
	 * @param drawChromosomes The drawChromosomes to set.
	 */
	public void setDrawChromosomes(boolean drawChromosomes) {
		this.drawChromosomes = drawChromosomes;
	}
	public Map(MainFrame mainFrame, World world)
	{
		this.world = world;
		this.mainFrame = mainFrame;
		
		mapListeners = new Vector();
		
		selectedControlPoint = null;
		savedPaths = new Vector();
		currentPath = new Path(SPLINE_T, SPLINE_RESOLUTION);
		startGoalPath = null;
		
		this.setBackground(Color.black);
		this.setDoubleBuffered(true);
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		
		
		world.setKnownPaths(savedPaths);	//Connect the GUI list with the world list.
		world.setStartGoalPath(startGoalPath);  //NULL initially
	}
	
	public void setMainFrameResizable(boolean val)
	{
		mainFrame.setResizable(val);
	}
	
	public void addMapListener(IMapListener obj)
	{
		mapListeners.add(obj);
	}
	public void removeMapListener(IMapListener obj)
	{
		mapListeners.remove(obj);
	}
	
	public void sendMapMessageEvent(String message)
	{
		Iterator iter = mapListeners.iterator();
		while(iter.hasNext())
		{
			IMapListener obj = (IMapListener) iter.next();
			obj.mapMessageEvent(message);
		}
	}
	
	/**
	 * @return Returns the currentPath.
	 */
	public Path getCurrentPath() {
		return currentPath;
	}
	/**
	 * @return Returns the savedPaths.
	 */
	public Vector getSavedPaths() {
		return savedPaths;
	}
	/**
	 * @return Returns the startGoalPath.
	 */
	public Path getStartGoalPath() {
		return startGoalPath;
	}
	public World getWorld()
	{
		return world;
	}
	
	
	public Path setStartGoalPath()
	{	
		Path ret = null;
		if(currentPath.getControlPoints().size() >= SPLINE_T)
		{
			startGoalPath = currentPath;
			ret = currentPath;
			world.setStartGoalPath(startGoalPath);
			this.repaint();
		}
		return ret;
	}
	
	public Path saveCurrentPath()
	{
		Path ret = null;
		if(currentPath.getControlPoints().size() >= SPLINE_T)
		{
			if(!savedPaths.contains(currentPath))
			{
				savedPaths.add(currentPath);
				ret = currentPath;
				this.repaint();
			}
		}
		return ret;
	}
	
	public Path resetCurrentPath()
	{
		currentPath = new Path(SPLINE_T, SPLINE_RESOLUTION);
		selectedControlPoint = null;
		this.repaint();
		return currentPath;
	}
	
	public void setCurrentPath(Path path)
	{
		currentPath = path;
		this.repaint();
	}
	
	public Path mergeStartGoalWithPath(Path path)
	{
		Path startGoal = getStartGoalPath();
		// TODO Auto-generated method stub
		Path compoundPath = new Path(3, 50);	//The goal start path (the two first points and the last point) + choromosome path

		Vector startGoalPoints = startGoal.getControlPoints();
		
		//Add first and second point
		compoundPath.addControlPoint((XYZ)startGoalPoints.firstElement(), false);
		compoundPath.addControlPoint((XYZ)startGoalPoints.elementAt(1), false);
		
		//Add intermediate points
		Vector otherPoints = path.getControlPoints();
		Iterator iter = otherPoints.iterator();
		while(iter.hasNext())
		{
			XYZ p = (XYZ)iter.next();
			compoundPath.addControlPoint(p, false);
		}	
		//Add goal point from startgoal & update curve!
		compoundPath.addControlPoint((XYZ)startGoalPoints.lastElement());
		return compoundPath;
	}
	
	private XYZ findControlPoint(XYZ p)
	{
		XYZ best = null;
		Iterator iter = currentPath.getControlPoints().iterator();
		while(iter.hasNext())
		{
			XYZ ctrlPoint = (XYZ)iter.next();
			if(best == null)
			{
				best = ctrlPoint;
			}
			double distBest = XYZ.distance(best, p);
			double distNew = XYZ.distance(ctrlPoint, p);
			
			if(distNew < distBest)
			{
				best = ctrlPoint;
			}
		}
		return best;
	}
	
	/*
	 * Paint the component
	 */
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);  //paint background
		draw(g);
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent arg0) 
	{
		if(arg0.isShiftDown())
		{
			if(arg0.getButton() == MouseEvent.BUTTON1)
			{
				int x = arg0.getX();
				int y = arg0.getY();
				currentPath.addControlPoint(new XYZ(x, y, 0));		
			}
			
			if(arg0.getButton() == MouseEvent.BUTTON3)
			{
				
			}
			this.repaint();
		}
		else
		{
			if(arg0.getButton() == MouseEvent.BUTTON1)
			{
				selectedControlPoint = findControlPoint(new XYZ(arg0.getPoint().x, arg0.getPoint().y, 0));
				this.repaint();
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent arg0) {
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent arg0) {
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent arg0) 
	{
		if(arg0.getButton() == MouseEvent.BUTTON1)
		{
			XYZ best = findControlPoint(new XYZ(arg0.getPoint().x, arg0.getPoint().y, 0));
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent arg0) 
	{
		if(arg0.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println(arg0.getPoint().toString());
		}

	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent arg0) {
		// TODO Auto-generated method stub
		if(arg0.isControlDown())
		{
			selectedControlPoint.x = arg0.getPoint().x;
			selectedControlPoint.y = arg0.getPoint().y;
			currentPath.update();
			this.repaint();
		}	
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent arg0) {
		// TODO Auto-generated method stub
		//classifyTerrain(arg0.getPoint().x, arg0.getPoint().y);
		
	}
	/* (non-Javadoc)
	 * @see yaes.application.empiricalpathplanner.IDrawable#draw(java.awt.Graphics)
	 */
	public void draw(Graphics g) {
		// TODO Auto-generated method stub

		//Draw image map
		Graphics2D g2d = (Graphics2D) g;
		g2d.drawImage(world.getImage(),null,null);
		
		//System.out.println(world.getKnownPaths().size() + " " + this.savedPaths.size());
		
		if(currentPath != null)
		{
			//Paint control points
			g.setColor(Color.RED);
			
			Iterator iter = currentPath.getControlPoints().iterator();
			while(iter.hasNext())
			{
				XYZ ctrlPoint = (XYZ)iter.next();
				g.fillOval((int)Math.round(ctrlPoint.x),(int)Math.round(ctrlPoint.y),8,8);
			}
			
			//Paint spline points
			g.setColor(Color.WHITE);
			if(currentPath.getCurve() != null)
			{
				for(int i=0; i<currentPath.getCurve().length;i++)
				{
					g.drawOval((int)Math.round(currentPath.getCurve()[i].x),(int)Math.round(currentPath.getCurve()[i].y),3,3);
				}
			}
			
			
			//Paint Spline line
			g.setColor(Color.RED);
			if(currentPath.getCurve() != null && currentPath.getControlPoints().size() > 2)
			{
				for(int i=0; i<currentPath.getCurve().length-1;i++)
				{
					Stroke stroke = new BasicStroke(2,BasicStroke.JOIN_ROUND, 
													BasicStroke.JOIN_ROUND, 0,
													new float[] { 1, 1 }, 0);
					g2d.setStroke(stroke);

					g.drawLine((int)Math.round(currentPath.getCurve()[i].x),(int)Math.round(currentPath.getCurve()[i].y),(int)Math.round(currentPath.getCurve()[i+1].x),(int)Math.round(currentPath.getCurve()[i+1].y));
				}
			}
		}
		
		//Paint start goal path
		g.setColor(Color.YELLOW);
		if(startGoalPath != null && startGoalPath.getCurve() != null)
		{
			Path tmpPath = startGoalPath;
			for(int i=0; i<tmpPath.getCurve().length-1;i++)
			{
				Stroke stroke = new BasicStroke(2,BasicStroke.JOIN_ROUND, 
												BasicStroke.JOIN_ROUND, 0,
												new float[] { 1, 1 }, 0);
				g2d.setStroke(stroke);
				g.drawLine((int)Math.round(tmpPath.getCurve()[i].x),(int)Math.round(tmpPath.getCurve()[i].y),(int)Math.round(tmpPath.getCurve()[i+1].x),(int)Math.round(tmpPath.getCurve()[i+1].y));
				Color oldColor = g.getColor();
				g.setColor(Color.LIGHT_GRAY);
				g.drawString("Start-Goal", (int)Math.round(tmpPath.getControlPointArray()[0].x), (int)Math.round(tmpPath.getControlPointArray()[0].y));
				g.setColor(oldColor);
			}
		}
		
		//Paint world path library lines
		Iterator iter = savedPaths.iterator();
		while(iter.hasNext())
		{
			Path tmpPath  = (Path) iter.next();
			g.setColor(Color.LIGHT_GRAY);
			
			if(tmpPath.getCurve() != null)
			{
				for(int i=0; i<tmpPath.getCurve().length-1;i++)
				{
					Stroke stroke = new BasicStroke(2,BasicStroke.JOIN_ROUND, 
													BasicStroke.JOIN_ROUND, 0,
													new float[] { 1, 1 }, 0);
					g2d.setStroke(stroke);

					g.drawLine((int)Math.round(tmpPath.getCurve()[i].x),(int)Math.round(tmpPath.getCurve()[i].y),(int)Math.round(tmpPath.getCurve()[i+1].x),(int)Math.round(tmpPath.getCurve()[i+1].y));
				
					Color oldColor = g.getColor();
					g.setColor(Color.LIGHT_GRAY);
					g.drawString(tmpPath.toString(), (int)Math.round(tmpPath.getControlPointArray()[0].x), (int)Math.round(tmpPath.getControlPointArray()[0].y));
					g.setColor(oldColor);
				}
			}
			
		}
		
		if(drawChromosomes)
		{
			//Paint Chromosomes
			g.setColor(Color.cyan);
			iter = world.getPathPlanner().getChromosomes().iterator();
			while(iter.hasNext())
			{
				Chromosome tmpChrom  = (Chromosome) iter.next();		
				if(tmpChrom.getChromosomePath().getCurve() != null)
				{
					for(int i=0; i<tmpChrom.getChromosomePath().getCurve().length-1;i++)
					{
						Stroke stroke = new BasicStroke(2,BasicStroke.JOIN_ROUND, 
														BasicStroke.JOIN_ROUND, 0,
														new float[] { 1, 1 }, 0);
						g2d.setStroke(stroke);
	
						g.drawLine((int)Math.round(tmpChrom.getChromosomePath().getCurve()[i].x),(int)Math.round(tmpChrom.getChromosomePath().getCurve()[i].y),(int)Math.round(tmpChrom.getChromosomePath().getCurve()[i+1].x),(int)Math.round(tmpChrom.getChromosomePath().getCurve()[i+1].y));
					
						//Color oldColor = g.getColor();
						//g.setColor(Color.LIGHT_GRAY);
						//g.drawString(tmpChrom.getChromosomePath().toString(), (int)Math.round(tmpChrom.getChromosomePath().getControlPointArray()[0].x), (int)Math.round(tmpChrom.getChromosomePath().getControlPointArray()[0].y));
						//g.setColor(oldColor);
					}
				}
			}
		}
		
		
		
		//	Paint evolved world path library lines based on start and goal path
		if(world.getStartGoalPath() != null)
		{
			g.setColor(Color.WHITE);
			iter = world.getPathPlanner().getChromosomes().iterator();
			while(iter.hasNext())
			{
				Chromosome tmpChrom  = (Chromosome) iter.next();		
				Path compoundPath = new Path(3, 50);	//The goal start path (the two first points and the last point) + choromosome path
				Vector startGoalPoints = world.getStartGoalPath().getControlPoints();
				
				//Add first and second point
				compoundPath.addControlPoint((XYZ)startGoalPoints.firstElement(), false);
				compoundPath.addControlPoint((XYZ)startGoalPoints.elementAt(1), false);
				//Add intermediate points
				Vector otherPoints = tmpChrom.getChromosomePath().getControlPoints();
				Iterator iter2 = otherPoints.iterator();
				while(iter2.hasNext())
				{
					XYZ p = (XYZ)iter2.next();
					compoundPath.addControlPoint(p, false);
				}	
				//Add goal point from startgoal & update curve!
				compoundPath.addControlPoint((XYZ)startGoalPoints.lastElement());
				
				currentPath = compoundPath;
				
				if(compoundPath.getCurve() != null)
				{
					for(int i=0; i<compoundPath.getCurve().length-1;i++)
					{
						Stroke stroke = new BasicStroke(2,BasicStroke.JOIN_ROUND, 
														BasicStroke.JOIN_ROUND, 0,
														new float[] { 1, 1 }, 0);
						g2d.setStroke(stroke);
	
						g.drawLine((int)Math.round(compoundPath.getCurve()[i].x),(int)Math.round(compoundPath.getCurve()[i].y),(int)Math.round(compoundPath.getCurve()[i+1].x),(int)Math.round(compoundPath.getCurve()[i+1].y));
					
						//Color oldColor = g.getColor();
						//g.setColor(Color.LIGHT_GRAY);
						//g.drawString(compoundPath.toString(), (int)Math.round(compoundPath.getControlPointArray()[0].x), (int)Math.round(compoundPath.getControlPointArray()[0].y));
						//g.setColor(oldColor);
					}
				}
				break;
			}
		}
		
		//Paint selected control point
		if(selectedControlPoint != null)
		{
			g.setColor(Color.ORANGE);
			g.fillOval((int)Math.round(selectedControlPoint.x),(int)Math.round(selectedControlPoint.y),8,8);	
		}
		
		if(currentPath != null && currentPath.getControlPoints().size() >= currentPath.getT())
		{
			prevCurveLenCost = curveLenCost;
			prevCurveCtrlPtsCost = curveCtrlPtsCost;
			prevCurveTerrainCost = curveTerrainCost;
			prevCurveCost = curveCost;
			
			curveLenCost = world.getPathPlanner().pathLengthCost(currentPath);
			curveCtrlPtsCost = world.getPathPlanner().controlPointDistanceCost(currentPath.getControlPointArray());
			curveTerrainCost = world.getPathPlanner().terrainCost(currentPath.getCurve());
			curveCost = curveLenCost + curveCtrlPtsCost + curveTerrainCost;
		
			curveLen = currentPath.getPathLenght();
			//curveCost = world.splineCost(currentPath.getCurve());
		}
		g.setColor(Color.WHITE);
		g.drawString("Spline Lenght: " + curveLen, 20,20);
		g.drawString("Spline Cost: " + curveCost + "(" + (curveCost-prevCurveCost)  + ")", 20,35);
		g.drawString("Spline Lenght Cost: " + curveLenCost + "(" + (curveLenCost-prevCurveLenCost) + ")", 20,50);
		g.drawString("Spline Point Cost: " + curveCtrlPtsCost + "(" + (curveCtrlPtsCost-prevCurveCtrlPtsCost) + ")", 20,65);
		g.drawString("Spline Terrain Cost: " + curveTerrainCost + "(" + (curveTerrainCost-prevCurveTerrainCost) + ")", 20,80);
	}	
}
