package yaes.application.empiricalpathplanner.spline;


/*
 * Created on Oct 11, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */

/**
 * @author Linus Luotsinen
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class BSpline 
{
	private static double splineBlend(int k, int T, int[] knots, double interval)
	{
		double value;
		if (T == 1) 
		{	
			if ((knots[k] <= interval) && (interval < knots[k+1]))
			{
				value = 1;
			}
			else
			{
				value = 0;
			}
		}
		else
		{
			if ((knots[k+T-1] == knots[k]) && (knots[k+T] == knots[k+1]))
			{
				value = 0;
			}
			else if (knots[k+T-1] == knots[k])
			{
				value = (knots[k+T] - interval) / (knots[k+T] - knots[k+1]) * splineBlend(k+1,T-1,knots,interval);
			}
			else if (knots[k+T] == knots[k+1])
			{
				value = (interval - knots[k]) / (knots[k+T-1] - knots[k]) * splineBlend(k,T-1,knots,interval);
			}
			else
			{
				value = (interval - knots[k]) / (knots[k+T-1] - knots[k]) * splineBlend(k,T-1,knots,interval) + 
		                 (knots[k+T] - interval) / (knots[k+T] - knots[k+1]) * splineBlend(k+1,T-1,knots,interval);
			}
		}
		return(value);
	}
	
	/*
	 * knots = the knots (must be allocated outside this function) a.k.a u
	 * n = control points - 1
	 * t = degree of polynomial
	 */
	private static void splineKnots(int []knots, int N, int T)
	{
	   int j;
	
	   for (j=0;j<=N+T;j++)
	   {
	      if (j < T)
	      {
	      	knots[j] = 0;
	      }
	      else if (j <= N)
	      {
	      	knots[j] = j - T + 1;
	      }
	      else if (j > N)
	      {
	      	knots[j] = N - T + 2;
	      }
	   }
	}
	
	private static void splinePoint(int[] knots, int N, int T, double interval, XYZ[] controlPoints, XYZ output)
	{
	   int k;
	   double b;

	   output.x = 0;
	   output.y = 0;
	   output.z = 0;

	   for (k=0;k<=N;k++) {
	      b = splineBlend(k,T,knots,interval);
	      output.x += controlPoints[k].x * b;
	      output.y += controlPoints[k].y * b;
	      output.z += controlPoints[k].z * b;
	   }
	}
	
	private static void splineCurve(XYZ[] controlPoints, int N, int[] knots, int T, XYZ[] outputs, int resolution)
	{
	   int i;
	   double interval,increment;

	   interval = 0;
	   increment = (N - T + 2) / (double)(resolution - 1);
	   for (i=0;i<resolution-1;i++) 
	   {
	      splinePoint(knots,N,T,interval,controlPoints,outputs[i]);
	      interval += increment;
	   }
	   outputs[resolution-1] = controlPoints[N];
	}
	
	/*
	 * T = Polynomial degree
	 * resolution = number of points in output
	 * inp = control points
	 */
	public static XYZ[] createSpline(final int T, int resolution, XYZ[] controlPoints)
	{
		if(controlPoints.length <= 0)
		{
			return null;
		}
		
		final int N = controlPoints.length - 1;	
		int[] knots = new int[N+T+1];
		XYZ[] outp = new XYZ[resolution];
		
		for(int i=0; i<resolution; i++)
		{
			outp[i] = new XYZ(0.0,0.0,0.0);
		}
		
		//Determine the knots
		splineKnots(knots,N,T);
		
		splineCurve(controlPoints,N,knots,T,outp,resolution);	
		return outp;
	}
	
	//	Lenght between the curve points
	public static double splineLength(XYZ[] curve)
	{
		double dist = 0.0;
		
		if(curve != null)
		{
			for(int i=0; i<curve.length-1;i++)
			{
				dist += XYZ.distance(curve[i],curve[i+1]);
			}
		}
		return dist;
	}
	
	public XYZ[] test()
	{
		final int N = 3;
		final int T = 3;
		final int RESOLUTION = 200;
		
		XYZ[] inp = new XYZ[N+1];
		inp[0] = new XYZ(0.0,0.0,0.0);
		inp[1] = new XYZ(100.0,0.0,300.0);
		inp[2] = new XYZ(200.0,0.0,100.0);
		inp[3] = new XYZ(400.0,0.0,400.0);
		
		int[] knots = new int[N+T+1];
		XYZ[] outp = new XYZ[RESOLUTION];
		
		for(int i=0; i<RESOLUTION; i++)
		{
			outp[i] = new XYZ(0.0,0.0,0.0);
		}
		
		splineKnots(knots,N,T);
		splineCurve(inp,N,knots,T,outp,RESOLUTION);
		
		for(int i=0;i<RESOLUTION;i++)
		{
			System.out.println(outp[i].x + " " + outp[i].y + " " + outp[i].z);
		}
		
		return outp;
	}
}
