/*
 * Created on Mar 7, 2005
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package yaes.application.marketgrid;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import yaes.application.marketgrid.brokers.Offer;
import yaes.application.marketgrid.marketmakers.SimpleInvestmentStrategy;
import yaes.application.marketgrid.marketmakers.MarketMakerAllocation;
import yaes.application.marketgrid.marketmakers.MarketMakerConversation;
import yaes.application.marketgrid.money.MarketAgent;
import yaes.application.marketgrid.money.Bank;
import yaes.application.marketgrid.resource.AllocationDescriptor;
import yaes.application.marketgrid.resource.simple.SimplePricePolicy;
import yaes.framework.agent.AbstractCommunicatingAgent;
import yaes.framework.agent.ExtendedMessage;
import yaes.framework.world.World;
import yaes.ui.text.TextUi;

/**
 * @author Lotzi Boloni
 *
 * To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class MarketMaker extends MarketAgent {

    private ArrayList<MarketMakerAllocation> resources;
    private Hashtable<String, MarketMakerConversation> conversations =
      new Hashtable<String, MarketMakerConversation>();
    private SimplePricePolicy pricePolicy;
    private SimpleInvestmentStrategy investmentStrategy = new SimpleInvestmentStrategy();
    private String investmentConversationId;
    /**
     * @param name
     */
    public MarketMaker(String name, World world, int initMoney) {
        super(name, world, initMoney);
        resources = new ArrayList<MarketMakerAllocation>();
        pricePolicy = new SimplePricePolicy();
        investmentConversationId = name + "_BUY_RESOURCES";
    }

    /* (non-Javadoc)
     * @see yaes.framework.agent.IAgent#action(yaes.framework.world.World)
     */
    public void action() {
        TextUi.println("ACTION at MarketMaker " + getName());

      boolean investment_encountered = false;
      // process the remaining messages
      while(true) {
              ExtendedMessage message = getExtendedMessage();
              if (message == null) {
                break;
              }
        if(message.getConversation().equals(investmentConversationId))
          investment_encountered = true;
        MarketMakerConversation conversation = conversations.get(message.getConversation());
        if(conversation == null){
          conversation = new MarketMakerConversation(this, message);
          conversations.put(message.getConversation(), conversation);
        }
        conversation.action(message);
      }

      if( ! investment_encountered)
        initInvestmentCycle();

      // clean up the terminated conversations
      for (Enumeration<String> iter = conversations.keys(); iter.hasMoreElements();) {
        String key = iter.nextElement();
        MarketMakerConversation element = conversations.get(key);
        if (element.isFinished()) {
          conversations.remove(key);
        }
      }
    }
    /**
        Matches resource
        @param requestedResource
        @return price of resource
     */
    public double getOffer(AllocationDescriptor requestedResource){
      for(MarketMakerAllocation resource : resources){
        if(resource.isMatch(requestedResource.resourceParameters) &&
          resource.getAvailableQuantity() >= requestedResource.quantity){
          return pricePolicy.getPrice(requestedResource.quantity, resource.getPricingRules());
        }
      }
      return -1.0;
    }
    /**
        Allocates resource, once when client agreed on price
        @param requestedResource
        @return provider
     */
    public String getProviderAndAllocateResource(AllocationDescriptor requestedResource){

      for(MarketMakerAllocation resource : resources){
        if(resource.isMatch(requestedResource.resourceParameters) &&
          resource.getAvailableQuantity() >= requestedResource.quantity){
          resource.allocateResource(requestedResource.quantity);
          return resource.getProvider();
        }
      }
      return null;
    }
    /**
        Exceutes purchase of resource from provider
        @param offer
     */
    public void purchaseResource(Offer purchase){

      boolean resource_merged = false;
      for(MarketMakerAllocation resource : resources){
        if(resource.isMatch(purchase.allocationDescriptor.resourceParameters) &&
           resource.getProvider().equals(purchase.provider)){
          resource.mergeResource(purchase.allocationDescriptor);
          resource_merged = true;
          break;
        }
      }
      if( ! resource_merged){
        MarketMakerAllocation resource = new MarketMakerAllocation(purchase.provider,
                                                                  purchase.allocationDescriptor);
        resources.add(resource);
      }
      double value = purchase.allocationDescriptor.expectedUnitPrice *
                     purchase.allocationDescriptor.quantity;
      Bank.getInstance().transfer(getName(), purchase.provider, (int)value);
    }
    /**
        @return Retrieves amount available to invest
     */
    public double getInvestment(){
      return investmentStrategy.invest(account.getValue(), resources);
    }
    /**
        Ask investments advisor to filter out offers. It uses current status
        of assets to provide accurate value to invest
        @parameter providersOffers
     */
    public void filterOffers(List<Offer> providersOffers){
      investmentStrategy.filterOffers(providersOffers, account.getValue(), resources);
    }
    /**
        Starts investment cycle by sending message to itself
     */
    private void initInvestmentCycle(){
             // start check investment level and buying loop
      ExtendedMessage message = new ExtendedMessage(getName(), getName(),
                                                    ExtendedMessage.Performative.REQUEST);
      message.setConversation(investmentConversationId);
      receiveMessage(message);
    }
}
