package yaes.application.marketgrid.providers;

import java.util.Collection;
import java.util.Hashtable;

import yaes.application.marketgrid.resource.IResource;

/**
      This class is intended to manage resource allocations on provider's side.
      It sees all resources as continuum, assuming that execution requests are
      normally consume only fractions of whole capacity.
      Preventing requests that are larger than max capacity of items that make
      continuum are left to Execution manager.

      NOTE: It assumes that all resources are identified with the same set of
            resource parameters !!!!
 */
public class ProviderAllocation{

  private MultiProvider provider;
  private Hashtable<String, IResource> resources = new Hashtable<String, IResource>();
  //
  private Hashtable<String, ProviderAllocationChunk> lockedChunks =
              new Hashtable<String, ProviderAllocationChunk>();

  private Hashtable<String, ProviderAllocationChunk> allocationChunks =
              new Hashtable<String, ProviderAllocationChunk>();

  public ProviderAllocation(MultiProvider provider){
    this.provider = provider;
    allocationChunks.put(provider.getName(),
          new ProviderAllocationChunk(0.0, ProviderAllocationChunk.AllocationState.FREE));
  }
  /**
      Adds new resource
      @param resource
   */
  public void addResource(IResource resource){
    resources.put(resource.getResourceId(), resource);
    ProviderAllocationChunk current_chunk = allocationChunks.get(provider.getName());
    current_chunk.quantity += resource.getAvailableQuantity();
  }
  /**
      @return currently not allocated (non-commited) quantity across all resources
   */
  public double getFreeResourcesQuantity(){
    //@FIXME  needs resource identification
    ProviderAllocationChunk free_chunk = allocationChunks.get(provider.getName());
    return free_chunk.quantity;
  }
  /**
        Allocates commited resource for the marketmakers
   */
  public boolean allocateCommited(double quantity, String user){

    ProviderAllocationChunk free_chunk = allocationChunks.get(provider.getName());
    if(! (free_chunk.quantity >= quantity))
      return false;

    ProviderAllocationChunk current_chunk = allocationChunks.get(user);
    if(current_chunk == null){
      current_chunk = new ProviderAllocationChunk(0.0,
                          ProviderAllocationChunk.AllocationState.COMMITED);
      allocationChunks.put(user, current_chunk);
    }
    free_chunk.quantity -= quantity;
    current_chunk.quantity += quantity;
    return true;
  }
  /**
      @param quantity
      @param user owner of resources against who it will lock
      @param lock identifier of lock source: conversationId
   */
  public boolean lockResource(double quantity, String user, String lock){

    ProviderAllocationChunk chunk = allocationChunks.get(user);
    // available quantity
    if( ! (chunk.quantity >= quantity))
      return false;
    // source already has lock ?
    if(lockedChunks.get(lock) != null)
      return false;

    chunk.quantity -= quantity;

    ProviderAllocationChunk lock_chunk = chunk.getCopy();
    lock_chunk.quantity = quantity;
    lock_chunk.lockingSource = lock;
    // save user to who allocation has to be returned
    lock_chunk.user = user;
    lockedChunks.put(lock, lock_chunk);
    return true;
  }
  /**
      Unlocks locked resource
      @param lock identifier
   */
  public void unlockResource(String lock){
    ProviderAllocationChunk lock_chunk = lockedChunks.get(lock);
    if(lock_chunk != null)
      lockedChunks.remove(lock);
    ProviderAllocationChunk chunk = allocationChunks.get(lock_chunk.user);
    chunk.quantity += lock_chunk.quantity;
  }
}
