package yaes.application.marketgrid.resource;

import java.util.Collection;
import java.util.List;

/**
 *  Describes single element of conditions that describe negotiated resource in terms of
 *  distingushing parameters. As means of negotiations, it contains range of values that
 *  are requested or offered so that negotiating sides can make decision e.g. acceptable
 *  frequency is 50Hz or 60Hz, whatever resurce provider has.
 */

public class AllocationRequestParameter{

  /**
   *   Unique identification of parameter within set of parameters that describe
   *   resource in distinguising way. E.g. PROCESSOR_SPEED, DISK_TRANSFER_RATE
   *
   *  NOTE: it is identical to value of an instance ResourceParameter for given resource
   *
   */
  public String parameterId;
  /**
   *   Lower boundary for numeric value as measure (if applicable) for given parameter
   */
  public double numericLowerValue;
  /**
   *   Upper boundary for numeric value as measure (if applicable) for given parameter
   */
  public double numericUpperValue;
  /**
   *   Lower AND upper boundary for string value (if applicable) for given parameter
   *   (for cases where resource may have variety of values, but denoting schema ensures
   *    validity e.g. Pentium-III, Pentium-IV)
   */
  public List<String> stringIntervalValues;
  /**
   *   List of acceptable values for string value (if applicable) for given parameter
   *   (e.g. 802.2, 802.11)
   */
  public List<String> stringItemValues;

  public boolean matchesStringInterval(List<String> stringIntervalValues){
    if(stringIntervalValues == null &&
       this.stringIntervalValues == null)
       return true;
     //@FIXME implement comparison
     return true;
  }

  public boolean matchesStringItem(List<String> stringItemValues){
    if(stringItemValues == null &&
       this.stringItemValues == null)
       return true;
     //@FIXME implement comparison
     return true;
  }

  public boolean matchesNumeric(double numericLowerValue, double numericUpperValue){
     if(this.numericLowerValue <= numericLowerValue &&
        this.numericUpperValue >= numericUpperValue)
        return true;
     return false;
  }

  /**
      Verifies if parameters1 are contained within parameters2
   */
  public static boolean matches(Collection<AllocationRequestParameter> parameters1,
                                Collection<AllocationRequestParameter> parameters2){

    for(AllocationRequestParameter param1 : parameters1){
      boolean idmatches = false;
      for(AllocationRequestParameter param2 : parameters2){
        if(param1.parameterId.equals(param2.parameterId)){
          idmatches = true;
          if(! (param2.matchesNumeric(param1.numericLowerValue, param1.numericUpperValue) &&
                param2.matchesStringInterval(param1.stringIntervalValues) &&
                param2.matchesStringItem(param1.stringItemValues)))
            return false; // parameter values not contained in respective parameters2
        }
      }
      if( ! idmatches)
          return false; // parameterID not contained in parameters2
    }
    return true;
  }
}

