package yaes.application.marketgrid.resource.simple;

import java.util.ArrayList;
import java.util.Collection;

import yaes.application.marketgrid.resource.IPricePolicy;
import yaes.application.marketgrid.resource.IPricingRule;
import yaes.application.marketgrid.resource.UsageChunk;

/**
    Simple implementation of pricing policy ignores usage patterns. It only applies
    price rules as function of quantity
 */
public class SimplePricePolicy
       implements IPricePolicy{

  protected Collection<IPricingRule>rules;

  public SimplePricePolicy(){
  }

  public SimplePricePolicy(IPricingRule rule){
    rules = new ArrayList<IPricingRule>();
    rules.add(rule);
  }

  public double getPrice(double quantity){
    return calculatePrice(quantity, rules);
  }

  public double getPrice(double quantity, Collection<IPricingRule>price_rules){
    return calculatePrice(quantity, price_rules);
  }

  public double getPrice(Collection<UsageChunk>usage){
    double value = 0.0;
    for(UsageChunk chunk: usage){
      value += calculatePrice(chunk.quantity, rules);
    }
    return value;
  }

  public double getPrice(Collection<UsageChunk>usage, Collection<IPricingRule>price_rules){
    double value = 0.0;
    for(UsageChunk chunk: usage){
      value += calculatePrice(chunk.quantity, price_rules);
    }
    return value;
  }

  public void setPricingRules(Collection<IPricingRule>rules){
    this.rules = rules;
  }

  protected double calculatePrice(double quantity, Collection<IPricingRule>price_rules){
    if(price_rules == null)
      return 0.0;
    double value = 0.0;
    for(IPricingRule rule : price_rules){
      switch(rule.getType()){
        case FLAT:
          value += rule.getRuleValue();
          break;
        case LINEAR:
          value += rule.getRuleValue() * quantity;
          break;
        case QUADRATIC:
          value += rule.getRuleValue() * quantity * quantity;
          break;
      }
    }
    return value;
  }
}
