package yaes.application.marketgrid.subprotocols;

import yaes.application.marketgrid.clients.Task;
import yaes.application.marketgrid.resource.AllocationDescriptor;
import yaes.framework.agent.AbstractCommunicatingAgent;
import yaes.framework.agent.ExtendedMessage;

public class SubProtProviderExecution extends AbstractSubprotocol {

	public static final String PARAM_REQUESTED_RESOURCES = "RequestedResourcesFree";
    public static final String PARAM_TASK = "TaskFree";

    public static final String MSG_EXECUTION_REQUEST = "ExecutionRequestFree";
    public static final String MSG_EXECUTION_ACCEPT = "ExecutionAcceptFree";
    public static final String MSG_EXECUTION_REJECT = "ExecutionRejectFree";
    public static final String MSG_EXECUTION_FINISHED = "ExecutionFinishedFree";

	public SubProtProviderExecution(AbstractCommunicatingAgent agent) {
		super("ProviderExecution", agent);
	}

	public ExtendedMessage createExecutionRequestMessage(String destination, AllocationDescriptor descriptor, Task task) {
        ExtendedMessage message = new ExtendedMessage(destination, agent.getName(),
                ExtendedMessage.Performative.REQUEST);
        message.setSubprotocol(getName());
        message.setSubprotocolMessage(MSG_EXECUTION_REQUEST);        
        message.setValue(
                PARAM_REQUESTED_RESOURCES,
                descriptor);
        message.setValue(PARAM_TASK, task);
        return message;
    }

    public ExtendedMessage createExecutionAcceptMessage(String destination, String conversation) {
        ExtendedMessage message = new ExtendedMessage(destination, agent.getName(), 
                ExtendedMessage.Performative.ACCEPT);
        message.setConversation(conversation);
        message.setSubprotocol(getName());
        message.setSubprotocolMessage(MSG_EXECUTION_ACCEPT);        
        return message;
    }
    
    public ExtendedMessage createExecutionRejectMessage(String destination, String conversation) {
        ExtendedMessage message = new ExtendedMessage(destination, agent.getName(), 
                ExtendedMessage.Performative.REJECT);
        message.setConversation(conversation);
        message.setSubprotocol(getName());
        message.setSubprotocolMessage(MSG_EXECUTION_REJECT);        
        return message;
    }

    public ExtendedMessage createExecutionFinishedMessage(String destination, String conversation) {
        ExtendedMessage message = new ExtendedMessage(destination, agent.getName(), 
                ExtendedMessage.Performative.FINISHED);
        message.setConversation(conversation);
        message.setSubprotocol(getName());
        message.setSubprotocolMessage(MSG_EXECUTION_FINISHED);        
        return message;
    }
}
