package yaes.application.uavaltclient.agents;

import yaes.application.uavaltclient.missionmgr.MissionManager;
import yaes.application.uavaltclient.nav.Pos;
import yaes.application.uavaltclient.simulation.SimException;
import yaes.application.uavaltclient.util.Props;
import yaes.framework.agent.GenericMessage;
import yaes.framework.world.World;
import yaes.application.uavaltclient.message.MessageLibrary;
import yaes.application.uavaltclient.message.MsgType;


/**
 * Bomber uav, capable of destroying ground targets.
 * 
 * @author dejongp
 */
public class Bomber extends UAV
{
	// Bomber configuration constants (bomber is leave navigable UAV) 
	private static final double UAV_MAX_HDG_DELTA = Props.getDouble("BOMBER_UAV_MAX_HDG_DELTA"); //max heading change per tick (degrees)
  private static final double UAV_MAX_ALT_DELTA = Props.getDouble("BOMBER_UAV_MAX_ALT_DELTA"); //(OpenGL coordinates/tick)
	
  /**
   * @param name uav's name, used for messaging
   */
  public Bomber(String name, World world, double firepower, double visibility, double maxVelocity, double stealth)
  {
  	super(name, world, firepower, visibility, maxVelocity, stealth);
  	type = UAVType.UAV_TYPE_BOMBER;
  	
  	try
    { 
			type = UAVType.UAV_TYPE_BOMBER;
    	sim.init(name, type);
    }
    catch(SimException e) 
    {
    	System.err.println("COULD NOT REGISTER " + name + " WITH SIMULATION: "+ e);
    	e.printStackTrace();
    }
  }
  
	/**
	 * Actions unique to bomber vehicles.
	 * 
	 * @param world YAES context
	 * @see yaes.framework.agent.IAgent#action(yaes.framework.world.World)
	 */
	public void action()
	{
		super.action();

		try
		{
			if (mgr != null && mgr.isBombsAwayFlag())
			{
				sim.dropBomb(); // stop worrying, love the bomb
				
				for (int i=0; i<squadronMembers.length; i++)
				  if (! name.equals(squadronMembers[i]))
				  	MessageLibrary.sendResumeMissionMsg(name, squadronMembers[i]);
				  	
				bombNegotiationFlag = false;
				mgr.resumeMission();
			}			
		}
		catch (SimException e) {}
		
	}  

	
	/**
	 * Obtain the maximum heading change per tick for this UAV.
	 * 
	 * @return max heading change per tick (degrees)
	 */
	public double getMaxHdgDelta()
	{
		return UAV_MAX_HDG_DELTA;
	}
  
	/**
	 * Obtain the maximum altitude change per tick for this UAV.
	 * 
	 * @return max heading change per tick (degrees)
	 */
	public double getMaxAltDelta()
	{
		return UAV_MAX_ALT_DELTA;
	}
	  
	/**
	 * Req'd by UAV superclass.  Processes messages not handled by superclass.
	 *
	 * @param msg an incoming message
	 */  
	protected void processMessage(GenericMessage msg, String msgType)
	{
		// this uav has been assigned bomber as a result of a bomber negotiation
		if (MsgType.MSG_ASSIGN_BOMB.equals(msgType))
		{
			System.out.println(name + " assigned bomber.");
			mgr.setCurrentMode(MissionManager.MODE_BOMBRUN);
			Pos mypos = mgr.getCurrentPosition();
			Pos target = (Pos) msg.getValue("pos");
			///TBD: target.setY(mypos.getY() - 20);//decend 20 units for bomb run
			target.setY(mypos.getY());//decend 20 units for bomb run
			mgr.setBombPosition(target);
		}
		
		// respond to a bomber negotiation call-for-proposal
		else if (MsgType.MSG_BOMBER_NEEDED.equals(msgType))
		{
			bombNegotiationFlag = true;
				
			MessageLibrary.sendProposeBombMsg(name, 
																				(String) msg.getValue("sender"), 
																				(Pos) mgr.getCurrentPosition());
		}

    else
    	System.err.println("MESSAGE TYPE NOT RECOGNIZED: " + msgType);
    
	}   
	
}
