package yaes.application.uavaltclient.agents;

import yaes.application.uavaltclient.message.MessageLibrary;
import yaes.application.uavaltclient.message.MsgType;
import yaes.application.uavaltclient.simulation.SimException;
import yaes.application.uavaltclient.util.Props;
import yaes.framework.agent.GenericMessage;
import yaes.framework.world.World;


/**
 * Subclass of UAV strategy for Fighter behavior.
 * 
 * @author dejongp
 */
public class Normal extends UAV
{
	// Normal UAV configuration constants
	private static final double UAV_MAX_HDG_DELTA = Props.getDouble("NORMAL_UAV_CRUISE_VEL"); //max heading change per tick (degrees) WAS 3.5
	private static final double UAV_MAX_ALT_DELTA = Props.getDouble("NORMAL_UAV_CRUISE_VEL"); //(OpenGL coordinates/tick) WAS 1.0
	
	/**
	 * @param name uav's name, used for messaging
	 */
	public Normal(String name, World world, double firepower, double visibility, double maxVelocity, double stealth)
	{
		super(name, world, firepower, visibility, maxVelocity, stealth);
		type = UAVType.UAV_TYPE_NORMAL;
  	
		try
		{ 
			type = UAVType.UAV_TYPE_NORMAL;
			sim.init(name, type);
		}
		catch(SimException e) 
		{
			System.err.println("COULD NOT REGISTER " + name + " WITH SIMULATION: "+ e);
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Obtain the maximum heading change per tick for this UAV.
	 * 
	 * @return max heading change per tick (degrees)
	 */
	public double getMaxHdgDelta()
	{
		return UAV_MAX_HDG_DELTA;
	}	
	
	/**
	 * Obtain the maximum altitude change per tick for this UAV.
	 * 
	 * @return max heading change per tick (degrees)
	 */
	public double getMaxAltDelta()
	{
		return UAV_MAX_ALT_DELTA;
	}

	/**
	 * Actions unique to normal vehicles.
	 * 
	 * @param world YAES context
	 * @see yaes.framework.agent.IAgent#action(yaes.framework.world.World)
	 */
	public void action()
	{
		super.action();
	}  
  
	/**
	 * Req'd by UAV superclass.  Processes messages not handled by superclass.
	 *
	 * @param msg an incoming message
	 */  
	protected void processMessage(GenericMessage msg, String msgType)
	{
		// respond to a bomber negotiation call-for-proposal
		if (MsgType.MSG_BOMBER_NEEDED.equals(msgType))
		{
			bombNegotiationFlag = true;
			MessageLibrary.sendDeclineBombMsg(name, (String) msg.getValue("sender"));  
		}   

		else
			System.err.println("MESSAGE TYPE NOT RECOGNIZED: " + msgType);
    
	}

}