package yaes.application.uavaltclient.coalitionformation;



import java.math.BigInteger;



/**

 * A depth-first search algorithm which limits the search depth (uses same interface as DepthFirst).  

 * After all possibilities are exhausted, the depth is incremented and the algorithm is re-run.  

 * nextCombo() will not return a combination which has already been attempted (based on combo size).

 * 

 * @author dejongp

 */

public class DFIDAlgorithm extends DepthFirstAlgorithm

{

  // THIS VALUE HAS PROFOUND EFFECT ON ALGORITHMIC EFFICIENCY

	private static final int STARTING_DEPTH = 3;

	

	private int depth = STARTING_DEPTH;



	/**

	 * For testing

	 * 

	 * @param args

	 */

	public static void main(String args[])

	{

		DepthFirstAlgorithm df = new DFIDAlgorithm(8);

		//df.addBadBranch(new int[]{0});

		df.addBadBranch(new int[]{7});

		//df.addBadBranch(new int[]{1,2,7});

		//df.addBadBranch(new int[]{0,1,2,3,4,5,6,7});

		

				

		int[] result = df.nextCombo();

		

		while (result != null)

		{

			printCombination(result);

			result = df.nextCombo();

		}

		

		System.out.println("NUMBER COMBINATIONS: " + df.getCombinationCount().toString());

	}

	

	

	/**

	 * Chained constructor

	 * 

	 * @param n number of items to generate combinations of

	 */

  public DFIDAlgorithm(int n)

  {

    super(n);

  }

  

	/**

	 * Generate the next combination.

	 * 

	 * @return int array containing next combination, or null if search is complete

	 */

	public int[] nextCombo()

	{

		do

		{

			iterate();

			if (in == -1) return null;

			

			if (isBadBranch())

			{					

				if (! nextBranch())

				{

					if (depth < n)

					{

					  depth ++;

					  //System.out.println("NEW DEPTH INSIDE badBranch=" + depth);

					  iterate();

					}

					else

						return null;					

				}

			}

		}

		while (! isSizeOK());

		

		counter = counter.add(BigInteger.ONE);

			

		int[] retarray = new int[in+1];

		System.arraycopy(ar, 0, retarray, 0, in+1);

		

		return retarray;

	}



	/**

	 * Perform one iteration of algorithm

	 */

	private void iterate()

	{

		// first iteration

		if (in == -1)	

		{

			ar[++in] = -1;

			ar[in] ++;

			return;

		}

				

		// deepen

		if ((in < (depth-1)) && (ar[in] < (n-1)))

		{		

			ar[++in] = ar[in - 1];

			ar[in] ++;

			return;

		}

		

		// backtrack

		for (; in>=0; in--)

		{

			if (ar[in] < (n-1))

			{

				ar[in] ++;

				return;

			}

		}		

		

		// change depth

		if (depth < n)

		{

			depth ++;

			//System.out.println("NEW DEPTH=" + depth);

				

			iterate();

			return;

		}

		

		// search complete

		in = -1;

	}



	/**

	 * Checks if the size of this combination warrants a return to the caller (e.g., they do not 

	 * need to see combinations of size=3 if they already did on the last pass though the 

	 * algorithm).

	 * 

	 * @return true if combination size is acceptable, false otherwise

	 */

	private boolean isSizeOK()

	{

		if ((depth == STARTING_DEPTH) || (in == (depth-1)) || (in == -1))

			return true;

		  		

		return false; 

	}

}

