package yaes.application.uavaltclient.coalitionformation;



import java.math.BigInteger;

import java.util.ArrayList;

import java.util.List;



import yaes.application.uavaltclient.coalitionformation.ga.BitmapUtils;

import yaes.application.uavaltclient.coalitionformation.ga.ChromosomeImpl;

import yaes.application.uavaltclient.coalitionformation.ga.CrossoverImpl;

import yaes.application.uavaltclient.coalitionformation.ga.FitnessFunctionImpl;

import yaes.application.uavaltclient.coalitionformation.ga.MutationImpl;

import yaes.application.uavaltclient.coalitionformation.timing.*;

import yaes.framework.algorithm.geneticalgorithm.GeneticPool;

import yaes.framework.algorithm.geneticalgorithm.ICrossover;

import yaes.framework.algorithm.geneticalgorithm.IFittnessFunction;

import yaes.framework.algorithm.geneticalgorithm.IMutation;

import yaes.application.simplegenetic.RandomSelection;



/**

 * @author dejongp

 */

public class GACoalitionFormation extends CoalitionFormation

{

	private int numGenerations = 150; //TBD: play with this number.  150 is better than 100, which is way better than 50...

	private int origCandidatesSize = 0;

	private long totNumCombosConsidered = 0;

	

	/**

	 * Constructor

	 */

	public GACoalitionFormation()

	{

		super();

	}

	

  /**

   * 

   */

  public void formCoalitions()

  {

		totalTime.start();

		origCandidatesSize = candidates.size();

		List unassignedMissions = new ArrayList();

		

		for (int i=0; i<missions.size(); i++)

		{

			if (candidates.size() == 0)

			{

				//System.out.println("Cannot continue without candidates");

				return;

			}

			

		  MissionRepresentation mRep = (MissionRepresentation) missions.get(i);

		  //System.out.println("GA: " + mRep.getName());

			Timing missionTiming = (Timing) missionFirstSolutionTimes.get(mRep.getName());

			missionTiming.start();

			

			if (qualityTimingFlag)

				startQualityTimers();

					

			IFittnessFunction fitnessFunction = new FitnessFunctionImpl(mRep, candidates);

			ICrossover crossover = new CrossoverImpl();

		  IMutation mutatation = new MutationImpl();

		  

			/// old: GeneticPool pool = new GeneticPool(fitnessFunction, crossover, mutatation);

			GeneticPool pool = new GeneticPool(fitnessFunction, new RandomSelection(), crossover, mutatation);

			pool.setMutationRate(.125);

			pool.setCrossoverRate(.125);

			

			for (int j=0; j<candidates.size(); j++)

			  pool.getChromosomes().add(new ChromosomeImpl(candidates.size()));



	    boolean firstSolutionFound = false;

			int bestCost = Integer.MAX_VALUE; 

			int qualityIndex = 0;

				

			for(int k=0; k < numGenerations; k++) 

			{

				//System.out.println("\nGENERATION " + k);

				pool.compete();



			  ChromosomeImpl best = (ChromosomeImpl) pool.getChromosomes().get(0);

				List coalitionList = BitmapUtils.formCoalitionList(best.getSquadBitmap(), candidates);

				int cost = (int) Math.round(CostFunctions.cost(coalitionList, mRep));

				

			  if (cost >= 0)

			  {

			  	if (! firstSolutionFound)

			  	{	

						missionTiming.finish();

				  	firstSolutionFound = true;

				  	if (breakAfterFirstSolution)

				  		break;

			  	}

			  	

					if (qualityTimingFlag && (cost < bestCost))

					{

						if (qualityIndex < MAX_QUALITY_TIMING_CNT)

						{

							qualityTimingObjects[qualityIndex].finish();

							try

							{

								qualityTime[qualityIndex] = qualityTimingObjects[qualityIndex].getElapsedTime();

							}

							catch (TimingException e)

							{

								e.printStackTrace();

							}

							qualityCost[qualityIndex] = cost;

						}

						else

							System.err.println("ERROR... NO MORE QUALITY TIMERS LEFT");

						

						qualityIndex ++;

						bestCost = cost;

					}

					

			  }

				

			}		



      ChromosomeImpl winner = (ChromosomeImpl) pool.getChromosomes().get(0);

			List coalitionList = BitmapUtils.formCoalitionList(winner.getSquadBitmap(), candidates);



      long numMissionCombos = numGenerations * candidates.size();

			missionTiming.addToCombinationCount(numMissionCombos);

			totNumCombosConsidered += numMissionCombos;

			

      if (CostFunctions.cost(coalitionList, mRep) >= 0.0)

      {

				coalitions.add(new Coalition(coalitionList, mRep));

				candidates.removeAll(coalitionList);

      }

      else

      {

				unassignedMissions.add(mRep);

      }

      

		}

		

		missions = unassignedMissions;

		totalTime.finish();

  }



  /**

   * 

   */

  public BigInteger getNumCombosConsidered()

  {

		return BigInteger.valueOf(totNumCombosConsidered);

  }

}

