package yaes.application.uavaltclient.coalitionformation.ga;



import java.util.List;



import yaes.application.uavaltclient.coalitionformation.CostFunctions;

import yaes.application.uavaltclient.coalitionformation.MissionRepresentation;

import yaes.framework.algorithm.geneticalgorithm.IChromosome;

import yaes.framework.algorithm.geneticalgorithm.IFittnessFunction;



/**

 * @author dejongp

 */

public class FitnessFunctionImpl implements IFittnessFunction

{

	private MissionRepresentation mission;

	private List candidates;



	/**

	 * Constructor

	 * 

	 * @param mission the mission which this GA is attempting to find a solution for

	 * @param candidates UAVRepresentation objects in a List, indexed the same as Chromosome.squadBitmap

	 */

	public FitnessFunctionImpl(MissionRepresentation mission, List candidates)

	{

		this.mission = mission;

		this.candidates = candidates;

	}

	

	/**

	 * Computes fitness of provided chromosome.  The fitness function makes use of the cost functions

	 * in CostFunctions, which measure the difference between mission requirements and uav capability

	 * for each capability component.  The ideal coalition is one which "just barely" meets mission

	 * requirements, since this conserves the most capabilities for other missions (i.e., min(cost) ).

	 * However, ALL positive cost beats negative cost (any feasible coaltion beats a non-feasbile coalition).

	 * For non-feasible coaltions, the greater the cost (closert to 0), the more fit the coalition is.

	 * 

	 * 1/23/05 Hmm, try 50 points for each feasble attribute.  If component cost < 0, add component cost.  If

	 *   component cost > 0, subtract, up to a maximum of 40.  Add additional fitness for small formations. 

	 *   Sum all components.  Utter incantation and hope for the best.

	 */

  public double fittness(IChromosome aChromosome)

  {

	  ChromosomeImpl chromosome = (ChromosomeImpl) aChromosome;

	  boolean[] squadBitmap = chromosome.getSquadBitmap();

	  List coalition = BitmapUtils.formCoalitionList(squadBitmap, candidates);

		double fitness = 0.0;

		

		if (coalition.size() == 0)

			return 0.0;

		

		// firepower component

		if (mission.getFirepower() > 0.0)

		{	

			double firepowerCost = CostFunctions.firepowerCost(coalition, mission);

			if (firepowerCost >= 0.0)

			{

				fitness += 50.0;

				fitness -= Math.min(firepowerCost, 40.0);

			}

			else

				fitness += firepowerCost;

		}

		

		// bomb component

		if (mission.getNumBombs() > 0)

	  {

	  	int bombCost = CostFunctions.bombCost(coalition, mission);

	  	if (bombCost >= 0)

	  	{

	  		fitness += 50.0;

	  		fitness -= Math.min((double) bombCost, 40.0);	

	  	}

	  	else

	  	  fitness += bombCost;

	  }

			

		// visibility component

		if (mission.getVisibility() > 0.0)

		{

			double visibilityCost = CostFunctions.visibilityCost(coalition, mission);

			if (visibilityCost >= 0.0)

			{

				fitness += 50.0;

				fitness -= Math.min(visibilityCost, 40.0);

			}

			else

				fitness += visibilityCost;

		}

			

		// velocity component

		if (mission.getMaxVelocity() > 0.0)

		{

			double velocityCost = CostFunctions.velocityCost(coalition, mission);

			if (velocityCost >= 0.0)

			{

				fitness += 50.0;

				fitness -= Math.min(velocityCost, 40.0);

			}

			else

				fitness += velocityCost;

		}

			

		// stealth component

		if (mission.getStealth() > 0.0)

		{

			double stealthCost = CostFunctions.stealthCost(coalition, mission);

			if (stealthCost >= 0.0)

			{

				fitness += 50.0;

				fitness -= Math.min(stealthCost, 40.0);

			}

			else

				fitness += stealthCost;			

		}

		

		// additional fitness for small size

		// TBD: 3/6: SHOULDN'T THIS BE max(30,...) ??

	  double sizeFitness = Math.min(30, 5 * (chromosome.getSquadBitmap().length - coalition.size())); 

	  fitness += sizeFitness;

		//System.out.println("  GA: adding " + sizeFitness + " to fitness for size!");

		

		//System.out.print(BitmapUtils.toString(squadBitmap) + ":fitness=" + fitness + " ");

	  

		return -fitness;

  }

}

