package yaes.application.uavaltclient.coalitionformation.testharness;



import java.io.IOException;

import java.util.ArrayList;

import java.util.List;

//import java.util.Map;



import yaes.application.uavaltclient.coalitionformation.CoalitionFormation;

import yaes.application.uavaltclient.coalitionformation.DFIDCoalitionFormation;

import yaes.application.uavaltclient.coalitionformation.DepthFirstCoalitionFormation;

import yaes.application.uavaltclient.coalitionformation.GACoalitionFormation;

import yaes.application.uavaltclient.coalitionformation.MissionRepresentation;

import yaes.application.uavaltclient.coalitionformation.UAVRepresentation;

import yaes.application.uavaltclient.coalitionformation.timing.Format;

//import yaes.application.uavaltclient.coalitionformation.timing.Timing;

//import yaes.application.uavaltclient.coalitionformation.timing.TimingException;

//import yaes.application.uavaltclient.nav.Pos;

//import yaes.ui.plot.MatlabBarGraph;

//import yaes.ui.plot.SimulationPlot;





/**

 * This class is the test harness which is used to run coalition formation without the rest of 

 * the UAV simulation.

 * 

 * @author dejongp

 */

public class Main

{

	private static int NUM_RUNS = 10; // TBD: change.  Number of times to execution each test (plus one "warm-up" run)

	private static boolean DEBUG_ON = true;



	public static void main(String args[]) throws IOException

	{

		

		long start = System.currentTimeMillis();

		

		timeToFirstSolution();

		firstSolutionVsPoolSize();

		qualityNonGA();

    qualityGA();

		

		long end = System.currentTimeMillis();

		

		System.out.println("Total execution time: " + ((end - start) / 1000) + " sec.");

  }





  public static void timeToFirstSolution()

  {

		CoalitionFormation[] cfs = null;

		double[] fsTimes = new double[]{0.0, 0.0, 0.0, 0.0, 0.0};

		

		List uavList = UAVPool.getRandomPool(UAVPool.aircraft.size());

		

  	for (int missionNumber = 0; missionNumber < MissionPool.missions.size(); missionNumber++)

  	{

  		MissionRepresentation mission = (MissionRepresentation) MissionPool.missions.get(missionNumber);

  		

			cfs = new CoalitionFormation[]

			{

				new DepthFirstCoalitionFormation(false),

				new DepthFirstCoalitionFormation(true), 

				new DFIDCoalitionFormation(false),

				new DFIDCoalitionFormation(true),

				new GACoalitionFormation()

			};

	

	    for (int i=0; i<fsTimes.length; i++) fsTimes[i] = 0.0;

			

	    for (int cfNum=0; cfNum < cfs.length; cfNum++)

	    {

	      CoalitionFormation cf = cfs[cfNum];

	      

	      cf.addMissionRepresentation(mission);

	      cf.addUAVRepresentations(uavList);

				cf.breakAfterFirstSolution();

	      cf.formCoalitions();

	      

	      double fsTime = cf.getFirstSolutionTime();

	      

	      System.out.println("Algorithm " + cfs[cfNum].getClass().getName() + " : " + 

          " mission " + mission.getName() + 

          " TIME: " + Format.format(fsTime));

	     

				fsTimes[cfNum] += fsTime;

	    }

  	}

  	

  	// results:

		System.out.println("AVERAGES: ");

  	

  	// compute averages

  	for (int i=0; i<fsTimes.length; i++)

  	{

  		fsTimes[i] /= MissionPool.missions.size();

			System.out.println("Algorithm " + cfs.getClass().getName() + " : " + 

				" TIME: " + Format.format(fsTimes[i]));  		

  	}



  }



  public static void firstSolutionVsPoolSize()

  {

		CoalitionFormation[] cfs = null;

		

		double[][] firstSolutionTimes = new double[UAVPool.aircraft.size() + 1][];

		boolean repeatPoolsize = false;

		

		for (int poolsize = 10; poolsize <= UAVPool.aircraft.size(); poolsize++)

		{

			if (repeatPoolsize)

			{

				poolsize --;

				repeatPoolsize = false;

			}

			

			double[] fsTimes = new double[]{0.0, 0.0, 0.0, 0.0, 0.0};

			

			for (int i=0; i<NUM_RUNS; i++) //NEED TO SCRAMBLE POOL INSIDE EACH RUN

      {

				System.out.println();

				

				List uavPool = UAVPool.getRandomPool(poolsize);



				for (int j=0; j<uavPool.size(); j++)

					System.out.print( ((UAVRepresentation) uavPool.get(j)).getName() + ", "); 

				

		    System.out.println();

	

				cfs = new CoalitionFormation[]

				{

					new DepthFirstCoalitionFormation(false),

					new DepthFirstCoalitionFormation(true), 

					new DFIDCoalitionFormation(false),

					new DFIDCoalitionFormation(true),

					new GACoalitionFormation()

				};

				

				for (int cfNum=0; cfNum < cfs.length; cfNum++)

				{

					CoalitionFormation cf = cfs[cfNum];

					cf.addMissionRepresentation(MissionPool.missionRecon);

					cf.addUAVRepresentations(uavPool);

					cf.breakAfterFirstSolution();

					cf.formCoalitions();

		      

					double fsTime = cf.getFirstSolutionTime();

					if (fsTime == -1)

					{

						System.out.println("NO SOLUTION");

						repeatPoolsize = true;

					}

					else

		      {   

						System.out.println("Poolsize " + poolsize + 

	            " Run " + i + 

		          " Algorithm " + cfs[cfNum].getClass().getName() + " : " + 

							" mission " + MissionPool.missionRecon.getName() + 

							" TIME: " + Format.format(fsTime));

			     

						fsTimes[cfNum] += fsTime;

		      }

		      

					if (repeatPoolsize) break;

				}

				if (repeatPoolsize) break;

			}



      // divide out for avg

      for (int i=0; i<fsTimes.length; i++)

      {

      	fsTimes[i] /= NUM_RUNS;

      }

			

			firstSolutionTimes[poolsize] = fsTimes;

		}

	

	  System.out.println("\nRESULTS: ");

	  

	  // results are in firstSolutionTimes

	  for (int poolsize=10; poolsize<firstSolutionTimes.length; poolsize++)

	  {

	  	System.out.println("POOLSIZE: " + poolsize);

	  	double[] fsTimes = firstSolutionTimes[poolsize];

	  	for (int i=0; i<fsTimes.length; i++)

	  	{

	  		System.out.println("ALGO " + i + ": " + Format.format(fsTimes[i]));

	  	}

	  }

  }





//	/**

//	 * TEST TWO:

//	 * 

//	 * Growth of first solution, exeuction time as uav pool size increased.  Loss of efficiency in GA as uav pool size increases.

//	 *

//	 */

//	public static void testTwo() throws IOException

//	{

//		System.out.println("-- TEST TWO --");

//		

//		List uavPool;

//		CoalitionFormation[] cfs = null;

//		

//		double[][] firstSolutionTimes = new double[UAVPool.aircraft.size()][];

//		double[][] totalTimes = new double[UAVPool.aircraft.size()][];

//		

//		for (int i=0; i<UAVPool.aircraft.size(); i++)

//		{

//			firstSolutionTimes[i] = new double[]{0.0,0.0,0.0,0.0,0.0};

//			totalTimes[i] = new double[]{0.0,0.0,0.0,0.0,0.0};

//		}

//		

//		for (int j=1; j<=UAVPool.aircraft.size(); j++)

//		{

//			System.out.println(" UAV POOL: " + j);

//			uavPool = new ArrayList();

//								

//			for (int k=0; k<j; k++)

//			{

//				UAVRepresentation uav;

//				

//				do

//				{

//	        uav = UAVPool.getRandomUAV();

//				}

//				while(uavPool.contains(uav));

//				

//			  uavPool.add(uav);

//			}

//			

//			// uav pool formed.  Execute each algorithm.

//			cfs = new CoalitionFormation[]

//			{

//				new DepthFirstCoalitionFormation(false),

//				new DepthFirstCoalitionFormation(true), 

//				new DFIDCoalitionFormation(false),

//				new DFIDCoalitionFormation(true),

//				new GACoalitionFormation()

//			};

//			

//			for (int k=0; k<cfs.length; k++)

//			{

//				//cfs[k].addMissionRepresentations(MissionPool.missions);

//				cfs[k].addMissionRepresentation(MissionPool.missionAssault);

//				cfs[k].addUAVRepresentations(uavPool);					

//				cfs[k].formCoalitions();

//				

//

//				try

//				{

//			    firstSolutionTimes[j-1][k] += cfs[k].getAvgFirstSolutionTime();

//			    totalTimes[j-1][k] += cfs[k].getTotalTime().getElapsedTime();

//			

//				  System.out.println("#uav: " + j + ", coalitions=" + cfs[k].getCoalitions().size() + ", 1st t: " + Format.format(cfs[k].getAvgFirstSolutionTime()) +

//					  ", tot t: " + Format.format(cfs[k].getTotalTime().getElapsedTime()) + ", alth: " + cfs[k].getClass().toString());

//			 

//				}

//				catch (TimingException e)

//				{

//					System.out.println("#uav: " + j + ", 1st t: N/A, tot t: N/A (no candidates), alth: " + cfs[k].getClass().toString());

//				}  

//				

//			}

//		}

//	

//		

//		double[] yDF1st = new double[UAVPool.aircraft.size() - 9];

//		double[] yDFH1st = new double[UAVPool.aircraft.size() - 9];

//		double[] yDFID1st = new double[UAVPool.aircraft.size() - 9];

//		double[] yDFIDH1st = new double[UAVPool.aircraft.size() - 9];

//		double[] yGA1st = new double[UAVPool.aircraft.size() - 9];

//		

//		double[] yDFtot = new double[UAVPool.aircraft.size() - 9];

//		double[] yDFHtot = new double[UAVPool.aircraft.size() - 9];

//		double[] yDFIDtot = new double[UAVPool.aircraft.size() - 9];

//		double[] yDFIDHtot = new double[UAVPool.aircraft.size() - 9];

//		double[] yGAtot = new double[UAVPool.aircraft.size() - 9];		

//		

//		// divide all stats by num runs

//		for (int i=0; i<UAVPool.aircraft.size(); i++)

//		{

//			for (int j=0; j<5; j++)

//			{

//				firstSolutionTimes[i][j] /= (double) NUM_RUNS;

//				totalTimes[i][j] /= (double) NUM_RUNS;

//				

//				if (i>=9)

//				{

//					int numUAVs = i+1;

//					

//					switch (j)

//					{

//						case 0: yDF1st[i - 9] = firstSolutionTimes[i][j]; yDFtot[i - 9] = totalTimes[i][j]; break;

//						case 1: yDFH1st[i - 9] = firstSolutionTimes[i][j]; yDFHtot[i - 9] = totalTimes[i][j]; break;

//						case 2: yDFID1st[i - 9] = firstSolutionTimes[i][j]; yDFIDtot[i - 9] = totalTimes[i][j]; break;

//						case 3: yDFIDH1st[i - 9] = firstSolutionTimes[i][j]; yDFIDHtot[i - 9] = totalTimes[i][j]; break; 

//						case 4: yGA1st[i - 9] = firstSolutionTimes[i][j]; yGAtot[i - 9] = totalTimes[i][j];

//					}

//					

//				}

//				

//				System.out.println("#uav:" + i + " type:" + j + " 1st avg:" + Format.format(firstSolutionTimes[i][j]) + " tot avg:" + Format.format(totalTimes[i][j]));

//			}

//		}

//		

//		// plot runs starting with 10... 1-9 contain too many unsolvable missions due to small pool size

//		SimulationPlot firstSolutionPlot = new SimulationPlot();

//		double[] xPoints = new double[UAVPool.aircraft.size() - 9];

//    for (int i=0; i< UAVPool.aircraft.size() - 9; i++)

//    	xPoints[i] = i + 10;

//

//		firstSolutionPlot.setXPoints(xPoints);

//		firstSolutionPlot.addOnePlot("DF", yDF1st, "b-");

//		firstSolutionPlot.addOnePlot("DF+H", yDFH1st, "b-");

//		firstSolutionPlot.addOnePlot("DFID", yDFID1st, "b-");

//		firstSolutionPlot.addOnePlot("DFID+H", yDFIDH1st, "b-");

//		firstSolutionPlot.addOnePlot("GA", yGA1st, "b-");

//		

//		firstSolutionPlot.generateMatlabPlotFile("C:\\MatLab6p5\\work\\first_solution_versus_poolsize2.m");

//				

//	}

	

	

	/**

	 * TEST FOUR:

	 * 

	 * Quality of solution versus time.  Single mission, complete AC pool.  

	 * Non-GA algorithms, avg over NUM_RUNS runs, results to plot.

	 */

	public static void qualityNonGA() throws IOException

	{

		System.out.println("-- TEST FOUR --");

		

		ArrayList qualCostList = new ArrayList();

		ArrayList qualTimeList = new ArrayList();

		

		for (int i=0; i<4; i++)

		{

			qualCostList.add(new int[]{});

			qualTimeList.add(new double[]{});

		}

		

		CoalitionFormation[] cfs = null;

		

		//List uavList = UAVPool.getRandomPool(17);

		List uavList = UAVPool.getRandomPool(UAVPool.aircraft.size());

		  	

		

		cfs = new CoalitionFormation[]

		{

			new DepthFirstCoalitionFormation(false),

			new DepthFirstCoalitionFormation(true), 

			new DFIDCoalitionFormation(false),

			new DFIDCoalitionFormation(true)

		};



		for (int j=0; j<cfs.length; j++)

		{

			if (DEBUG_ON) System.out.println(" ALGO " + j);

			cfs[j].enableQualityTiming();

			cfs[j].addMissionRepresentation(MissionPool.missionBombrun);

      cfs[j].addUAVRepresentations(uavList);

			cfs[j].formCoalitions();

			

//				//TBD: tmp!

//				Map map = cfs[j].getMissionFirstSolutionTimes();

//				Timing t = (Timing) map.get(MissionPool.missionRecon.getName());

//				try

//				{

//				  System.out.println("FIRST SOLUTION TIME: " + Format.format(t.getElapsedTime()));

//				}

//				catch( Exception e) {}

//				//TBD: end tmp!

			

			//cfs[j].display();

			



			int[] qualityCostThisRun = cfs[j].getQualityCost();

			int[] qualityCost = new int[qualityCostThisRun.length];

			System.arraycopy(qualityCostThisRun, 0, qualityCost, 0, qualityCostThisRun.length);

			qualCostList.set(j, qualityCost);

			

			double[] qualityTimeThisRun = cfs[j].getQualityTime();

			double[] qualityTime = new double[qualityTimeThisRun.length];

			System.arraycopy(qualityTimeThisRun, 0, qualityTime, 0, qualityTimeThisRun.length);

			qualTimeList.set(j, qualityTime);

			

			if (qualityCost.length != qualityTime.length) 

				throw new Error("ERROR: results length mismatch");					





		}

		

		

		// RESULTS:



		// divide by num runs

		for (int i=0; i<cfs.length; i++)

		{

			int[] qualityCost = (int[]) qualCostList.get(i);

			double[] qualityTime = (double[]) qualTimeList.get(i);

			

			System.out.println(cfs[i].getClass().toString() + " : ");

			System.out.print("TIME" + i + " : [");

			for (int j=0; j<qualityTime.length; j++)

      {

				System.out.print(Format.format(qualityTime[j]) + " ");			

			}

			System.out.println("]");

			

			System.out.print("COST" + i + " : [");

			for (int j=0; j<qualityCost.length; j++)

			{		

				System.out.print(qualityCost[j] + " ");

			}

			System.out.println("]");

			

		}

		

	}

	

	/**

	 * TEST FIVE:

	 * 

	 * Quality of solution versus time.  Single mission, complete AC pool.  

	 * GA algorithms, five runs plus one warm-up.  All five displayed on same plot.

	 */

	public static void qualityGA() throws IOException

	{

		System.out.println("-- TEST FIVE --");

		

		CoalitionFormation cf;

		

		for (int i=-1; i<5; i++)

		{

		  cf = new GACoalitionFormation();

			cf.enableQualityTiming();

			

		  cf.addMissionRepresentation(MissionPool.missionBombrun);

			cf.addUAVRepresentations(UAVPool.aircraft);

			cf.formCoalitions();

		  

		  if (i>=0)

		  {

			  int[] qualityCost = cf.getQualityCost();

			  double[] qualityTime = cf.getQualityTime();

			  

				System.out.print("TIME" + i + " : [");

				for (int j=0; j<qualityTime.length; j++)

				{

					System.out.print(Format.format(qualityTime[j]) + " ");			

				}

				System.out.println("]");

			

				System.out.print("COST" + i + " : [");

				for (int j=0; j<qualityCost.length; j++)

				{

					System.out.print(qualityCost[j] + " ");

				}

				System.out.println("]");			  

		  }

		}

	}

	

}