package yaes.application.uavaltclient.coalitionformation.testharness;



import java.util.ArrayList;

import java.util.List;

import java.util.Random;



import yaes.application.uavaltclient.coalitionformation.MissionRepresentation;

import yaes.application.uavaltclient.nav.Pos;



/**

 * Mission pool for test harness.  An attempt is made to legitimize the assigned mission parameters

 * with a short "real-world" narritive.

 * 

 * @author dejongp

 */

public class MissionPool

{	

	// for now, everybody will start in the same location

	public static final Pos STARTING_POS = new Pos(50.0, 0.0, 50.0);

	

	private static Random generator = new Random();

	

	/* mission one:  advance toward a suspected bombing target.  Enemy fighters may

	 * be in the area.  

	 * 

	 * REQUIREMENTS:

	 *   Firepower:  200

	 *   Num bombs: 5

	 *   Visibility: 50

	 *   Velocity: 2.0 units/sec

	 *   Stealth: 15.0

	 */ 

	public static MissionRepresentation missionBombrun = new MissionRepresentation("mission-bombrun", 200.0, 5, 50.0, 2.0, 15.0, (Pos) STARTING_POS.clone());

	

	/* mission two:  perform reconaissance in a mountainous, uncharted area.  Attempt to remain unspotted.  Carry

	 * only enough firepower as is necessary for self defense if spotted.

	 * 

	 * REQUIREMENTS:

	 *   Firepower:  35

	 *   Num bombs: 0 (not required)

	 *   Visibility: 75

	 *   Velocity: 2.5 units/sec

	 *   Stealth: 75.0

	 */ 

	public static MissionRepresentation missionRecon = new MissionRepresentation("mission-recon", 35.0, 0, 75.0, 2.5, 75.0, (Pos) STARTING_POS.clone());	

	

	/* mission three:  deliver urgent supplies to a remote location.  Speed of delivery is the most important factor.

	 * 

	 * REQUIREMENTS:

	 *   Firepower:  10

	 *   Num bombs: 0 (not required)

	 *   Visibility: 0 (not required)

	 *   Velocity: 3.0 units/sec

	 *   Stealth: 0 (not required)

	 */ 

	public static MissionRepresentation missionDelivery = new MissionRepresentation("mission-delivery", 10.0, 0, 0.0, 3.0, 0.0, (Pos) STARTING_POS.clone());

	

	/* mission four: perform an all-out unmanned assault on a known target which is like to be heavily guarded.

	 * 

	 * REQUIREMENTS:

	 *   Firepower:  300

	 *   Num bombs: 7

	 *   Visibility: 25

	 *   Velocity: 1.0 units/sec

	 *   Stealth: 0 (not required)

	 */ 

	public static MissionRepresentation missionAssault = new MissionRepresentation("mission-assault", 400.0, 7, 25, 1.0, 0, (Pos) STARTING_POS.clone());	

	

	/* mission five: perform the impossible (a worst-case search scenario)

	 * 

	 * REQUIREMENTS:

	 *   Firepower:  500

	 *   Num bombs: 50

	 *   Visibility: 100

	 *   Velocity: 10 units/sec

	 *   Stealth: 100

	 */ 

	//public static MissionRepresentation missionImpossible = new MissionRepresentation("mission-impossible", 500.0, 50, 100.0, 10.0, 100.0, (Pos) STARTING_POS.clone());

	

	public static final List missions = new ArrayList();

	

	static

	{

		missions.add(missionBombrun);

		missions.add(missionRecon);

		missions.add(missionDelivery);

		missions.add(missionAssault);

		//missions.add(missionImpossible);

	}

	

	/**

	 * Randomly select one of the missions in the pool and return it.

	 * 

	 * @return randomly selected mission from pool

	 */

	public static MissionRepresentation getRandomMission()

	{

		int missionIndex = generator.nextInt(missions.size());

		return (MissionRepresentation) missions.get(missionIndex);			

	}

			

}

