package yaes.application.uavaltclient.coalitionformation.timing;



import java.math.BigInteger;



/**

 * Structure which holds timing information, including total time, time to first solution,

 * and combination count.  Can use either a platform-independant java timer uses 

 * System.currentTimeMillis(), or a much better resolution native win32 timer (obviously 

 * only available on Windows platforms).

 * 

 * @author dejongp

 */

public abstract class Timing

{

	// NOTE:

	// Using this native system call will greatly enhance timer resolution

	// (see http://www.javaworld.com/javaworld/javaqa/2003-01/01-qa-0110-timing.html)

	//

	// In order to use native timer, put hrtlib.dll somewhere visible (like c:\windows\system32)

	private static boolean useNativeWinTimerFlag = false;

	protected BigInteger combinationCount = BigInteger.ZERO;

	

	static

	{

		if (java.lang.System.getProperty("os.name").indexOf("Windows") >= 0)

		  useNativeWinTimerFlag = true;

	}

	

	/**

	 * 

	 * @return new instance of either JavaTiming or Win32NativeTiming

	 */

  public static Timing getInstance()

  {

  	if (useNativeWinTimerFlag)

  		return new Win32NativeTiming();

  	else

  		return new JavaTiming();

  }

	

	/**

	 * Returns time taken between startT and finishT, in seconds.

	 * 

	 * @return time taken between startT and finishT, in seconds 

	 * @throws TimingException is it is not possible to get the time

	 */

	public abstract double getElapsedTime() throws TimingException;



  /**

   * Start the timer.

   */

  public abstract void start();

  

  /**

   * Finish the timer.

   */

  public abstract void finish(); 

  

  /**

   * @return

   */

  public BigInteger getCombinationCount()

  {

    return combinationCount;

  }



  /**

   * @param integer

   */

  public void addToCombinationCount(BigInteger integer)

  {

		combinationCount = combinationCount.add(integer);

  }

  

	/**

	 * @param integer

	 */

	public void addToCombinationCount(long l)

	{

		combinationCount = combinationCount.add(BigInteger.valueOf(l));

	}  



}