package yaes.application.uavaltclient.nav;

/**
 * Structure which holds positional state information.
 * 
 * @author dejongp
 */
public class Pos implements Cloneable
{
  private double x; // (OpenGL map coordinates)
	private double y; // (OpenGL map coordinates)
	private double z; // (OpenGL map coordinates)
	private double hdg; // (OpenGL map coordinates)
	private double v; // (OpenGL map coordinates/tick)
	private double alt; // (OpenGL map coordinates AGL)
	// TBD: aoa?
	
	/**
	 * Constructor for fixed-space positions.
	 * 
	 * @param x
	 * @param y
	 * @param z
	 */
	public Pos(double x, double y, double z)
	{
		this.x = x;
		this.y = y;
		this.z = z;
	}
	
	/**
	 * Constuctor for uav dynamic positions.
	 * 
	 * @param x
	 * @param y
	 * @param z
	 */
	public Pos(double x, double y, double z, double hdg, double v, double alt)
	{
		this.x = x;
		this.y = y;
		this.z = z;
		this.hdg = hdg;
		this.v = v;
		this.alt = alt;
	}
	
	/**
	 * Return a clone of this Pos instance.  Overrides Object.clone().
	 * 
	 * @return Pos the cloned Pos object
	 */
	public Object clone()
	{
		try
		{
		  return super.clone();
		}
		catch (CloneNotSupportedException e)
		{
			throw new InternalError(e.toString());
		}
	}

  /**
   * @return
   */
  public double getAlt()
  {
    return alt;
  }

  /**
   * @return
   */
  public double getHdg()
  {
    return hdg;
  }

  /**
   * @return
   */
  public double getV()
  {
    return v;
  }

  /**
   * @return
   */
  public double getX()
  {
    return x;
  }

  /**
   * @return
   */
  public double getY()
  {
    return y;
  }

  /**
   * @return
   */
  public double getZ()
  {
    return z;
  }

  /**
   * @param d
   */
  public void setAlt(double d)
  {
    alt = d;
  }

  /**
   * @param d
   */
  public void setHdg(double d)
  {
    hdg = d;
  }

  /**
   * @param d
   */
  public void setV(double d)
  {
    v = d;
  }

  /**
   * @param d
   */
  public void setX(double d)
  {
    x = d;
  }

  /**
   * @param d
   */
  public void setY(double d)
  {
    y = d;
  }

  /**
   * @param d
   */
  public void setZ(double d)
  {
    z = d;
  }

	/**
	 * Return string form of this Position.
	 * 
	 * @return String form of this objective
	 */
  public String toString()
  {
  	return "x:" + x + " y:" + y + " z:" + z + " hdg:" + hdg + " v: " + v + " alt:" + alt;
  }
}
