package yaes.application.uavaltclient.ui;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;

import yaes.application.uavaltclient.agents.HQ;


/**
 * Primitive shell-based user interface for interacting with running client simulation.  Runs in
 * its own thread.
 * 
 * @author dejongp
 */
public class UIServer implements Runnable 
{
  private static final int PORT = 1202;
  
	private Socket clientSocket;
	private PrintWriter connectionOut = null;
  private HQ hq; //directory agent
 
  /**
   * Constructor
   * 
   * @param hq reference to the headquarters agent which spun off the UIServer in its own thread.
   */
  public UIServer(HQ hq)
  {
  	this.hq = hq;
  }
 
  /**
   * Wait for new user connections on PORT, in a loop.
   */
	public void run() 
	{
		try 
		{
			ServerSocket serverSocket = new ServerSocket(PORT);
			
      for (;;)
      {
				clientSocket = serverSocket.accept();
				InputStreamReader isr = new InputStreamReader(clientSocket.getInputStream() );
				BufferedReader connectionIn = new BufferedReader(isr);
				connectionOut = new PrintWriter(clientSocket.getOutputStream(), true);
				connectionOut.write("Connected to UI Server.\r\n\r\n");
							
        while (!serverSocket.isClosed())
        {
          String msg = connectionIn.readLine();
          
          if (msg != null)
          {
            if (msg.startsWith("quit"))
            {
							connectionOut.write("bye.\r\n");         	
            	connectionOut.close();
            	connectionOut = null;
            	connectionIn.close();
            	clientSocket.close();
              break;
            }
            else if (msg.startsWith("test"))
              write("test ok.");
            else if (msg.endsWith("xx"))
              write("command cancelled.");
            else
            {
							hq.command(msg);
            }           
          }
        }
      }
		}
		catch (Exception e) 
		{
			System.err.println(e + " : " + e.getMessage() );
		}        
	}
	
	/**
	 * Write information to user terminal.
	 * 
	 * @param response contains text which will be written to user terminal
	 */
	public synchronized void write(String response)
	{
		try
		{
		  connectionOut.write(response + "\r\n");         	
		  connectionOut.flush();
		}
		catch (Exception e)
		{
			System.err.println("Could not respond to client: " + e + ":" + e.getMessage() );		
		}
	}
	
	/**
	 * For testing.
	 * 
	 * @param args
	 */
	public static void main(String args[])
	{
		UIServer ui = new UIServer(null);
		Thread t = new Thread(ui);
		t.start();
	}
	
}
