package yaes.application.uavaltclient.util;

import java.util.Properties;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * Obtain system properties from config file.
 * 
 * @author dejongp
 */
public class Props
{
	private static final String PROPERTIES_FILE = "config.properties";
	private static Properties props = null;
	
	/**
	 * Load properties from properties file 
	 */
	public static void initProperties()
	{
		File conf = new File(PROPERTIES_FILE);
		try
		{
			props = new Properties();
			props.load(new FileInputStream(conf) );
		}
		catch (IOException e)
		{
			System.err.println("Error opening Configuration File " + PROPERTIES_FILE);
		}		
	}
	
	/**
	 * Return specified properties value as an int.
	 * 
	 * @param key properties to obtain value for
	 * @return value as int, or -1 if error
	 */
	public static int getInt(String key)
	{
		if (props == null)
			initProperties();
					
		try
		{
			return Integer.parseInt((String) props.get(key));
		}
		catch (Exception e)
		{
			System.err.println("key not found: " + key);
			return -1;
		}
	}
	
	/**
	 * Return specified properties value as a String
	 * 
	 * @param key properties to obtain value for
	 * @return value as String, or null if error
	 */
	public static String getString(String key)
	{
		if (props == null)
			initProperties();
					
		try
		{
			return (String) props.get(key);
		}
		catch (Exception e)
		{
			System.err.println("key not found: " + key);
			return null;
		}
	}	
	
	/**
	 * Return specified properties value as a double
	 * 
	 * @param key properties to obtain value for
	 * @return value as double, or -1.0 if error
	 */
	public static double getDouble(String key)
	{
		if (props == null)
			initProperties();
					
		try
		{
			return Double.parseDouble((String) props.get(key));
		}
		catch (Exception e)
		{
			System.err.println("key not found: " + key);
			return -1.0;
		}
	}		

	/**
	 * Return specified properties value as a boolean
	 * 
	 * @param key properties to obtain value for
	 * @return value as boolean (false if error)
	 */
	public static boolean getBoolean(String key)
	{
	  if (props == null)
	    initProperties();
	    
	  try
	  {
	    return ((String) props.get(key)).equalsIgnoreCase("true");
	  }
	  catch (Exception e)
	  {
	    System.err.println("key not found: " + key);
	    return false;
	  }
	}
	
}