/*
 * Created on Oct 12, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */

package yaes.framework.algorithm.geneticalgorithm;
import java.util.ArrayList;
import java.util.Set;

import java.util.Collections;

import java.util.List;


/**
 * @author Lotzi Boloni
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */

public class GeneticPool {  
	private List chromosomes;
	private ISelectionMethod selectionMethod;
	private IFittnessFunction fittnessFunction;
	private ICrossover crossover;
	private IMutation mutation; 
    private double crossoverRate = 0.5;
    private double mutationRate = 0.5;
    private boolean keepUnmutated = false;
    private boolean populationSize;

	private boolean useMultiCrossover = false;
	
	/**
	 * @param fittnessFunction
	 * @param crossover
	 * @param mutation
	 */

	public GeneticPool(IFittnessFunction fittnessFunction, ISelectionMethod selectionMethod, 
			ICrossover crossover, IMutation mutation) {
		super();
		this.fittnessFunction = fittnessFunction;
		this.crossover = crossover;
		this.mutation = mutation;
		this.selectionMethod = selectionMethod;
		chromosomes = new ArrayList();
	}

	
	public void compete() {

		int n = chromosomes.size();
		int mutations = (int)(n * mutationRate);
		int crossovers = (int)(n * crossoverRate);
		
		//Select and save Elitist
		List elitist = chromosomes.subList(0, 1); 
		
		//Select survivors
		ArrayList newGeneration = new ArrayList();
		int survivorCount = n - crossovers;
		
		for(int i=0; i<survivorCount;i++)
		{
			newGeneration.add(selectionMethod.SelectChromosomes(chromosomes));
		}
		
		//Apply Crossover Operator (Select by using selection method)
		int count = 0;
		while(count < crossovers) 
		{
			//Select a two parents
			IChromosome chrom1 = selectionMethod.SelectChromosomes(chromosomes);
			IChromosome chrom2 = selectionMethod.SelectChromosomes(chromosomes);
			
			//Select until one finds some which can mate...
			if (!crossover.canMate(chrom1, chrom2)) {
				continue;
			}
			
			//Perform multi crossover (more than one child is created)
			if (useMultiCrossover) {
				Set children = crossover.multipleCrossover(chrom1, chrom2);
				newGeneration.addAll(children);
				count += children.size();
			} 
			else//Perform single crossover (one child is generated) 
			{
				IChromosome newChrom = crossover.singleCrossover(chrom1, chrom2);
				newGeneration.add(newChrom);
				count++;
			}
		}
		
		//Apply Mutation Operator (Select with uniform probability)
		for(int i = 0; i != mutations; i++) {
			IChromosome chrom1 = getRandomChromosome(newGeneration);
			IChromosome newChrom = mutation.mutate(chrom1);

			if (!keepUnmutated) {
				newGeneration.remove(chrom1);
			}
			newGeneration.add(newChrom);			
		}
		
		//Update the population to the newly created chromosomes
		chromosomes = newGeneration;
		
		//Append the elitist chromosomes
		chromosomes.addAll(elitist);
		
		//Calculate fitness and sort the chromosomes by fitness (lowest fitness first in list)
		FittnessFunctionComparator comp = new FittnessFunctionComparator(fittnessFunction);
		Collections.sort(chromosomes, comp);
		
		//Get rid of extra chromosomes created
		chromosomes = chromosomes.subList(0, n); 
	}


	/**
	 * @return Returns the crossoverRate.
	 */
	public double getCrossoverRate() {
		return crossoverRate;
	}
	/**
	 * @param crossoverRate The crossoverRate to set.
	 */
	public void setCrossoverRate(double crossoverRate) {
		this.crossoverRate = crossoverRate;
	}

	/**
	 * @return Returns the keepUnmutated.
	 */
	public boolean isKeepUnmutated() {
		return keepUnmutated;
	}
	/**
	 * @param keepUnmutated The keepUnmutated to set.
	 */
	public void setKeepUnmutated(boolean keepUnmutated) {
		this.keepUnmutated = keepUnmutated;
	}
	/**
	 * @return Returns the mutationRate.
	 */
	public double getMutationRate() {
		return mutationRate;
	}
	/**
	 * @param mutationRate The mutationRate to set.
	 */
	public void setMutationRate(double mutationRate) {
		this.mutationRate = mutationRate;
	}
	
	/**
	 * @return Returns the chromosomes.
	 */

	public List getChromosomes() {
		return chromosomes;
	}

	/**
	 * @return Returns the fittnessFunction.
	 */
	public IFittnessFunction getFittnessFunction() {
		return fittnessFunction;
	}
	/**
	 * @return Returns the crossover.
	 */
	public ICrossover getCrossover() {
		return crossover;
	}
	/**
	 * @param crossover The crossover to set.
	 */
	public void setCrossover(ICrossover crossover) {
		this.crossover = crossover;
	}
	/**
	 * @return Returns the mutation.
	 */
	public IMutation getMutation() {
		return mutation;
	}
	/**
	 * @param mutation The mutation to set.
	 */
	public void setMutation(IMutation mutation) {
		this.mutation = mutation;
	}
	/**
	 * @return Returns the selectionMethod.
	 */
	public ISelectionMethod getSelectionMethod() {
		return selectionMethod;
	}
	/**
	 * @return Returns the useMultiCrossover.
	 */
	public boolean isUseMultiCrossover() {
		return useMultiCrossover;
	}

	/**
	 * @param useMultiCrossover
	 *            The useMultiCrossover to set.
	 */
	public void setUseMultiCrossover(boolean useMultiCrossover) {
		this.useMultiCrossover = useMultiCrossover;
	}
	

	/**
	 * @return
	 */
	private IChromosome getRandomChromosome(List chromosomeGeneration) {

		int n = chromosomeGeneration.size();
		int selected = (int)(Math.random() * n);		
		return (IChromosome)chromosomeGeneration.get(selected);
	}
}
