package yaes.framework.algorithm.neuralnetwork.rnn;


public class RandomNeuralNetwork extends Abstract3LayeredNN {

	protected double positive_ihWeights[][];

	protected double negative_ihWeights[][];

	protected double positive_hoWeights[][];

	protected double negative_hoWeights[][];

	protected double hiddenLayerR[];

	protected double outputLayerR[];

	public RandomNeuralNetwork(int inputNeurons, int hiddenNeurons,
			int outputNeurons) {
		this.inputNeurons = inputNeurons;
		this.hiddenNeurons = hiddenNeurons;
		this.outputNeurons = outputNeurons;
		inputLayerActivation = new double[inputNeurons];
		hiddenLayerActivation = new double[hiddenNeurons];
		hiddenLayerR = new double[hiddenNeurons];
		outputLayerActivation = new double[outputNeurons];
		outputLayerR = new double[outputNeurons];
		positive_ihWeights = new double[inputNeurons][hiddenNeurons];
		negative_ihWeights = new double[inputNeurons][hiddenNeurons];
		positive_hoWeights = new double[hiddenNeurons][outputNeurons];
		negative_hoWeights = new double[hiddenNeurons][outputNeurons];
	}

	/**
	 * Re-calculates the R values. Normally needed after every weight update
	 */
	public void calculateR() {
		for (int i = 0; i < hiddenLayerR.length; i++) {
			hiddenLayerR[i] = 0;
			for (int j = 0; j < inputNeurons; j++) {
				hiddenLayerR[i] += positive_ihWeights[j][i]
						+ negative_ihWeights[j][i];
			}
		}
		for (int i = 0; i < outputLayerR.length; i++) {
			outputLayerR[i] = 0;
			for (int j = 0; j < hiddenNeurons; j++) {
				outputLayerR[i] += positive_hoWeights[j][i]
						+ negative_hoWeights[j][i];
			}
		}

	}

	// number of neurons in the input layer
	//public static final int input = 32;

	// number of neurons in the hidden layer
	//public static final int hidden = 36;

	// number of neurons in the output layer
	//public static final int output = 32;

	// learning coeficient
	public static final double ETA = 0.3;

	//FILE *fp;
	//FILE *msefp;

	/*
	 * float *qnew,*r,*excitation,*inhibition,*N,*D,*gammaplus,*gammaminus,*des;
	 * float **wplus,**wminus,**W,**Wdiff,**Winv,*windowvector,*lambda; float
	 * **A,**B,**C; float
	 * vectormatrixplus,vectormatrixminus,sum1,sum2,mse,mseold,a,max,meansquareerror;
	 * int i,j,u,v,p,temp,maxindex,pixel,pattern; int
	 * total,count,mistake,hor1,ver1,hor2,ver2; time_t t ;
	 * 
	 * float IN[4][32]; // intensity using i-technique float OUT[4][32]; //
	 * output float TEST[9][32]; // test
	 */

	/*
	 * (non-Javadoc)
	 * 
	 * @see yaes.framework.algorithm.neuralnetwork.rnn.IGenericNeuralNetwork#updateWeights(yaes.framework.algorithm.neuralnetwork.rnn.InputOutputPair)
	 */
	public void updateWeights(InputOutputPair pair) {
		/* updating the weights */
		int total = inputNeurons + hiddenNeurons + outputNeurons;
		double gammaplus[] = new double[total + 1];
		double gammaminus[] = new double[total + 1];
		double W[][] = new double[total + 1][total + 1];
		double Wdiff[][] = new double[total + 1][total + 1];
		double Winv[][] = new double[total + 1][total + 1];
		double A[][] = new double[total + 1][total + 1];
		double B[][] = new double[total + 1][total + 1];
		double C[][] = new double[total + 1][total + 1];
		double D[] = new double[total + 1];

		double qnew[] = new double[total+1];
		double des[] = new double[outputNeurons + 1];
		
		// prepare the desired sample
		Sample outputSample = pair.getOutput();
		for (int i = 0; i < outputSample.getSize(); i++) {
			des[1+ i] = outputSample.getSample(i);
		}
		// find the current output qnew
		calculateOutput(pair.getInput());
		for (int i = 0; i < inputLayerActivation.length; i++) {
			qnew[i+1] = inputLayerActivation[i];
		}
		for (int i = 0; i < hiddenLayerActivation.length; i++) {
			qnew[i+inputNeurons + 1] = hiddenLayerActivation[i];
		}
		for (int i = 0; i < outputLayerActivation.length; i++) {
			qnew[i+inputNeurons + hiddenNeurons + 1] = outputLayerActivation[i];
		}
		
		
		// initialize the wplus and wminus matrices
		double wplus[][] = new double[total + 1][total + 1];
		double wminus[][] = new double[total + 1][total + 1];
		for (int i = 0; i < inputNeurons; i++) {
			for (int j = 0; j < hiddenNeurons; j++) {
				wplus[i + 1][inputNeurons + j + 1] = positive_ihWeights[i][j];
				wminus[i + 1][inputNeurons + j + 1] = negative_ihWeights[i][j];
			}
		}
		for (int i = 0; i < hiddenNeurons; i++) {
			for (int j = 0; j < outputNeurons; j++) {
				wplus[i + inputNeurons + 1][j + inputNeurons + hiddenNeurons
						+ 1] = positive_hoWeights[i][j];
				wminus[i + inputNeurons + 1][j + inputNeurons + hiddenNeurons
						+ 1] = negative_hoWeights[i][j];
			}
		}
		// now, directly from the C stuff
		for (int j = 1; j <= total; j++)
			for (int i = 1; i <= total; i++) {
				W[j][i] = (wplus[j][i] - (wminus[j][i] * qnew[i])) / D[i];
				if (j == i)
					Wdiff[j][i] = Winv[j][i] = 1.0 - W[j][i];
				else
					Wdiff[j][i] = Winv[j][i] = -W[j][i];
			}

		for (int i = 1; i <= inputNeurons; i++)
			for (int j = 1; j <= hiddenNeurons; j++) {
				A[i][j] = Wdiff[i][j + inputNeurons];
			}

		for (int i = 1; i <= outputNeurons; i++)
			for (int j = 1; j <= hiddenNeurons; j++) {

				B[j][i] = Wdiff[j + inputNeurons][i + inputNeurons + hiddenNeurons];
			}

		matrix_by_matrix(A, B, C, inputNeurons, hiddenNeurons, outputNeurons);

		for (int i = 1; i <= inputNeurons + hiddenNeurons; i++)
			for (int j = inputNeurons + 1; j <= total; j++) {
				if (((i <= inputNeurons) && (j <= inputNeurons + hiddenNeurons))
						|| ((i >= inputNeurons + 1) && (j >= inputNeurons + hiddenNeurons + 1)))
					Winv[i][j] = -Wdiff[i][j];
				if ((i <= inputNeurons) && (j >= inputNeurons + hiddenNeurons + 1))
					Winv[i][j] = C[i][j - inputNeurons - hiddenNeurons];
			}

		/* updating the weights */

		for (int u = 1; u <= inputNeurons; u++) {
			for (int v = inputNeurons + 1; v <= inputNeurons + hiddenNeurons; v++) {
				for (int i = 1; i <= total; i++) {
					gammaplus[i] = gammaminus[i] = 0.0;
					if ((u != i) && (v == i)) {
						gammaminus[i] = -qnew[i] / D[i];
						gammaplus[i] = 1.0 / D[i];
					}
					if ((u == i) && (v != i))
						gammaplus[i] = gammaminus[i] = -1.0 / D[i];
					if ((u == i) && (v == i))
						gammaminus[i] = -(1.0 + qnew[i]) / D[i];
				}

				double sum1 = 0;
				double sum2 = 0;
				for (int i = inputNeurons + hiddenNeurons + 1; i <= total; i++) {
					double vectormatrixplus = gammaplus[u] * Winv[u][i] + gammaplus[v]
							* Winv[v][i];
					double vectormatrixminus = gammaminus[u] * Winv[u][i]
							+ gammaminus[v] * Winv[v][i];
					sum1 += vectormatrixplus
							* (qnew[i] - (des[i - inputNeurons - hiddenNeurons])) * qnew[u];
					sum2 += vectormatrixminus
							* (qnew[i] - (des[i - inputNeurons - hiddenNeurons])) * qnew[u];
				}

				wplus[u][v] += -ETA * sum1;
				wminus[u][v] += -ETA * sum2;

				if (u == v)
					wplus[u][v] = wminus[u][v] = 0.0;
				if (wplus[u][v] < 0)
					wplus[u][v] = 0.0;
				if (wminus[u][v] < 0)
					wminus[u][v] = 0.0;
			}
		}
		for (int u = inputNeurons + 1; u <= inputNeurons + hiddenNeurons; u++) {
			for (int v = inputNeurons + hiddenNeurons + 1; v <= total; v++) {
				for (int i = 1; i <= total; i++) {
					gammaplus[i] = gammaminus[i] = 0.0;
					if ((u != i) && (v == i)) {
						gammaminus[i] = -qnew[i] / D[i];
						gammaplus[i] = 1.0 / D[i];
					}
					if ((u == i) && (v != i))
						gammaplus[i] = gammaminus[i] = -1.0 / D[i];
					if ((u == i) && (v == i))
						gammaminus[i] = -(1.0 + qnew[i]) / D[i];
				}

				double sum1 = 0;
				double sum2 = 0;
				for (int i = inputNeurons + hiddenNeurons + 1; i <= total; i++) {
					double vectormatrixplus = gammaplus[u] * Winv[u][i] + gammaplus[v]
							* Winv[v][i];
					double vectormatrixminus = gammaminus[u] * Winv[u][i]
							+ gammaminus[v] * Winv[v][i];
					sum1 += vectormatrixplus
							* (qnew[i] - (des[i - inputNeurons - hiddenNeurons])) * qnew[u];
					sum2 += vectormatrixminus
							* (qnew[i] - (des[i - inputNeurons - hiddenNeurons])) * qnew[u];
				}

				wplus[u][v] += -ETA * sum1;
				wminus[u][v] += -ETA * sum2;

				if (u == v)
					wplus[u][v] = wminus[u][v] = 0.0;
				if (wplus[u][v] < 0)
					wplus[u][v] = 0.0;
				if (wminus[u][v] < 0)
					wminus[u][v] = 0.0;
			}
		}

	}

	/**
     * @param a
     * @param b
     * @param c
     * @param inputNeurons
     * @param hiddenNeurons
     * @param outputNeurons
     */
    private void matrix_by_matrix(double[][] a, double[][] b, double[][] c, int inputNeurons, int hiddenNeurons, int outputNeurons) {
      throw new Error("FIXME: implement me");
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see yaes.framework.algorithm.neuralnetwork.rnn.IGenericNeuralNetwork#calculateOutput(yaes.framework.algorithm.neuralnetwork.rnn.Sample)
	 */
	public Sample calculateOutput(Sample sample) {
		// initializing the input layer from the given input
		for (int i = 0; i < inputLayerActivation.length; i++) {
			inputLayerActivation[i] = sample.getSample(i);
			if (inputLayerActivation[i] > 1.0) {
				inputLayerActivation[i] = 1.0;
			}
			if (inputLayerActivation[i] < 0.0) {
				inputLayerActivation[i] = 0.0;
			}
		}

		for (int i = 0; i < hiddenLayerActivation.length; i++) {
			double N = 0;
			double D = 0;
			for (int j = 0; j < inputNeurons; j++) {
				N += inputLayerActivation[j] * positive_ihWeights[j][i];
				D += inputLayerActivation[j] * negative_ihWeights[j][i];
			}
			D += hiddenLayerR[i];
			hiddenLayerActivation[i] = N / D;
			if (hiddenLayerActivation[i] > 1.0) {
				hiddenLayerActivation[i] = 1.0;
			}
			if (hiddenLayerActivation[i] < 0.0) {
				hiddenLayerActivation[i] = 0.0;
			}
		}

		for (int i = 0; i < outputLayerActivation.length; i++) {
			double N = 0;
			double D = 0;
			for (int j = 0; j < hiddenNeurons; j++) {
				N += hiddenLayerActivation[j] * positive_hoWeights[j][i];
				D += hiddenLayerActivation[j] * negative_hoWeights[j][i];
			}
			D += outputLayerR[i];
			outputLayerActivation[i] = N / D;
			if (outputLayerActivation[i] > 1.0) {
				outputLayerActivation[i] = 1.0;
			}
			if (outputLayerActivation[i] < 0.0) {
				outputLayerActivation[i] = 0.0;
			}
		}
		// and now, read out a new sample as the output
		Sample output = new Sample(outputNeurons);
		for (int i = 0; i < outputLayerActivation.length; i++) {
			output.setSample(i, outputLayerActivation[i]);
		}
		return output;
	}
}