/*

 * Created on Sep 7, 2004

 *

 * To change the template for this generated file go to

 * Window - Preferences - Java - Code Generation - Code and Comments

 */

package yaes.framework.algorithm.routing.wca;



import java.util.ArrayList;

import java.util.Collections;

import java.util.Comparator;

import java.util.Iterator;



import yaes.framework.world.network.cluster.ClusterHead;

import yaes.framework.world.network.sensornetwork.SensorNode;

import yaes.framework.world.network.sensornetwork.SensorNetwork;





/**

 * @author Lotzi B?l?ni

 * 

 * To change the template for this generated type comment go to Window -

 * Preferences - Java - Code Generation - Code and Comments

 */

public class WeightedClusteringAlgorithm {



    private SensorNetwork n;

    private WCAParameters wcaparam;



    public WeightedClusteringAlgorithm(SensorNetwork n, WCAParameters wcaparam) {

        this.n = n;

        this.wcaparam = wcaparam;

    }



    ClusterHead whichClusterHeadYouBelongTo(SensorNode itNeighbor) {

        for (Iterator iter = n.clusterHeadSet.iterator(); iter.hasNext();) {

            ClusterHead temp = (ClusterHead) iter.next();

            SensorNode clusterHeadNode = temp.getTheNode();

            if (clusterHeadNode.failed) {

                continue;

            }

            //   ... computes distance between clusterHead and it's neighbor

            double distance = clusterHeadNode.distance(itNeighbor);

            // FIXME: in some settings we will need to simulate such that you

            // are in the

            // cluster if you are closer than the minimum between the two

            // ranges.

            if (distance <= clusterHeadNode.getTxRange()) { return temp; }

        }

        //   ... return NULL when neighbor node does not belong to any clusterHead

        //   ... from clusterHeadSet

        return null;

    }



    public void computeOriginalClusters(double currentTime, int maxDisplacement) {

        n.clusterHeadSet.clear();

        computeNeighbor();

        while (noMoreNodeLeft() != null) {

            findClusterHeadInOneTransmissionRange(wcaparam, n.nodeInfoList.size(), currentTime, maxDisplacement);

        }

    }



    /**

     *   This function computes the Delta_v and the D_v for all 

     *   the nodes.

     */

    void computeNodeWeigth() {

        for (Iterator iter = n.nodeInfoList.iterator(); iter.hasNext();) {

            SensorNode sourceNode = (SensorNode) iter.next();

            if (sourceNode.getMark()) {

                continue;

            }

            //   ... reset the nodeDegree to zero

            sourceNode.setDegree(0);

            //   ... reset the acctWeight to zero

            sourceNode.setAccumulateWeight(0);

            //    ... get the first neighbor of the sourceNode

            for (Iterator it2 = sourceNode.neighborList.iterator(); it2

                    .hasNext();) {

                SensorNode destNode = (SensorNode) it2.next();

                if (destNode.getMark() == false) {

                    double distance = sourceNode.distance(destNode);

                    //   ... acctWeight += distance

                    sourceNode.setAccumulateWeight(sourceNode

                            .getAccumulateWeight()

                            + distance);

                    //   ... increment node degree by 1

                    sourceNode.setDegree(sourceNode.getDegree() + 1);

                } else {

                    //   ... neighborToSource node is no longer belonging to

                    // SourceNode

                    //   ... remove this neighbor node

                    //   ... remove nodeToBeRemoved from neighbor list

                    it2.remove();

                }

            }

            //   ... compute node weight

            //   for (MAX_DEGREE=MAX_DEGREE_START; MAX_DEGREE=MAX_DEGREE_STOP;

            // MAX_DEGREE+=MAX_DEGREE_CHANGE)

            sourceNode.setNodeWeight(Math.abs(sourceNode.getDegree()

                    - wcaparam.getOptimalDegree()));

        } //   ... end while

    }



    /**

     * This function computes the original clusters and clusterheads.

     *  

     */

    public void findClusterHeadInOneTransmissionRange(WCAParameters wcaparam, int networkSize, double currentTime, int maxDisplacement) {

        ArrayList possibleClusterHead = new ArrayList();

        //   ... recompute nodeWeight and accumulate distance for

        //   ... each node in the list that has not been used yet

        computeNodeWeigth();

        for (Iterator iter = n.nodeInfoList.iterator(); iter.hasNext();) {

            SensorNode thisNode = (SensorNode) iter.next();

            //   ... thisNode may be a clusterHead

            //   ... insert it into clusterHead list

            if (thisNode.getMark() == false) {

                ClusterHead nodeToInsert = new ClusterHead(thisNode);

                possibleClusterHead.add(nodeToInsert);

            }

        }

        // find the best

        Comparator comp = new WCAComparator(wcaparam, networkSize, currentTime, maxDisplacement);

        ClusterHead clusterHeadNode = (ClusterHead) Collections

                .min(possibleClusterHead, comp);

        n.clusterHeadSet.add(clusterHeadNode);

        //   ... now we need to go back to the nodeInfoList to

        //   ... mark this clusterHead and all it's neighbor so

        //   ... that the other cannot use these nodes

        markNodeBelongToThisClusterHead(clusterHeadNode);

    }



    /**

     * Returns the next unmarked node, but in practice, it is used as a boolean

     * 

     * @return

     */

    public SensorNode noMoreNodeLeft() {

        for (Iterator iter = n.nodeInfoList.iterator(); iter.hasNext();) {

            SensorNode element = (SensorNode) iter.next();

            if (element.getMark() == false) { return element; }

        }

        return null;

    }



    /**

     * This function marks all the nodes as being part of this cluster. It also

     * updates the cluster content variable

     * 

     * @param ch

     *            is the node

     */

    private void markNodeBelongToThisClusterHead(ClusterHead ch) {

        //   ... set isClusterHead flag to TRUE

        //   ... it indicates that this node is clusterHead

        ch.getTheNode().setClusterHead(ch);

        //   ... mark this node so that the other would not use this

        ch.getTheNode().setMark(true);

        //   ... increment clusterId by one

        // FIXME: you probably don't need this

        // ch.getTheNode().setClusterHeadId(++clusterHeadId);

        ch.resetClusterContent();

        //   ... Mark all neighbor of this cluster head so that

        //   ... the other could not use them

        for (Iterator iter = ch.getTheNode().neighborList.iterator(); iter

                .hasNext();) {

            SensorNode neighbor = (SensorNode) iter.next();

            neighbor.setMark(true);

            ch.addNodeToCluster(neighbor);

        }

    }



    /**

     * Computes the neighbors of every node which is visible according to the

     * txRange of the network. FIXME: this should depend on a transmission range

     * which is a property of the node.

     *  

     */

    public void computeNeighbor() {

        for (Iterator iter = n.nodeInfoList.iterator(); iter.hasNext();) {

            SensorNode thisNode = (SensorNode) iter.next();

            //   ... for each node, we try to delete all it's neighbors

            thisNode.neighborList.clear();

            //   ... set mark to FALSE

            //   ... it means that this node is available for neighbor or

            // clusterHead

            thisNode.setMark(false);

            //   ... set isClusterHead to FALSE

            //   ... it means that this node is not a clusterHead anymore

            thisNode.setClusterHead(null);

        }



        for (Iterator iter = n.nodeInfoList.iterator(); iter.hasNext();) {

            SensorNode sourceNode = (SensorNode) iter.next();

            for (Iterator iter2 = n.nodeInfoList.iterator(); iter2.hasNext();) {

                SensorNode destNode = (SensorNode) iter2.next();

                if (sourceNode == destNode) {

                    continue;

                }

                double distance = sourceNode.distance(destNode);

                if (distance <= sourceNode.getTxRange()) {

                    sourceNode.neighborList.add(destNode);

                }

            }

        }

    }





    /**

     * This is a simplified version done by me...

     * @param am

     * @return

     */    

    public boolean doesClusterHeadSetChange(AlgorithmMetrics am) {

        am.noTimesNeighborChange = 0;

        for (Iterator iter = n.nodeInfoList.iterator(); iter.hasNext();) {

            SensorNode thisNode = (SensorNode) iter.next();

            ClusterHead ch = whichClusterHeadYouBelongTo(thisNode);

            if ((ch == null) || ch.getTheNode().failed) {

                return true;

            }

        }

        return false;

    }

    

    

    boolean areYouBelongToThisClusterHead(SensorNode clusterHeadNode,

            SensorNode neighbor) {

        // if the clusterhead failed, we do not belong any more.

        if (clusterHeadNode.failed ) {

            return false;

        }

        double distance;

        //   ... computes distance between a clusterHead and it's neighbor

        distance = clusterHeadNode.distance(neighbor);        

        //   ... return TRUE if neighbor still belongs to it's clusterHead

        //   ... otherwise return FALSE

        // FIXME: in some settings we will need to simulate such that you are in

        // the

        // cluster if you are closer than the minimum between the two ranges.

        return (distance <= clusterHeadNode.getTxRange());

    }



}