package yaes.framework.entity.vehicle.car;



import java.io.FileWriter;

import java.io.IOException;

import java.util.Iterator;

import yaes.application.convoydriving.AbstractSimulation;

import yaes.application.convoydriving.RandomGenerator;

import yaes.framework.entity.control.ControlException;

import yaes.framework.entity.control.speedcontrol.AbstractSpeedCoalitionControl;

import yaes.framework.entity.vehicle.Vehicle;

import yaes.framework.world.World;

import yaes.framework.world.map.Location;

import yaes.framework.world.map.Speed;



public class Car extends Vehicle {



    int stableCoalitionCounter;

    FileWriter writer;

    private double currentTime;

    private double lastTime;



    double[] currentYValues = new double[AbstractSimulation.numberOfSimulationSteps];

    double[] projectedYValues = new double[AbstractSimulation.numberOfSimulationSteps];

    double[] currentSpeedValues = new double[AbstractSimulation.numberOfSimulationSteps];

    int dataCount=0;

    double originalY;



    public Car(int id, Location location, Speed speed, Speed lowLimit,

            Speed highLimit) {

        super(id);

        this.location = location;

        this.lastTime = 0.0;

        this.speed = speed;

        this.lastSpeed = this.speed;

        this.speedHighLimit = highLimit;

        this.speedLowLimit = lowLimit;

        this.originalSpeed = speed;

        this.originalY = location.getY();



        try {

            writer = new FileWriter(vehicleId + ".txt");

        } catch (IOException iex) {

            iex.printStackTrace();

        }

        formCoalition();

    }



    public Car(int id) {

        super(id);

        this.vehicleId = id;

        this.location = new Location(10, RandomGenerator.getInstance().nextInt(500));

        this.speedLowLimit = new Speed(10,20 + RandomGenerator.getInstance().nextInt(20));

        this.speedHighLimit = new Speed(10,50 + RandomGenerator.getInstance().nextInt(20));

        this.speed = new Speed(10,speedLowLimit.getYSpeed()

                + RandomGenerator.getInstance().nextInt((int)(speedHighLimit.getYSpeed() - speedLowLimit.getYSpeed())));

        this.lastTime = 0.0;

        this.lastSpeed = this.speed;

        this.originalSpeed = speed;

        this.originalY = this.location.getY();



        try {

            writer = new FileWriter(vehicleId + ".txt");

        } catch (IOException iex) {

            iex.printStackTrace();

        }

        formCoalition();

    }



    // Update Discrete simulation

    public void update(World world) throws ControlException {

      // update the current position

      this.currentTime = world.getTime();

      updatePosition();

      if (this.lastSpeed == this.speed)

          stableCoalitionCounter++;

      else

          stableCoalitionCounter = 0;



      if (stableCoalitionCounter > 3) {

          offerToJoinCoalition(world);

          stableCoalitionCounter = 0;

      }

      influence(world);



      if (control != null) {

        control.control();

      }

/*

      //if (agent != null) {

        //agent.action(world);

      //}

 */

      lastTime = currentTime;



    }



    public void updatePosition() {

        double deltaTime = (currentTime - lastTime);

        double xPos = location.getX() + speed.getXSpeed() * deltaTime;

        double yPos = location.getY() + speed.getYSpeed() * deltaTime;

        setLocation(new Location(xPos, yPos));

        //System.out.println("New pos:" + location);

        double newXSpeed = speed.getXSpeed(); //+ acceleration.getXAcceleration() * deltaTime;

        double newYSpeed = speed.getYSpeed();// + acceleration.getYAcceleration() * deltaTime;

        speed = new Speed(newXSpeed, newYSpeed);

         // FIXME: see how to generate plot

        //this.currentYValues[dataCount] = location.getY();

        //this.projectedYValues[dataCount] = this.originalY + this.originalSpeed.getYSpeed() * (currentTime);

        //this.currentSpeedValues[dataCount] = this.speed.getYSpeed();

        //dataCount++;

    }



    public int getCoalitionId() {

        return this.coalitionId;

    }



    public double[] getCurrentYValues() {

      return this.currentYValues;

    }

    public double[] getProjectedYValues() {

      return this.projectedYValues;

    }

    public double[] getCurrentSpeedValues() {

      return this.currentSpeedValues;

    }

    // form a new coalition

    public void formCoalition() {

        this.coalitionId = RandomGenerator.getInstance().nextInt(500);

    }



    // influence vehicles in the vicinity

    public void influence(World world) {

        for (Iterator it = world.getVehicles().iterator(); it.hasNext();) {

            Car veh = (Car) it.next();

            if (veh.getId() != this.vehicleId

                    && (Math.abs(location.getY() - veh.location.getY()) < 5.0)) {

                ((AbstractSpeedCoalitionControl)veh.getControl()).inform(this.vehicleId,this.coalitionId,this.speed,this.location);

                   // .influenceDistance(this.vehicleId, this.coalitionId,

                        //this.speed, location.getX(), location.getY());

            }

        }

    }



    // offer vehicles in the vicinity to join coalition

    public void offerToJoinCoalition(World world) {

        for (Iterator it = world.getVehicles().iterator(); it.hasNext();) {

            Car veh = (Car) it.next();

            if (veh.getId() != this.vehicleId

                    && Math.abs(location.getY() - veh.getLocation().getY()) < 20.0) {

                ((AbstractSpeedCoalitionControl)veh.getControl()).offerToJoin(this.vehicleId, this.coalitionId,

                        this.speed, this.location);

            }

        }

    }

/*

    public void influenceSpeedOnly(int sourceNode, int coalitionId, int speed,

            double x, double y) {

        if (coalitionId != this.coalitionId) return;

        if (location.getY() - y < 0) {

          this.lastSpeed = this.speed;

          this.speed = speed;

        }

    }



    // Get influence from other vehicles based on avg speed

    public void influenceAvgSpeed(int sourceNode, int coalitionId, int speed,

            double x, double y) {

        if (coalitionId != this.coalitionId) return;

        int avgSpeed, distSpeed = 0, influenceSpeed;



        //System.out.println(nodeID+":"+"Source node:"+sourceNode +" informed

        // speed:"+speed);

        avgSpeed = (speed + this.speed) / 2;



//        if (avgSpeed <= speed + 4 && avgSpeed >= speed -4)

//          influenceSpeed = speed;

//        else

//          influenceSpeed = avgSpeed;

        influenceSpeed = avgSpeed;

        if (influenceSpeed > speedHighLimit)

            this.speed = speedHighLimit;

        else if (influenceSpeed < speedLowLimit)

            this.speed = speedLowLimit;

        else

            this.speed = influenceSpeed;



        this.lastSpeed = this.speed;

        System.out.println(coalitionId + ":" + vehicleId + ":" + " SourceNode:"

                + sourceNode + " Avg speed:" + avgSpeed + " DistSpeed:"

                + distSpeed + " Influence Speed:" + influenceSpeed

                + " adapted speed:" + this.speed);

    }



    // distance only based influence

    public void influenceDistance(int sourceNode, int coalitionId, int speed,

            double x, double y) {

        if (coalitionId != this.coalitionId) return;

        int avgSpeed = 0, distSpeed = 0, influenceSpeed;



        distSpeed = computeDistancebasedSpeed(speed, x, y);

        if (distSpeed > 0) {

            influenceSpeed = distSpeed;

        } else {

            influenceSpeed = this.speed;

        }



        if (influenceSpeed > speedHighLimit)

            this.speed = speedHighLimit;

        else if (influenceSpeed < speedLowLimit)

            this.speed = speedLowLimit;

        else

            this.speed = influenceSpeed;



        System.out.println(coalitionId + ":" + vehicleId + ":" + " SourceNode:"

                + sourceNode + " Avg speed:" + avgSpeed + " DistSpeed:"

                + distSpeed + " Influence Speed:" + influenceSpeed

                + " adapted speed:" + this.speed + " last speed:"

                + this.lastSpeed);

        this.lastSpeed = this.speed;

    }



    public int computeDistancebasedSpeed(int speed, double x, double y) {

        double distance;

        int force;

        int distSpeed = 0;

        distance = Math.abs(location.getY() - y);

        System.out.println("Distance:" + (location.getY() - y));

        // if this agent is behind the given agent

        int avgSpeed;

        int distFactor;

        //System.out.println(nodeID+":"+"Source node:"+sourceNode +" informed

        // speed:"+speed);

        avgSpeed = (speed + this.speed) / 2;

        distFactor = (int)(((-2000 / Math.pow(distance, 3)) + (2 / Math

                  .pow(distance, .8))) * 1);



              System.out.println("distance factor:"+distFactor);





      if (location.getY() - y < 0 && this.speed != speed)

         distSpeed = this.speed + distFactor;

      else if (location.getY() - y > 0 && this.speed != speed)

          distSpeed = this.speed - distFactor;

      else

        distSpeed = speed;



      return distSpeed;

    }



    // Got an offer to join coalition

    public void offerToJoinCoalition(int sourceNode, int coalitionId,

            int speed, double x, double y) {

        if (coalitionId == this.coalitionId) return;

        System.out.println(vehicleId + ": Offer received:" + coalitionId);

        //if ((location.getY() - y) < 0) {

        this.coalitionId = coalitionId;

        System.out.println(vehicleId + ": Offer accepted:" + this.coalitionId);

        //}

    }

*/

}