package yaes.framework.entity.vehicle.ugv;



import yaes.application.convoydriving.AbstractSimulation;

import yaes.framework.agent.IUGVControlAgent;

import yaes.framework.entity.control.ControlException;

import yaes.framework.entity.vehicle.Vehicle;

import yaes.framework.world.World;

import yaes.framework.world.map.Acceleration;

import yaes.framework.world.map.IMap;

import yaes.framework.world.map.Location;

import yaes.framework.world.map.Speed;

import yaes.framework.world.map.Path;



public class UGV  extends Vehicle {



    private IMap map;

    private IUGVControlAgent agent;



    private double maxSpeed = 1.0;

    private double maxAcceleration = 1.0;

    private double currentTime;

    private double lastTime;



    private Path currentPath;



    public UGV(int id, IMap map) {

        super(id);

        this.map = map;

        speed = new Speed(0,0);

        acceleration = new Acceleration(0,0);

        location = new Location(0,0);

    }



    // Update Discrete simulation

    public void update(World world) throws ControlException {

        // update the current position

        this.currentTime = world.getTime();

        updatePosition();

        if (control != null) {

           control.control();

        }

        if (agent != null) {

            agent.action();

        }

        lastTime = currentTime;

    }



    public void updatePosition() {

        double deltaTime = (currentTime - lastTime);

        double xPos = location.getX() + speed.getXSpeed() * deltaTime;

        double yPos = location.getY() + speed.getYSpeed() * deltaTime;

        setLocation(new Location(xPos, yPos));

        //System.out.println("New pos:" + location);

        double newXSpeed = speed.getXSpeed() + acceleration.getXAcceleration() * deltaTime;

        double newYSpeed = speed.getYSpeed() + acceleration.getYAcceleration() * deltaTime;

        speed = new Speed(newXSpeed, newYSpeed);

    }







    /**

     * @return Returns the maxAcceleration.

     */

    public double getMaxAcceleration() {

        return maxAcceleration;

    }

    /**

     * @param maxAcceleration The maxAcceleration to set.

     */

    public void setMaxAcceleration(double maxAcceleration) {

        this.maxAcceleration = maxAcceleration;

    }

    /**

     * @return Returns the maxSpeed.

     */

    public double getMaxSpeed() {

        return maxSpeed;

    }

    /**

     * @param maxSpeed The maxSpeed to set.

     */

    public void setMaxSpeed(double maxSpeed) {

        this.maxSpeed = maxSpeed;

    }

    /* (non-Javadoc)

     * @see simulation.vehicles.Vehicle#update(simulation.applications.AbstractSimulation)

     */

    public void update(AbstractSimulation simulation) {

        // TODO Auto-generated method stub



    }





    /**

     * @return Returns the agent.

     */

    public IUGVControlAgent getAgent() {

        return agent;

    }

    /**

     * @param agent The agent to set.

     */

    public void setAgent(IUGVControlAgent agent) {

        this.agent = agent;

    }

  public Path getCurrentPath() {

    return currentPath;

  }

  public void setCurrentPath(Path currentPath) {

    this.currentPath = currentPath;

  }

}