package yaes.ui.text;



import java.io.BufferedReader;

import java.io.File;

import java.io.IOException;

import java.io.InputStreamReader;

import java.io.PrintStream;

import java.util.ArrayList;

import java.util.Date;

import java.util.HashMap;

import java.util.Iterator;

import java.util.List;

import java.util.logging.Logger;



public class TextUi {



    private static final String EMPTY = "Empty";

    private static final String RESTART = "RESTART";

    private static final String DONE = "DONE";

    private static final String ADD_ALL = "ADD ALL";

    private static final Logger LOGGER = Logger.getLogger("kraken.textui");

    private static final String SEPARATOR = "------------------------------------------------------------------";

    public static final int CHOICE_YES = 1;

    public static final int CHOICE_NO = 2;

    public static final int CHOICE_ABORT = 3;

    static InputStreamReader isr = new InputStreamReader(System.in);

    static BufferedReader reader = new BufferedReader(isr);

    static PrintStream writer = System.out;

    static PrintStream errorWriter = System.err;



    private static void printLabeledSeparator(String label) {

        if (label.equals("-")) {

            println(SEPARATOR);

            return;

        }

        String realLabel = " " + label.substring(1) + " ";

        int pads = SEPARATOR.length() - realLabel.length();

        if (pads < 0) {

            println("---" + realLabel + "---");

            return;

        }

        int i = 0;

        for (i = 0; i < pads / 2; i++) {

            print("-");

        }

        print(realLabel);

        for (; i < pads; i++) {

            print("-");

        }

        println("");

    }



    public static String menu(List<String> menuitems, String defaultItem, String prompt) {

        String thePrompt = null;

        String value = null;

        HashMap map = new HashMap();

        int i = 0;

        if (prompt != null) {

            thePrompt = prompt;

        } else {

            thePrompt = "Enter your choice: ";

        }

        while (true) {

            i = 0;

            writer.println("\n" + SEPARATOR);

            for (Iterator it = menuitems.iterator(); it.hasNext();) {

                String item = (String) it.next();

                if (item.startsWith("-")) {

                    printLabeledSeparator(item);

                } else {

                    i++;

                    if (item.equals(defaultItem)) {

                        println("[" + i + "]\t" + item);

                    } else {

                        println(" " + i + " \t" + item);

                    }

                    map.put(new Integer(i), item);

                }

            }

            writer.println(SEPARATOR);

            try {

                print(thePrompt);

                value = reader.readLine();

                if (value.equals("")) {

                    if (defaultItem != null) {

                        return defaultItem;

                    } else {

                        continue;

                    }

                }

                Integer val = Integer.valueOf(value);

                String retvalue = (String) map.get(val);

                if (retvalue == null) {

                    writer.println("Invalid value, try again!");

                    continue;

                }

                return retvalue;

            } catch (IOException ioex) {

                return "ERROR";

            } catch (NumberFormatException nfex) {

                String selectedItem = null;

                int countmatch = 0;

                for (Iterator it = menuitems.iterator(); it.hasNext();) {

                    String menuitem = (String) it.next();

                    if (menuitem.toLowerCase().indexOf(value.toLowerCase()) != -1) {

                        selectedItem = menuitem;

                        countmatch++;

                    }

                }

                if (countmatch == 1) {

                    return selectedItem;

                }

                if (countmatch > 1) {

                    TextUi

                            .println(value

                                    + " matches more than one menu items!");

                    continue;

                }

                if (countmatch == 0) {

                    TextUi.println(value

                            + " does not match any of the menu items!");

                    continue;

                }

            }

        }

    }



    /**

     * @param string

     * @param oldValue

     */

    public static String inputString(String prompt) {

        return inputString(prompt, null);

    }



    /**

     * @param prompt

     * @param oldValue

     * @return

     */

    public static String inputString(String prompt, String oldValue) {

        prompt = processPrompt(prompt);

        if (oldValue != null) {

            println(prompt + oldValue);

            println("Press <ENTER> to keep");

        }

        writer.print(prompt);

        try {

            String value = reader.readLine();

            if (value.equals("") && (oldValue != null)) {

                value = oldValue;

            }

            if (value.startsWith("+")) {

                value = oldValue + value.substring(1);

            }

            return value;

        } catch (Exception ex) {

            // should never happen

            LOGGER.severe(ex.toString());

            return null;

        }

    }



    /**

     * @param string

     * @param oldValue

     */

    public static Integer inputInteger(String prompt) {

        return inputInteger(prompt, null);

    }



    public static Integer inputInteger(String prompt, Integer oldValue) {

        prompt = processPrompt(prompt);

        if (oldValue != null) {

            println(prompt + oldValue);

            println("<ENTER> to keep the old value");

        }

        while (true) {

            try {

                print(prompt);

                String value = reader.readLine();

                if (value.equals("") && (oldValue != null)) {

                    return oldValue;

                }

                return Integer.valueOf(value);

            } catch (NumberFormatException nfex) {

                writer.print("Invalid value, enter an integer number");

            } catch (Exception ex) {

                // should never happen

                LOGGER.severe(ex.toString());

                return null;

            }

        }

    }



    /**

     * @param string

     * @param oldValue

     */

    public static Date inputDate(String prompt) {

        return inputDate(prompt, null);

    }



    public static Date inputDate(String prompt, Date oldValue) {

        prompt = processPrompt(prompt);

        if (oldValue != null) {

            println(prompt + oldValue);

            println("<ENTER> to keep the old value");

        }

        while (true) {

            try {

                writer.print(prompt);

                String propertyValue = reader.readLine();

                if (propertyValue.equals("") && (oldValue != null)) {

                    return oldValue;

                }

                //return DataTypeParserHelper.parseStringToDate(propertyValue);

            } catch (Exception ex) {

                // should never happen

                LOGGER.severe(ex.toString());

                return null;

            }

        }

    }



    /**

     * @param string

     * @param oldValue

     */

    public static List inputList(String prompt) {

        return inputList(prompt, null);

    }



    private static void printListWithPrompt(String prompt, List value) {

        println(prompt);

        int i = 0;

        for (Iterator iter = value.iterator(); iter.hasNext();) {

            String element = (String) iter.next();

            println("" + i + "\t" + element);

            i++;

        }

        println("<ENTER> to keep the old value, \n+newvalue to add new value, \n-number to remove one of the elements\nor comma separated list for direct entry");

    }



    public static List inputList(String prompt, List oldValue) {

        prompt = processPrompt(prompt);

        if (oldValue == null) {

            oldValue = new ArrayList();

        }

        while (true) {

            try {

                printListWithPrompt(prompt, oldValue);

                String propertyValue = reader.readLine();

                if (propertyValue.equals("") && (oldValue != null)) {

                    return oldValue;

                }

                if (propertyValue.startsWith("+")) {

                    LOGGER.warning("Adding new value");

                    //List newList = DataTypeParserHelper

                    //        .parseStringToList(propertyValue.substring(1));

                    oldValue.add(propertyValue.substring(1));

                    continue;

                }

                if (propertyValue.startsWith("-")) {

                    String number = propertyValue.substring(1);

                    int num = Integer.parseInt(number);

                    LOGGER.warning("Removing field " + num);

                    oldValue.remove(num);

                    continue;

                }

                // parse a new one

                //oldValue = DataTypeParserHelper.parseStringToList(propertyValue);

            } catch (Exception ex) {

                // should never happen

                LOGGER.severe(ex.toString());

                return null;

            }

        }

    }



    /**

     * Customized for source files - files from which you want to read

     * 

     * @param prompt

     * @return

     */

    public static File inputExistingFile(String prompt) {

        return inputFile(prompt, true);

    }



    /**

     * Allows to input a file name, doing various checks

     * 

     * @param prompt

     * @param checkForExistence -

     *            if true, it checks for the existence of the file

     * @return

     */

    public static File inputFile(String prompt, boolean checkForExistence) {

        prompt = processPrompt(prompt);

        while (true) {

            try {

                writer.print(prompt);

                String propertyValue = reader.readLine();

                if (propertyValue.equals("")) {

                    return null;

                }

                File theFile = new File(propertyValue);

                if (!checkForExistence || theFile.exists()) {

                    return theFile;

                }

                println("Unexistent file, check the spelling!");

            } catch (IOException ex) {

                // should never happen

                LOGGER.severe(ex.toString());

                return null;

            }

        }

    }



    /**

     * Inputs a directory through iterative navigation

     * 

     * @param prompt

     * @return

     */

    public static File inputDirectory(String prompt) {

        File currentDir = new File(".");

        currentDir = currentDir.getAbsoluteFile();

        while (true) {

            List menuItems = new ArrayList();

            File dirs[] = currentDir.listFiles();

            for (int i = 0; i != dirs.length; i++) {

                if (dirs[i].isDirectory()) {

                    menuItems.add(dirs[i].getName());

                }

            }

            menuItems.add("-");

            menuItems.add("UP");

            menuItems.add("HOME");

            menuItems.add("CANCEL");

            menuItems.add("DIRECT ENTRY");

            menuItems.add("ACCEPT");

            String value = menu(menuItems, null, prompt + "\nCurrent dir is:"

                    + currentDir + "\nNavigate directory: ");

            if (value.equals("UP")) {

                currentDir = currentDir.getParentFile();

                continue;

            }

            if (value.equals("HOME")) {

                currentDir = new File(java.lang.System

                        .getProperty("user.home"));

                continue;

            }

            if (value.equals("ACCEPT")) {

                return currentDir;

            }

            if (value.equals("CANCEL")) {

                return null;

            }

            if (value.equals("DIRECT ENTRY")) {

                try {

                    writer.print("Type in directory: ");

                    String propertyValue = reader.readLine();

                    File newDir = new File(propertyValue);

                    if (newDir.isDirectory()) {

                        currentDir = newDir;

                    }

                    TextUi.println("Not a directory: " + newDir);

                } catch (IOException ex) {

                    // should never happen

                    LOGGER.severe(ex.toString());

                    return null;

                }

            }

            currentDir = new File(currentDir, value);

        }

    }



    public static File inputFile2() {

        File currentDir = new File(".");

        currentDir = currentDir.getAbsoluteFile();

        while (true) {

            List menuItems = new ArrayList();

            File dirs[] = currentDir.listFiles();

            for (int i = 0; i != dirs.length; i++) {

                if (dirs[i].isDirectory()) {

                    menuItems.add(dirs[i].getName());

                }

            }

            menuItems.add("-");

            for (int i = 0; i != dirs.length; i++) {

                if (!dirs[i].isDirectory()) {

                    menuItems.add(dirs[i].getName());

                }

            }

            menuItems.add("-");

            menuItems.add("UP");

            menuItems.add("HOME");

            menuItems.add("CANCEL");

            String value = menu(menuItems, null, "Current dir is:" + currentDir

                    + "\nNavigate directory or select the file: ");

            if (value.equals("HOME")) {

                currentDir = new File(java.lang.System

                        .getProperty("user.home"));

                continue;

            }

            if (value.equals("UP")) {

                currentDir = currentDir.getParentFile();

                continue;

            }

            if (value.equals("CANCEL")) {

                return null;

            }

            currentDir = new File(currentDir, value);

            if (currentDir.isFile()) {

                return currentDir;

            }

        }

    }



    /**

     * @param parentDirectory

     * @param oldValues

     * @return

     */

    public static List inputListOfFileNames(File parentDirectory,

            List oldValues) {

        List newValues = new ArrayList(oldValues);

        List possibleValues = new ArrayList();

        if (!parentDirectory.isDirectory()) {

            throw new Error("Call this method with an existing directory!");

        }

        // fill the Vector with the (top level) files

        File content[] = parentDirectory.listFiles();

        for (int i = 0; i != content.length; i++) {

            if (content[i].isDirectory()) {

                continue;

            }

            possibleValues.add(content[i].getName());

        }

        // create a menu with the possible values

        while (true) {

            ArrayList menuItems = new ArrayList();

            menuItems.addAll(possibleValues);

            // remove the ones already in the collection

            for (Iterator it = newValues.iterator(); it.hasNext();) {

                if (menuItems.contains(it.next())) {

                    menuItems.remove(it.next());

                }

            }

            menuItems.add("-");

            menuItems.add(RESTART);

            menuItems.add(EMPTY);

            menuItems.add(DONE);

            String result = menu(menuItems, null, null);

            if (result.equals(RESTART)) {

                newValues = new ArrayList(oldValues);

            }

            if (result.equals(DONE)) {

                return newValues;

            }

            if (result.equals(EMPTY)) {

                newValues = new ArrayList();

            }

        }

    }



    private static List prepareChoices(List currentList, List choices) {

        List currentChoices = new ArrayList();

        for (Iterator it = choices.iterator(); it.hasNext();) {

            String item = (String) it.next();

            if (!currentList.contains(item)) {

                currentChoices.add(item);

            }

        }

        currentChoices.add("-");

        currentChoices.add(RESTART);

        currentChoices.add(ADD_ALL);

        currentChoices.add(DONE);

        return currentChoices;

    }



    public static List selectFromChoices(List oldList, List choices) {

        List currentList = new ArrayList(oldList);

        while (true) {

            List currentChoices = prepareChoices(currentList, choices);

            String prompt = "Current elements: " + currentList + "\nChoice: ";

            String choice = menu(currentChoices, DONE, prompt);

            if (choice == DONE) {

                return currentList;

            }

            if (choice == RESTART) {

                currentList = new ArrayList();

                continue;

            }

            if (choice == ADD_ALL) {

                currentList = new ArrayList(choices);

                continue;

            }

            currentList.add(choice);

        }

    }



    public static void println(String text) {

        writer.println(text);

    }



    public static void print(String text) {

        writer.print(text);

    }



    /**

     * @param string

     * @return

     */

    public static boolean inputBoolean(String prompt) {

        prompt = processPrompt(prompt);

        writer.print(prompt);

        try {

            String value = reader.readLine();

            if (value.equals("y") || value.equals("Y")) {

                return true;

            } else {

                return false;

            }

        } catch (Exception ex) {

            // should never happen

            LOGGER.severe(ex.toString());

            return false;

        }

    }



    public static int tripleChoice(String prompt, boolean defaultChoice) {

        prompt = processQuestionPrompt(prompt);

        while (true) {

            if (defaultChoice) {

                writer.print(prompt + " ([y]/n/a): ");

            } else {

                writer.print(prompt + " (y/[n]/a): ");

            }

            try {

                String value = reader.readLine();

                if (value.equals("")) {

                    if (defaultChoice) {

                        return CHOICE_YES;

                    } else {

                        return CHOICE_NO;

                    }

                }

                if (value.equals("y") || value.equals("Y")) {

                    return CHOICE_YES;

                }

                if (value.equals("n") || value.equals("N")) {

                    return CHOICE_NO;

                }

                if (value.equals("a") || value.equals("A")) {

                    return CHOICE_ABORT;

                }

                println("Incorrect value entered, use y, n or a!");

            } catch (Exception ex) {

                // should never happen

                LOGGER.severe(ex.toString());

                return CHOICE_NO;

            }

        }

    }



    /**

     * @param string

     * @return

     */

    public static boolean confirm(String prompt, boolean defaultChoice) {

        prompt = processQuestionPrompt(prompt);

        while (true) {

            if (defaultChoice) {

                writer.print(prompt + " ([y]/n): ");

            } else {

                writer.print(prompt + " (y/[n]): ");

            }

            try {

                String value = reader.readLine();

                if (value.equals("")) {

                    return defaultChoice;

                }

                if (value.equals("y") || value.equals("Y")) {

                    return true;

                }

                if (value.equals("n") || value.equals("N")) {

                    return false;

                }

                println("Incorrect value entered, use y or n!");

            } catch (Exception ex) {

                // should never happen

                LOGGER.severe(ex.toString());

                return false;

            }

        }

    }



    private static String processPrompt(String prompt) {

        if (prompt.endsWith("=") || prompt.endsWith(":")

                || prompt.endsWith("= ")) {

            return prompt;

        }

        return prompt + " = ";

    }



    private static String processQuestionPrompt(String prompt) {

        if (prompt.endsWith("?") || prompt.endsWith("? ")) {

            return prompt;

        }

        return prompt + "?";

    }



    public static void main(String args[]) {

        /*

         * Vector v = new Vector(); v.add("First"); v.add("Second"); String

         * retval = menu(v, "First", null); System.out.println("Retval" +

         * retval); retval = menu(v, "First", null); System.out.println("Retval" +

         * retval); retval = menu(v, "First", null); System.out.println("Retval" +

         * retval);

         */

        //println("" + inputDirectory());

        println("" + inputFile2());

    }



    /**

     * @param string

     */

    public static void errorPrint(String string) {

        errorWriter.println(string);

    }



    /**

     * @param string

     * @return

     */

    public static File inputDestinationFile(String prompt) {

        return inputFile(prompt, false);

    }



    /**

     * @param string

     * @return

     */

    public static String inputPassword(String string) {

        return inputString(string);

    }



    /**

     * 

     */

    public static void drawLine() {

        println("--------------------------------------------------------------");

    }

}